/**
 * グリッドの生成を行なうクラス
 */
define([
  'dijit/form/Button',
  'dijit/form/CheckBox',
  'dijit/form/ComboBox',
  'dijit/form/Select',
  'dijit/registry',
  'dojo/_base/array',
  'dojo/_base/declare',
  'dojo/_base/fx',
  'dojo/_base/lang',
  'dojo/aspect',
  'dojo/dom',
  'dojo/dom-class',
  'dojo/dom-construct',
  'dojo/dom-style',
  'dojo/json',
  'dojo/on',
  'dojo/query',
  'dojo/request',
  'dojo/string',
  'dojo/store/Memory',
  'dojo/topic',
  'dojox/widget/Standby',
  'dgrid/Grid',
  'dgrid/Keyboard',
  'dgrid/OnDemandGrid',
  'dgrid/extensions/ColumnHider',
  'dgrid/extensions/ColumnReorder',
  'dgrid/extensions/ColumnResizer',
  'dgrid/extensions/Pagination',
  'dgrid/Selection',
  'dgrid/Selector',
  'dstore/Request',
  'dstore/Trackable',
  'wagby/BarGauge'
], function(
  Button,
  CheckBox,
  ComboBox,
  Select,
  registry,
  array,
  declare,
  fx,
  lang,
  aspect,
  dom,
  domClass,
  domConstruct,
  domStyle,
  JSON,
  on,
  query,
  request,
  string,
  Memory,
  topic,
  Standby,
  Grid,
  Keyboard,
  OnDemandGrid,
  ColumnHider,
  ColumnReorder,
  ColumnResizer,
  Pagination,
  Selection,
  Selector,
  Request,
  Trackable,
  BarGauge
) {
  return declare(Grid, {
    constructor: function(args) {
      this.modelId = args.modelId;                 // モデルID
      this.idProperty = args.idProperty;           // 主キー項目ID
      this.gridId = 'jfc' + args.modelId + 'Grid'; // グリッドのID
      this.defaultColumns = args.columns;          // グリッドカラム定義
      this.isUsePagination = args.isUsePagination; // ページネーションを利用するか
      this.rowsPerPage = args.rowsPerPage;         // 1ページに表示する行数
      this.pageSizeOptions = args.pageSizeOptions; // 表示件数切り替えオプション
      this.links = args.links;                     // 詳細ボタン等のリンク要素の設定
      this.height = args.height;                   // グリッドの高さ
      this.initSearch = args.initSearch;           // 検索画面遷移直後の検索処理
      this.foreignKeyProperties = args.foreignKeyProperties; // 外部キー設定
      this.isInTab = args.isInTab;                 // タブ内に配置されているか
      this.batchProcessing = args.batchProcessing; // 一括処理が設定されているか
      this.isGridCreated = false;                  // Grid 生成済みか
      this.isSearchListGrid = args.isSearchListGrid; // 検索画面のグリッドかどうか
      this.textSearchJfcHiddenItem = null;         // 全文検索フォームの隠し検索パラメータが配置される要素
      this.gridSettingsFilter = [                  // 保存するグリッドデータ
        'columns', 'id', 'hidden', 'label', 'value', 'textAlign',
        'verticalAlign', 'width', 'whiteSpace', 'sortable', 'style',
        'sort', 'descending', 'property', 'rowsPerPage', 'currentPage'
      ];
      this.CompositePrimaryKeySeparator = '$SEP$';
      this.setPrimaryKeys();
      this.getConditionForm = this.getConditionFormFunc();
      this.getForm = this.getFormFunc();
      this.isOverlaySearchEventSet = false;
      this.isNeedResetHeaderColumnStyle = false;
      this.restorePreviousPage = false;
      this.ID_VALUE_KEY = "jfcIDValue";
    },

    setPrimaryKeys: function() {
      this.primaryKeys = this.idProperty.split(this.CompositePrimaryKeySeparator);
    },

    init: function() {
      var gridContainer = dom.byId(this.gridId);
      if (!gridContainer) {
        return;
      }
      var that = this;
      try {
        // 一括処理が設定されている場合、columns に行選択の設定を追加する
        this.setRowSelectorToColumns();
        // グリッド設定をプリファレンスから取得する
        this.getGridSettingsFromPreference(function() {
          // グリッド共通設定を取得する
          that.getGridCommonSettings(function(gridSettings) {
            // 共通設定を配置するコンテナを生成する
            that.initGridCommonSettings();
            // 共通設定のGUIを生成する
            that.createGridCommonSetting(gridSettings);
          });
        });
      } catch(e) {
        console.error(e);
      }
      setTimeout(function() {
        // 全文検索の隠し検索パラメータが配置される div タグを取得する
        that.getTextSearchJfcHiddenItem();
      }, 1);
    },

    getGridSettingsFromPreference: function(callbackFunc) {
      var that = this;
      // 処理待ちローダーオブジェクトを生成
      this.createStandby();
      this.myGridSettings = {};
      var url = this.getGridSettingsBaseURL();
      request(url, {
        handleAs: "json",
        headers: {
          "Content-Type": "application/x-www-form-urlencoded; charset=utf-8"
        }
      }).then(function(gridSettings) {
        // 取得したグリッド設定を保持する
        that.myGridSettings = gridSettings;
        // グリッド設定取得後にグリッドを生成する
        // タブ内にグリッドが設定されている場合、タブ表示時のイベントを設定する
        if (that.isInTab) {
          that.setCreateGridEvent();
        } else {
          that.createGrid();
        }
        if (callbackFunc) {
          callbackFunc(gridSettings);
        }
      }, function(err) {
console.error('err: ', err);
        jfcBubble.error(res.ERROR_MSG_GET_GRID_SETTINGS);
      });
    },

    initGridCommonSettings: function() {
      var gridNavigationDom = query("div#jfc" + this.modelId + "Grid")[0];
      var id = this.getGridCommonSettingsId() + "Container";
      this.commonSettingsContainer = domConstruct.create("div", {
        id: id,
        style: "text-align: right",
      }, gridNavigationDom, "after");
    },

    getGridCommonSettings: function(callbackFunc) {
      var url = this.getGridCommonSettingsURL();
      var that = this;
      request(url, {
        handleAs: "json",
        headers: {
          "Content-Type": "application/x-www-form-urlencoded; charset=utf-8"
        }
      }).then(function(gridSettings) {
        // 共通設定の連想配列を生成する
        that.jfcGridCommonSettingItems = {};
        for (var i=0,l=gridSettings.commonsetting.items.length; i<l; i++) {
          var commonsettingItem = gridSettings.commonsetting.items[i];
          that.jfcGridCommonSettingItems[commonsettingItem.name] = commonsettingItem;
        }
        if (callbackFunc) {
          callbackFunc(gridSettings);
        }
      }, function(err) {
console.error('err: ', err);
        jfcBubble.error(res.ERROR_MSG_GET_GRID_SETTINGS);
      });
    },

    createGridCommonSetting: function(gridSettings) {
      if (!gridSettings || !gridSettings.commonsetting || !gridSettings.commonsetting.items) {
        return;
      }
      var commonSetting = dom.byId(this.getGridCommonSettingsId());
      if (commonSetting) {
        return;
      }
      // 共通設定の選択欄を生成する
      this.createJfcGridCommonSettings(gridSettings, this.commonSettingsContainer);
      // 共通設定保存ボタンを生成する
      this.createGridCommonSettingSaveButton(this.commonSettingsContainer);
      // 共通設定削除ボタンを生成する
      this.createGridCommonSettingDeleteButton(this.commonSettingsContainer);
    },

    getJfcGridCommonSettingsValue: function() {
      if (this.jfcGridCommonSettingsCombobox) {
        return this.jfcGridCommonSettingsCombobox.get("value");
      } else if (this.jfcGridCommonSettingsSelect) {
        return this.jfcGridCommonSettingsSelect.get("value");
      }
      return "";
    },

    refreshJfcGridCommonSettings: function() {
      var that = this;
      // グリッド共通設定を取得する
      this.getGridCommonSettings(function(gridSettings) {
        // 設定コンボボックスのストアオブジェクトを更新する
        var store = that.getGridCommonSettingComboboxStore();
        that.jfcGridCommonSettingsCombobox.set("store", store);
        // 設定コンボボックスを初期値にセットする
        that.setJfcGridCommonSettingsComboboxDefalut();
      });
    },

    createGridCommonSettingSaveButton: function(commonSettingsContainer) {
      domConstruct.create("button", {
        id: this.getGridCommonSettingsSaveBtnId(),
        type: "button",
      }, commonSettingsContainer);
      var that = this;
      this.btnSaveGridCommonSettings = new Button({
        label: res.LABEL_BTN_SAVE_GRID_SETTING,
        style: "display: none",
        onClick: function() {
          var callbackFunc = function(){}
          if (that.GridSettingSendMethod === "POST") {
            var value = that.getJfcGridCommonSettingsValue();
            callbackFunc = function() {
              that.refreshJfcGridCommonSettings();
              that.showGridCommonSettingButton();
            }
          }
          that.sendGridCommonSettings(callbackFunc);
        },
      }, this.getGridCommonSettingsSaveBtnId());
    },

    createGridCommonSettingDeleteButton: function(commonSettingsContainer) {
      domConstruct.create("button", {
        id: this.getGridCommonSettingsDeleteBtnId(),
        type: "button",
      }, commonSettingsContainer);
      var that = this;
      this.btnDeleteGridCommonSettings = new Button({
        label: res.LABEL_BTN_DELETE_GRID_SETTING,
        style: "display: none",
        onClick: function() {
          that.changeGridSettingSendMethod("DELETE");
          var value = that.getJfcGridCommonSettingsValue();
          that.sendGridCommonSettings(function() {
            that.refreshJfcGridCommonSettings();
          });
        },
      }, this.getGridCommonSettingsDeleteBtnId());
    },

    showGridCommonSettingButton: function(style) {
      this.setGridCommonSettingButtonStyle("");
    },

    hideGridCommonSettingButton: function(style) {
      this.setGridCommonSettingButtonStyle("none");
    },

    setGridCommonSettingButtonStyle: function(display) {
      domStyle.set(this.btnSaveGridCommonSettings.domNode, "display", display);
      domStyle.set(this.btnDeleteGridCommonSettings.domNode, "display", display);
    },

    sendGridSettings: function(url, method, callbackFunc) {
      var options = {
        handleAs: "json",
        headers: {
          "Content-Type": "application/x-www-form-urlencoded; charset=utf-8"
        },
        method: method,
      };
      if (method !== "DELETE" && url.indexOf("/copy/") === -1) {
        var data = JSON.stringify(this.myGridSettings, this.gridSettingsFilter);
        if (!data || data === "null") {
          return;
        }
        options.data = data;
        options.headers["Content-Type"] = "application/json; charset=utf-8";
      }
      var that = this;
      request(url, options).then(function(data) {
        if (callbackFunc) {
          callbackFunc(data);
        }
      }, function(err) {
console.error('err: ', err);
        jfcBubble.error(res.ERROR_MSG_SET_GRID_SETTINGS);
      });

    },

    sendGridCommonSettings: function(callbackFunc) {
      if (this.GridSettingSendMethod === "") {
        return;
      }
      var method = this.GridSettingSendMethod;
      var url = this.getGridCommonSettingsURL() + "/" + this.getJfcGridCommonSettingsValue();
      if (method === "POST") {
        url = url + "/new";
      }
      this.changeGridSettingSendMethod("");
      var that = this;
      var successMsg = res.INFO_MSG_SAVE_GRID_SETTINGS_SUCCESS;
      if (method === "DELETE") {
        successMsg = res.INFO_MSG_DELETE_GRID_SETTINGS_SUCCESS;
      }
      this.sendGridSettings(url, method, function() {
        jfcBubble.info(successMsg);
        if (callbackFunc) {
          callbackFunc();
        }
      });
    },

    getGridCommonSettingsId: function() {
      return "jfcGridCommonSettings_" + this.modelId;
    },

    getGridCommonSettingsSaveBtnId: function() {
      return "jfcBtnSaveGridCommonSettings_" + this.modelId;
    },

    getGridCommonSettingsDeleteBtnId: function() {
      return "jfcBtnDeleteGridCommonSettings_" + this.modelId;
    },

    createJfcGridCommonSettings: function(gridSettings, commonSettingsContainer) {
      domConstruct.create("div", { id: this.getGridCommonSettingsId() }, commonSettingsContainer);
      if (gridSettings.commonsetting.editable) {
        // 共通設定コンボボックスを生成する
        this.createGridCommonSettingCombobox(gridSettings, commonSettingsContainer);
      } else {
        // 共通設定リストボックスを生成する
        this.createGridCommonSettingSelect(gridSettings, commonSettingsContainer);
      }
    },

    createGridCommonSettingCombobox: function(gridSettings, commonSettingsContainer) {
      var gridCommonSettingsComboboxOnChangeFunc = this.getGridCommonSettingsComboboxOnChangeFunc();
      var store = this.getGridCommonSettingComboboxStore();
      this.jfcGridCommonSettingsCombobox = new ComboBox({
        item: store.data[0],
        store: store,
        onChange: function(value) {
          gridCommonSettingsComboboxOnChangeFunc(value);
        }
      }, this.getGridCommonSettingsId());
    },

    getGridCommonSettingComboboxStore: function() {
      var store = new Memory({
        idProperty: "id",
        data: [
          {id: " ", name: ""},
        ]
      });
      for (var name in this.jfcGridCommonSettingItems) {
        store.add({
          id: name,
          name: name,
        });
      }
      store.add({
        id: "reset",
        name: res.LABEL_CLEAR_GRID_SETTING,
      });
      return store;
    },

    createGridCommonSettingSelect: function(gridSettings, commonSettingsContainer) {
      var gridCommonSettingsSelectOnChangeFunc = this.getGridCommonSettingsSelectOnChangeFunc();
      var options = [];
      options.push({
        label: "",
        value: " ",
      });
      for (var name in this.jfcGridCommonSettingItems) {
        options.push({
          label: name,
          value: name,
        });
      }
      options.push({
        label: res.LABEL_CLEAR_GRID_SETTING,
        value: "reset",
      });

      this.jfcGridCommonSettingsSelect = new Select({
        options: options,
        onChange: function(value) {
          gridCommonSettingsSelectOnChangeFunc(value);
        }
      }, this.getGridCommonSettingsId());
    },

    changeGridSettingSendMethod: function(method) {
      this.GridSettingSendMethod = method;
      if (method === "POST") {
        // 新規登録時は削除ボタンを非表示にする
        domStyle.set(this.btnDeleteGridCommonSettings.domNode, "display", "none");
      }
    },

    getGridCommonSettingsComboboxOnChangeFunc: function() {
      var that = this;
      return function(value) {
        if (!value || string.trim(value) === "") {
          that.hideGridCommonSettingButton();
          that.changeGridSettingSendMethod("");
          return;
        }
        var store = that.jfcGridCommonSettingsCombobox.get("store");
        var item = store.query({name: value});
        if (item.total === 0) {
          // 新規登録モードにする
          that.showGridCommonSettingButton();
          that.changeGridSettingSendMethod("POST");
        } else {
          if (item[0].id === " ") {
            // グリッド共通設定を選択した場合
            that.hideGridCommonSettingButton();
            that.changeGridSettingSendMethod("");
            return;
          }
          if (item[0].id === "reset") {
            // グリッド設定初期化
            that.resetGridSettings();
          } else {
            // グリッド共通設定を適用
            that.changeGridSettings(value);
          }
        }
      }
    },

    getGridCommonSettingsSelectOnChangeFunc: function() {
      var that = this;
      return function(value) {
        if (!value || string.trim(value) === "") {
          that.changeGridSettingSendMethod("");
          return;
        }
        if (value === "reset") {
          // グリッド設定初期化
          that.resetGridSettings();
        } else {
          // グリッド共通設定を適用
          that.changeGridSettings(value);
        }
      }
    },

    resetGridSettings: function() {
      var url = this.getGridSettingsBaseURL();
      var that = this;
      this.sendGridSettings(url, "DELETE", function() {
        that.resetMyGridSettings();
      });
    },

    resetMyGridSettings: function() {
      this.clearMyGridSettings();
      this.myGridSettings.columns = [];
      var columns = this.getMyGridColumnSettings();
      for (var key in columns) {
        var columnSettings = columns[key];
        delete columnSettings["hidden"];
        delete columnSettings["width"];
      }
      this.grid.set("columns", columns);
      this.setGridColumns();
      // 共通設定の選択肢を初期値にセットする
      this.setJfcGridCommonSettingsDefalut();
    },

    setJfcGridCommonSettingsDefalut: function() {
      if (this.jfcGridCommonSettingsCombobox) {
        this.setJfcGridCommonSettingsComboboxDefalut();
      } else if (this.jfcGridCommonSettingsSelect) {
        this.setJfcGridCommonSettingsSelectDefalut();
      }
    },

    setJfcGridCommonSettingsComboboxDefalut: function() {
      this.jfcGridCommonSettingsCombobox.set("item", this.jfcGridCommonSettingsCombobox.store.data[0]);
    },

    setJfcGridCommonSettingsSelectDefalut: function() {
      this.jfcGridCommonSettingsSelect.set("value", " ");
    },

    changeGridSettings: function(value) {
      var url = this.getGridSettingsBaseURL() + "/copy/" + value;
      var that = this;
      this.sendGridSettings(url, "POST", function(gridSettings) {
        if (that.jfcGridCommonSettingsCombobox) {
          // 更新モードにする
          that.changeGridSettingSendMethod("PUT");
          that.showGridCommonSettingButton();
        }
        that.updateGridColumnSettingsByCommonSettings(gridSettings);
      });
    },

    updateGridColumnSettingsByCommonSettings: function(gridSettings) {
      if (!gridSettings) {
        return;
      }
      // 設定値を初期化しておく
      var columns = this.getMyGridColumnSettings();
      for (var key in columns) {
        var columnSettings = columns[key];
        delete columnSettings["hidden"];
        delete columnSettings["width"];
      }
      this.myGridSettings = gridSettings;
      var columns = this.getMyGridColumnSettings();
      this.grid.set("columns", columns);
      this.setGridColumns();
    },

    getGridSettingsBaseURL: function() {
      var url = "rest/jfcpref/gridsetting/" + this.modelId;
      if (!this.foreignKeyProperties) {
        return url;
      }
      // 外部キー子モデルの場合、親モデルのモデルIDを付加する
      return url + "$" + this.foreignKeyProperties[0].idref;
    },

    getGridCommonSettingsURL: function() {
      return this.getGridSettingsBaseURL() + "/commonsetting";
    },

    createStandby: function() {
      var standbyDomNode = registry.byId(this.gridId + 'Standby');
      if (standbyDomNode) {
        this.standby = standbyDomNode;
        return;
      }
      this.standby = new Standby({
        id: this.gridId + 'Standby',
        target: this.gridId,
        style: 'width: 100%;'
      });
      document.body.appendChild(this.standby.domNode);
      this.standby.startup();
    },

    setRowSelectorToColumns: function() {
      if (!this.batchProcessing) {
        return;
      }
      var that = this;
      this.defaultColumns.unshift({
        id: 'selectorField',
        label: res.LABEL_SHOWLIST_CHOICE,
        resizable: false,
        selector: function(column, selected, cell, object) {
          domClass.add(cell, 'dgrid-selector');
          domStyle.set(cell, {
            textAlign: 'center',
            verticalAlign: 'middle'
          });
          var widget = new CheckBox({
            name: 'pkey_jsh_cb',
            checked: selected,
            tabIndex: isNaN(column.tabIndex) ? -1 : column.tabIndex,
          }).placeAt(cell);
          // 全選択チェックボックス列の場合、ヘッダクリックでチェックボックスの
          // チェック状態が変わるようにイベントを設定する
          on(cell, 'click', function(e) {
            if (e.type === 'click' &&
                (e.target.nodeName === 'TH' || e.target.nodeName === 'TD')) {
              widget.set('checked', !widget.get('checked'));
            }
          });
          widget.focusNode.value = object[that.ID_VALUE_KEY] || '';
          return widget.focusNode;
        },
        style: {
          label: {
            textAlign: 'center',
            width: '30px',
          }
        }
      });
    },

    setCreateGridEvent: function() {
      var gridDomNode = dom.byId(this.gridId);
      if (!gridDomNode) {
        throw 'modelId:['+this.modelId+'] grid node not exist';
      }
      var that = this;
      var contentPaneNode = gridDomNode.parentNode;
      if (contentPaneNode.tagName === "FORM") {
        contentPaneNode = contentPaneNode.parentNode;
      }
      var tab = registry.byId(contentPaneNode.id);
      if (!tab) {
        throw 'modelId:['+this.modelId+'] tab node not exist';
      }
      var tabContainerNode = contentPaneNode.parentNode.parentNode.parentNode;
      var tabContainer = registry.byId(tabContainerNode.id);
      if (!tabContainer) {
        throw 'modelId:['+this.modelId+'] tabContainer node not exist';
      }
      var tabOnShowFunc;
      // タブの表示状態を確認する
      if (domStyle.get(contentPaneNode, 'visibility') !== 'hidden') {
        // 初期表示されている場合は、createGrid を実行する
        this.createGrid();
        tabOnShowFunc = function() {
          tabContainer.resize();
        };
      } else {
        // 非表示の場合は、onShow イベント発生時に createGrid を実行する
        tabOnShowFunc = function() {
          that.createGrid();
          tabContainer.resize();
        };
      }
      tab.on('show', tabOnShowFunc);
    },

    getMyGridColumnSettings: function() {
      if (!this.myGridSettings || !this.myGridSettings.columns ||
          !(this.myGridSettings.columns instanceof Array) || this.myGridSettings.columns.length === 0) {
        // カラム設定が保存されていない場合、デフォルト設定を返す
        return this.defaultColumns;
      }
      var myGridColumnIds = {};
      var myGridColumns = [];
      var myGridColumn = this.myGridSettings.columns;
      for (var i=0,l=myGridColumn.length; i<l; i++) {
        var column = myGridColumn[i];
        var defaultColumn = this.getDefaultColumns(column.id);
        if (!defaultColumn) {
          continue;
        }
        myGridColumns.push(lang.mixin(defaultColumn, column));
        myGridColumnIds[column.id] = true;
      }
      if (myGridColumns.length === this.defaultColumns.length) {
        return myGridColumns;
      }
      // 保存されているカラム数と異なっている場合、追加する
      for (var i=0,l=this.defaultColumns.length; i<l; i++) {
        defaultColumn = this.defaultColumns[i];
        if (myGridColumnIds[defaultColumn['id']]) {
          continue;
        }
        var copiedColumn = lang.mixin({}, defaultColumn);
        if (defaultColumn['id'] === 'selectorField') {
          // 行選択列の場合は、行頭に追加
          myGridColumns.unshift(copiedColumn);
        } else {
          myGridColumns.push(copiedColumn);
        }
      }
      return myGridColumns;
    },

    getDefaultColumns: function(id) {
      for (var i=0, l=this.defaultColumns.length; i<l; i++) {
        if (this.defaultColumns[i]['id'] === id) {
          return this.defaultColumns[i];
        }
      }
      return null;
    },

    getMyGridSortSettings: function() {
      // グリッドのソート設定を取得する
      if (!this.myGridSettings || !this.myGridSettings.sort ||
          !(this.myGridSettings.sort instanceof Array) || this.myGridSettings.sort.length === 0) {
        return null;
      }
      return this.myGridSettings.sort;
    },

    clearMyGridSortSettings: function() {
      this.saveMySortSettings([]);
    },

    getMyGridRowsPerPageSettings: function() {
      // グリッドのページの表示数設定を取得する
      if (!this.myGridSettings || !this.myGridSettings.rowsPerPage) {
        return null;
      }
      return parseInt(this.myGridSettings.rowsPerPage, 10);
    },

    getMyGridCurrentPage: function() {
      // グリッドの表示ページ設定を取得する
      if (!this.myGridSettings || !this.myGridSettings.currentPage) {
        return null;
      }
      return this.myGridSettings.currentPage;
    },

    getGridModules: function() {
      var gridModules = [ColumnHider, ColumnReorder, ColumnResizer, Keyboard, Selection, Selector];
      if (this.isUsePagination) {
        gridModules.unshift(Pagination);
        gridModules.unshift(Grid);
      } else {
        gridModules.unshift(OnDemandGrid);
      }
      return gridModules;
    },

    createGrid: function() {
      if (this.isGridCreated) {
        return;
      }
      // 保存されているグリッドのカラム設定を復元する
      var myColumns = this.getMyGridColumnSettings();
      // 保存されているページの表示数設定を復元する
      var myRowsPerPageSettings = this.getMyGridRowsPerPageSettings();
      var rowsPerPage = this.rowsPerPage;
      if (myRowsPerPageSettings) {
        rowsPerPage = myRowsPerPageSettings;
      }
      var that = this;
      this.grid = new (declare(this.getGridModules()))({
        columns: myColumns,
        rowsPerPage: rowsPerPage,
        pageSizeOptions: that.pageSizeOptions,
        selectionMode: 'none',
        allowSelectAll: true,
      }, this.gridId);
      // ページネーション設定
      if (this.isUsePagination) {
        this.grid.set('pagingTextBox', true);
      }
      if (this.height === 'auto') {
        this.grid.set('className', 'dgrid-autoheight');
      } else {
        domStyle.set(this.grid.domNode, 'height', this.height);
      }
      // 一括処理が設定されている場合、行選択の関数をカスタマイズする
      this.setUpdateRowSelectors();
      // 保存されているソート設定を復元する
      var mySortSettings = this.getMyGridSortSettings();
      if (mySortSettings) {
        this.grid.set('sort', mySortSettings);
      }
      // 検索のRequestオブジェクトを生成する
      this.initCollection();
      // 検索のイベントを設定する
      if (!this.isSearchListGrid) {
        this.setSearchEvent();
        // ボタンの押下可・不可を切り替える関数
        this.buttonDisableFunc = this.getButtonDisableFunc();
      } else {
        this.buttonDisableFunc = function(){};
      }
      // 詳細ボタン等の表示設定を行なう
      this.setGridColumns();
      // 初期検索を行なう
      if (this.initSearch) {
        this.restorePreviousPage = true;
        this.search();
      } else {
        // 初期検索を行なわない場合、ボタンを押下不可とする
        this.buttonDisableFunc(0);
      }
      this.setGridEvent();
      this.isGridCreated = true;
    },

    setUpdateRowSelectors: function() {
      if (!this.batchProcessing) {
        return;
      }
      this.grid._updateRowSelectors = function (value, event) {
        var rows = event.rows;
        var lenRows = rows.length;
        var lenCols = this._selectorColumns.length;
        for (var iRows = 0; iRows < lenRows; iRows++) {
          for (var iCols = 0; iCols < lenCols; iCols++) {
            var column = this._selectorColumns[iCols];
            var element = this.cell(rows[iRows], column.id).element;
            if (!element) {
              continue;
            }
            // 行選択の dijit/form/CheckBox のノードを取得する
            element = element.childNodes[0].childNodes[0];
            if (!element || element.disabled) {
              continue;
            }
            if (element.id) {
              var widget = registry.byId(element.id);
              widget.set('checked', value);
            }
          }
        }
      };
    },

    getDownloadButtons: function() {
      var that = this;
      var downloadButtons = array.filter(this.widgets, function(widget) {
        return (widget.id === 'downloadButton1' || widget.id === 'downloadButton2');
      });
      return downloadButtons;
    },

    getActionButtons: function() {
      var actionButtons = [];
      query('table.action_table').forEach(function(table) {
        query('span[role="button"]', table).forEach(function(button) {
          var button = registry.byId(button.id);
          if (button) {
            actionButtons.push(button);
          }
        });
      });
      return actionButtons;
    },

    getNavigationButtons: function() {
      var navigationButtons = [];
      query('form[name="navigation"]').forEach(function(form) {
        query('span[role="button"]', form).forEach(function(button) {
          var id = button.id;
          if (id && id.lastIndexOf("btnNewInsert", 0) === -1 &&
                id.lastIndexOf("btnSearchUpdateList", 0) === -1) {
            var button = registry.byId(id);
            if (button) {
              navigationButtons.push(button);
            }
          }
        });
      });
      return navigationButtons;
    },

    getButtonDisableFunc: function() {
      var that = this;
      var buttons = [];
      [].push.apply(buttons, this.getDownloadButtons());
      [].push.apply(buttons, this.getActionButtons());
      [].push.apply(buttons, this.getNavigationButtons());
      if (buttons.length === 0 || this.foreignKeyProperties) {
        return function(){}
      }
      var buttonDisableFunc = function(total) {
        var disabled = !(total > 0);
        for (var i=0,l=buttons.length; i<l; i++) {
          buttons[i].set('disabled', disabled);
        }
      }
      return buttonDisableFunc;
    },

    getParseFunc: function() {
      var that = this;
      return function(response) {
        var responseData = JSON.parse(response);
        // 検索結果受信後の処理
        that.postProcessAfterGetResponse(responseData);
        return {
          items: responseData.items,
          total: responseData.total
        }
      }
    },

    postProcessAfterGetResponse: function(responseData) {
      // 検索結果から現在表示されているページを保存する
      this.saveCurrentPageSettings(responseData);
      // 検索結果によって、ボタンの押下可・不可を切り替える
      this.buttonDisableFunc(responseData.total);
      // 主キー値をセットする
      this.setPrimaryKeyData(responseData);
      var that = this;
      setTimeout(function() {
        // 全文検索が設定されている場合、全文検索のフォームに検索条件をセットする
        that.setConditionToTextSearchForm(responseData);
      }, 1);
    },

    setPrimaryKeyData: function(responseData) {
      for (var i=0,l=responseData.items.length; i<l; i++) {
        var item = responseData.items[i];
        item[this.ID_VALUE_KEY] = this.getPrimaryKeyValue(responseData.entity[i]);
      }
    },

    getPrimaryKeyValue: function(entity) {
      var value = this.getEntityValue(entity[this.primaryKeys[0]]);
      for (var i=1,l=this.primaryKeys.length; i<l; i++) {
        value = value + this.CompositePrimaryKeySeparator + this.getEntityValue(entity[this.primaryKeys[i]]);
      }
      return value;
    },

    getEntityValue: function(value) {
      if (!(value instanceof Array)) {
        return value;
      }
      for (var i=0,l=value.length; i<l; i++) {
        var item = value[i];
        if (item.choose) {
          return item.id;
        }
      }
      return "";
    },

    getCollectionIdProperty: function() {
      return this.ID_VALUE_KEY;
    },

    getCollectionTargetBase: function() {
      return 'rest/' + this.modelId;
    },

    getCollectionTarget: function() {
      return this.getCollectionTargetBase() + '/list';
    },

    initCollection: function() {
      // 一覧表示データ取得の Request オブジェクト作成
      this.collection = new (declare([Request, Trackable]))({
        idProperty: this.getCollectionIdProperty(),
        headers: { 'X-Wagby-RESTAPIVersion': 'v2' },
        target: this.getCollectionTarget(),
        rangeStartParam: 'firstresultjshparam', // 「検索結果の取得開始位置」のリクエストパラメータ名
        rangeCountParam: 'pagesizejshparam',    // 「検索結果から一度に取得する最大件数」のリクエストパラメータ名
        sortParam: 'sortkeyjshparam',           // ソートのリクエストパラメータ名
        parse: this.getParseFunc(),             // レスポンスデータから現在表示しているページを取得するための関数
      });
      // ソートのリクエストパラメータ値を取得する関数を Wagby に合わせてカスタマイズ
      this.setCustomizeRenderSortParamsFunc(this.collection);
    },

    setCustomizeRenderSortParamsFunc: function(collection) {
      collection._renderSortParams = function(sort) {
        var sortString = array.map(sort, function (sortOption) {
          var sortParam = encodeURIComponent(sortOption.property);
          if (sortOption.descending) {
            sortParam += ':DESC';
          }
          return sortParam;
        });
        var params = [];
        if (sortString) {
          params.push(encodeURIComponent(this.sortParam) + '=' + sortString);
        }
        return params;
      }
    },

    setSearchEvent: function() {
      // 検索フォーム内の widget をセットする
      this.setWidgetsInForm();
      var that = this;
      // 絞込等で画面再描画なしで入力欄に変更があった場合、入力欄を再取得する
      topic.subscribe("jfcUpdateContent", function() {
        that.refreshWidget();
      });
      // 検索ボタン押下イベントを設定する
      array.filter(this.widgets, function(widget) {
        return widget.type === 'button' &&
                 widget.id.indexOf('btnSearchCondition') !== -1;
      }).forEach(function(btnSearch) {
        btnSearch.on('click', function() {
          // info, warn, error のメッセージが表示されている場合、非表示とする
          that.hideScreenMessage();
          that.search();
        });
      });
      // リセットボタン押下イベントを設定する
      array.filter(this.widgets, function(widget) {
        return widget.type === 'button' &&
                 widget.id.indexOf('btnReset') !== -1;
      }).forEach(function(btnReset) {
        if (!that.isSearchListGrid) {
          btnReset.on('click', function() {
            that.clearMyGridSettings();
            that.clearMyGridSortSettings();
          });
        } else {
          btnReset.onClick = function() {
            that.clearMyGridSettings();
            that.clearMyGridSortSettings();
            that.grid.refresh();
          }
        }
      });
    },

    setWidgetsInForm: function() {
      var form = this.getForm();
      if (!form) {
        this.widgets = [];
        return;
      }
      this.widgets = registry.findWidgets(form);
      // ボタン以外の入力フォームをセットする
      this.widgetsExceptButtons = array.filter(this.widgets, function(widget) {
        return widget.type !== 'button';
      });
    },

    refreshWidget: function() {
      this.widgets = [];
      this.setWidgetsInForm();
    },

    setOverlaySearchEvent: function() {
      if (this.isOverlaySearchEventSet) {
        return;
      }
      this.setSearchEvent();
      this.isOverlaySearchEventSet = true;
    },

    hideScreenMessage: function() {
      query('div.infomsg,div.warnmsg,div.errormsg').forEach(function(node) {
        var fxHandler = fx.fadeOut({
          node: node,
          duration: 500,
        });
        fxHandler.on('End', function(node) {
          domStyle.set(node, 'display', 'none');
        })
        fxHandler.play();
      });
    },

    clearMyGridSettings: function() {
      if (!this.myGridSettings) {
        return;
      }
      // 現在ページの情報をクリアする
      this.myGridSettings.currentPage = 1;
      this.saveMyGridSettings();
    },

    search: function() {
      this.standby.show();
      var conditions = this.getConditions();
      var collection = this.collection.filter(conditions);
      this.grid.set('collection', collection);
    },

    gotoPage: function() {
      var currentPage = this.getMyGridCurrentPage();
      if (!currentPage || currentPage === 1) {
        return;
      }
      this.grid.gotoPage(currentPage);
    },

    setGridEvent: function() {
      if (!this.grid) {
        return;
      }
      var that = this;
      // ヘッダカラム入れ替え、ヘッダカラム非表示時のイベント
      this.grid.on('dgrid-columnreorder, dgrid-columnstatechange', function(event) {
        that.saveMyColumnSettings(event.grid.subRows);
        // ヘッダカラム入れ替え時はスタイルを再適用するためフラグをセットする
        if (event && event.type === 'dgrid-columnreorder') {
          that.isNeedResetHeaderColumnStyle = true;
        }
      });
      // ヘッダカラム幅変更時のイベント
      this.grid.on('dgrid-columnresize', function(event) {
        var column = that.getColumn(event.columnId);
        if (column) {
          column.width = event.width;
          var width = event.width + "px";
          column.style.label.width = width;
          domStyle.set(column.headerNode.childNodes[0].parentNode, "width", width);
          that.grid.styleColumn(column.id, 'width: ' + width);
          that.saveMyColumnSettings(event.grid.subRows);
        }
      });
      // ソート時のイベント
      this.grid.on('dgrid-sort', function(event) {
        that.saveMySortSettings(event.sort);
      });
      // ページの表示数変更時のイベント
      this.grid.on('.dgrid-page-size:change', function(event) {
        that.saveMyRowsPerPageSettings(event.target.value);
      });
      // エラー時のイベント
      this.grid.on('dgrid-error', function(event) {
        that.standby.hide();
        // セッションタイムアウト
        if (event.error.response.status === 401) {
          var redirectURL = that.getRedirectURL();
          location.href = redirectURL;
        }
      });
      // 行削除時にセル内のDojoのウィジェットを削除する処理を設定
      aspect.before(this.grid, 'removeRow', function(rowElement) {
        that.standby.show();
        that.destroyWidgetsInRow(rowElement);
      });
      // ヘッダカラム入れ替え後はヘッダ行表示後にスタイルを再設定する
      aspect.after(this.grid, 'renderHeader', function() {
        if (that.isNeedResetHeaderColumnStyle) {
          that.resetHeaderColumnStyle();
          that.isNeedResetHeaderColumnStyle = false;
        }
      });
      // グリッドの全行表示後に処理待ちローダを非表示にする
      aspect.after(this.grid, 'renderArray', function(rows) {
        that.standby.hide();
        return rows;
      });

      // 検索画面のグリッドの場合、値行のクリックイベントを設定する
      if (this.isSearchListGrid) {
        this.grid.on('.dgrid-content .dgrid-row:click', function(e) {
          var row = that.grid.row(e);
          if (that.setContentValue) {
            that.setContentValue(row);
          } else {
            console.log('setContentValue not defined');
          }
        });
      }
    },

    setContentValueFunc: function(func) {
      this.setContentValue = func;
    },

    destroyWidgetsInRow: function(rowElement) {
      var columns = this.grid.get('columns');
      for (var i=0,l=columns.length; i<l; i++) {
        var cloumnId = columns[i].id;
        var cellElement = this.grid.cell(rowElement, cloumnId).element;
        if (!cellElement || !cellElement.widgets) {
          continue;
        }
        var widgets = cellElement.widgets;
        for (var j=0,k=widgets.length; j<k; j++) {
          widgets[j].destroyRecursive();
        }
      }
    },

    getRedirectURL: function() {
      var redirectURL = this.getMainMenuURL() + '?name=showList' + capFirst(this.modelId);
      var conditions = this.getConditions();
      for (var key in conditions) {
        if (conditions[key] !== '') {
          redirectURL = redirectURL + '&' + key + '=' + conditions[key];
        }
      }
      return redirectURL;
    },

    getMainMenuURL: function() {
      var pathnames = window.location.pathname.split('/');
      var projectId = pathnames[1];
      return '/' + projectId + '/mainMenu.do';
    },

    saveMyGridSettings: function() {
      this.sendGridSettings(this.getGridSettingsBaseURL(), "PUT");
    },

    saveMyColumnSettings: function(subRows) {
      if (!subRows || !(subRows instanceof Array) || subRows.length === 0) {
        return;
      }
      this.myGridSettings.columns = subRows[0];
      this.saveMyGridSettings();
    },

    saveMySortSettings: function(sort) {
      this.myGridSettings.sort = sort;
      this.saveMyGridSettings();
    },

    saveMyRowsPerPageSettings: function(rowsPerPage) {
      this.myGridSettings.rowsPerPage = rowsPerPage;
      this.saveMyGridSettings();
    },

    saveCurrentPageSettings: function(responseData) {
      if (!this.myGridSettings || !responseData.result) {
        return;
      }
      var currentPage;
      try {
        currentPage = Math.ceil(responseData.result.current / responseData.result.pageSize);
      } catch(e) {
      }
      if (!currentPage) {
        currentPage = 1;
      }
      if (this.restorePreviousPage && this.isUsePagination) {
        this.restorePreviousPage = false;
        var maxPage = Math.ceil(responseData.total / responseData.result.pageSize);
        var previousPage = this.getMyGridCurrentPage();
        if (previousPage && previousPage !== 1 && maxPage >= previousPage) {
          // ページネーション利用時は保存されている表示ページ設定を復元する
          this.gotoPage();
          return;
        }
      }
      this.myGridSettings.currentPage = currentPage;
      this.saveMyGridSettings();
    },

    getColumn: function(id) {
      for (var i=0; i<this.grid.subRows.length; i++) {
        var columns = this.grid.subRows[i];
        for (var j=0; j<columns.length; j++) {
          var column = columns[j];
          if (column.id === id) {
            return column;
          }
        }
      }
      return null;
    },

    getActionCommand: function() {
      return {
        action_Search_Condition: ''
      };
    },

    getConditions: function() {
      var conditions = this.getActionCommand();
      this.setParameters(this.widgetsExceptButtons, conditions);
      if (this.foreignKeyProperties) {
        // 外部キー子モデルの一覧表示の場合、親モデルの主キー値を検索条件に加える
        for (var i=0,l=this.foreignKeyProperties.length; i<l; i++) {
          var foreignKey = this.foreignKeyProperties[i];
          for (var j=0,m=foreignKey.conditionParams.length; j<m; j++) {
            conditions[foreignKey.conditionParams[j]] =
              this.getParentId(foreignKey.idref, foreignKey.refname);
          }
        }
      }
      return conditions;
    },

    setParameters: function(widgets, parameters) {
      if (!widgets || widgets.length === 0) {
        return;
      }
      for (var i=0,l=widgets.length; i<l; i++) {
        var widget = widgets[i];
        if (!widget.id || (widget.id.lastIndexOf('jfcdelete_', 0) === -1 && widget.id.lastIndexOf(this.modelId, 0) === -1)) {
          continue;
        }
        var key = widget.id;
        if (widget.baseClass.indexOf('dijitRadio') !== -1 || widget.id.lastIndexOf('jfcdelete_', 0) === 0) {
          if (!widget.get('checked')) {
            continue;
          }
          key = widget.name;
        }
        parameters[key] = this.getWidgetValue(widget);
      }
      query("input[type='hidden']", "jfc_hidden_item").forEach(function(hiddenItem) {
        parameters[hiddenItem.name] = hiddenItem.value || "";
      });
    },

    getWidgetValue: function(widget) {
      var value;
      if (widget.baseClass.indexOf('dijitDateTextBox') !== -1 ||
            widget.declaredClass === 'dijit.form.FilteringSelectEx') {
        value = widget.get('displayedValue');
      } else {
        value = widget.get('value');
      }
      if (value !== value) {
        value = '';
      }
      return value || '';
    },

    getConditionFormFunc: function() {
      var conditionFormId = 'condition' + capFirst(this.modelId) + 'Cp'
      return function() {
        return dom.byId(conditionFormId);
      }
    },

    getFormId: function() {
      return 'condition' + capFirst(this.modelId) + 'Cp';
    },

    getFormFunc: function() {
      var formId = this.getFormId();
      return function() {
        return dom.byId(formId);
      }
    },

    setGridColumns: function() {
      // 幅・高さ・表示位置(横、縦)設定
      this.setColumnSetting();
      // 詳細・更新・コピーボタンの表示設定
      this.setLinks();
      // カラム設定完了後に resize メソッドを実行してグリッドの表示を整える
      this.grid.resize();
    },

    getButtonLabelWidth: function() {
      if (!this.links || this.links.length == 0) {
        return "auto";
      }
      var totalLabelLength = 0;
      for (var i=0,l=this.links.length; i<l; i++) {
        var link = this.links[i];
        if (link.type !== 'button') {
          continue;
        }
        totalLabelLength = totalLabelLength + link.label.length + 2;
      }
      return totalLabelLength + "em";
    },

    getLabelStyles: function(column) {
      if (!column.style || !column.style.label) {
        return {};
      }
      if (column.id === "buttonField" && !column.style.label.width) {
        column.style.label.width = this.getButtonLabelWidth();
      }
      return column.style.label;
    },

    getValueStyles: function(column) {
      if (!column.style || !column.style.value) {
        return {};
      }
      return column.style.value;
    },

    setColumnSetting: function() {
      if (!this.grid.columns) {
        return;
      }
      for (var key in this.grid.columns) {
        var column = this.grid.columns[key];
        // ヘッダ部のスタイル設定
        var styles = this.getLabelStyles(column);
        if (Object.keys(styles).length > 0) {
          domStyle.set(this.grid.columns[column.id].headerNode, styles);
          // 値部の幅をヘッダの幅に合わせる
          if (styles.width) {
            this.grid.styleColumn(column.id, 'width: ' + styles.width);
          }
        }
        this.setRenderCellFunc(column.id, column);
      }
    },

    setRenderCellFunc: function(columnId, column) {
      if (!this.grid.columns[columnId] || columnId === 'selectorField') {
        return;
      }
      this.grid.columns[columnId].renderCell = this.getRenderCellFunc(this.grid.columns[columnId]);
    },

    getRenderCellFunc: function(column) {
      var that = this;
      var styles = this.getValueStyles(column);
      return function(object, value, cell, options) {
        if (Object.keys(styles).length > 0) {
          // 値部のスタイル設定
          domStyle.set(cell, styles);
        }
        if (!column.type) {
          that.grid._defaultRenderCell(object, value, cell, options);
        } else {
          // 入力タイプ設定がある場合は、ウィジェットを表示する
          that.setWidget(column, object, value, cell, options);
        }
      };
    },

    setWidget: function(column, object, value, node, options) {
      if (column.type !== "barGauge") {
        this.grid._defaultRenderCell(object, value, node, options);
        return;
      }
      var props = this.getBarGaugeProps(column);
      var that = this;
      setTimeout(function() {
        // バーゲージを配置するDOMをグリッドの行表示後に生成しないと
        // 高さ、幅が取得できないため、表示を 20ms 遅らせる
        var widget = that.getBarGaugeWidget({
          id: that.modelId + '_lp$002fitem$005b' + object._row_number + '$005d$002f' + column.id,
          value: value,
          props: props,
          node: node,
        });
        node.widgets = [widget];
      }, 20);
    },

    getBarGaugeProps: function(column) {
      var props = {};
      var styles = this.getValueStyles(column);
      if (Object.keys(styles).length === 0) {
        styles = {
          'height': '30px',
          'width': '100%',
        };
      } else {
        if (!styles['height']) {
          styles['height'] = '30px';
        }
      }
      props.styles = styles;
      if (column.barGaugeProps) {
        if (column.barGaugeProps.unit) {
          props.unit = column.barGaugeProps.unit;
        }
        if (column.barGaugeProps.maximum) {
          props.maximum = column.barGaugeProps.maximum;
        }
        if (column.barGaugeProps.color) {
          props.color = column.barGaugeProps.color;
        }
      }
      return props;
    },

    getBarGaugeWidget: function(args) {
      var node = domConstruct.create("div", {
        id: args.id,
      }, args.node);
      domStyle.set(node, args.props.styles);
      var props = {};
      props.value = args.value;
      if (args.props.maximum) {
        props.maximum = args.props.maximum;
      }
      if (args.props.unit) {
        props.unit = args.props.unit;
      }
      if (args.props.color) {
        props.color = args.props.color;
      }
      var barGauge = new BarGauge(props, node);
      return barGauge;
    },

    setLinks: function() {
      if (!this.links || this.links.length == 0) {
        return;
      }
      var isLinkDefined = false;
      var buttons = [];
      for (var i=0,l=this.links.length; i<l; i++) {
        var link = this.links[i];
        if (link.type === 'button') {
          buttons.push(link);
        } else if (link.type === 'click') {
          // 行クリックのイベント設定
          this.setClickEvent(link);
          isLinkDefined = true;
        }
      }
      this.setButtons(buttons);
      this.setGridSelectionMode(isLinkDefined);
    },

    setButtons: function(buttons){
      var buttonFieldCell = this.grid.columns['buttonField'];
      if (!buttonFieldCell) {
        return;
      }
      var that = this;
      buttonFieldCell.renderCell = this.getButtonFieldRenderCell(buttons);
    },

    setGridSelectionMode: function(isLinkDefined) {
      // 行選択で詳細表示画面へ遷移する設定が行われている
      if (isLinkDefined) {
        return;
      }
      // 一括処理が設定されていない
      if (!this.batchProcessing) {
        return;
      }
      // 行選択で一括処理対象のチェックボックスの選択状態を切り替えられるようにする
      this.grid.set('selectionMode', 'toggle');
    },

    getActionName: function(action, id) {
      return 'action_' + action + ':' + this.idProperty + ':' + id;
    },

    getButtonId: function(button, object) {
      return 'btn' + button.action + this.modelId + this.getActionName(button.action, object[this.ID_VALUE_KEY]);
    },

    getButtonFieldRenderCell: function(buttons) {
      var that = this;
      var styles = this.getValueStyles(this.grid.columns['buttonField']);
      return function(object, value, node) {
        domStyle.set(node, styles);
        widgets = [];
        for (var i=0; i<buttons.length; i++) {
          var actionName = that.getActionName(buttons[i].action, object[that.ID_VALUE_KEY]);
          var id = that.getButtonId(buttons[i], object);
          var btnNodeStyle = {};
          if (i > 0 && i < buttons.length) {
            btnNodeStyle['style'] = 'padding-left: 7px';
          }
          var btnNode = domConstruct.create('span', btnNodeStyle, node);
          var button = that.createButton({
            id: id,
            jfcCount: object.jfcCount,
            label: buttons[i].label,
            iconClass: buttons[i].iconClass,
            action: buttons[i].forward,
            actionName: actionName,
            node: btnNode,
          });
          button.on('click', that.getButtonClickEventFunc({
            action: buttons[i].forward,
            actionName: actionName,
            jfcCount: object.jfcCount,
          }));
          widgets.push(button);
        }
        node.widgets = widgets;
      }
    },

    setClickEvent: function(link) {
      var that = this;
      // 行クリックの詳細表示画面遷移のイベント
      this.grid.on('.dgrid-content .dgrid-row:click', function(event) {
        // 行選択チェックボックス、ボタン配置列をクリックした場合は詳細表示画面への遷移は行わない
        var node = event.target;
        if ((node.nodeName === 'INPUT' && node.name === 'pkey_jsh_cb') ||
            (node.nodeName === 'TD' && (domClass.contains(node, 'dgrid-column-selectorField') ||
                                        domClass.contains(node, 'dgrid-column-buttonField')))) {
          return;
        }
        if (!event.selectorTarget.id) {
          return;
        }
        var ids = event.selectorTarget.id.split('-');
        if (ids.length !== 3) {
          return;
        }
        var id = ids[2];
        var form = event.selectorTarget.parentNode.parentNode.parentNode.parentNode;
        form.action = link.forward;
        form.SENDACTION.name = that.getActionName(link.action, id);
        form.submit();
      });
    },

    getButtonClickEventFunc: function(args) {
      return function(e) {
        // 行クリックで詳細表示画面遷移が設定されているとボタン押下で詳細表示画面へ
        // 遷移するため、イベントの伝播を止める
        e.stopPropagation();
        e.preventDefault();
        e.target.form.action = args.action;
        e.target.form.SENDACTION.name = args.actionName;
        e.target.form.submit();
      }
    },

    createButton: function(args) {
      var button = registry.byId(args.id);
      if (button) {
        button.destroyRecursive();
      }
      button = new Button({
        id: args.id,
        type: 'button',
        label: args.label,
      }).placeAt(args.node);
      if (args.onClick) {
        button.on('click', function() {
          args.onClick();
        });
      }
      // アイコンのCSSが設定されている場合、ボタンをアイコンで表示する
      if (args.iconClass) {
        button.set('iconClass', args.iconClass);
        button.set('showLabel', false);
        button.set('title', args.label);
      } else {
        button.set('label', args.label);
      }
      domStyle.set(button.domNode, 'margin', '0');
      domStyle.set(button.domNode.firstChild, 'padding', '0 1px');
      return button;
    },

    getParentId: function(idref, refname) {
      // input(hidden)タグを取得する
      id = idref + '_p$002f' + refname;
      node = dom.byId(id);
      if (node) {
        return node.value;
      }
      return '';
    },

    resetHeaderColumnStyle: function() {
      if (!this.grid.columns) {
        return;
      }
      for (var key in this.grid.columns) {
        var column = this.grid.columns[key];
        // ヘッダ部のスタイル設定
        var styles = this.getLabelStyles(column);
        if (Object.keys(styles).length > 0) {
          domStyle.set(this.grid.columns[column.id].headerNode, styles);
        }
      }
    },

    getTextSearchJfcHiddenItem: function() {
      var that = this;
      query("form[name^='condition'][name$='CpIndex'] div#jfc_hidden_item").forEach(function(node) {
        that.textSearchJfcHiddenItem = node;
      });
    },

    setConditionToTextSearchForm: function(responseData) {
      if (!this.textSearchJfcHiddenItem) {
        return;
      }
      var checkboxConditionItems = this.getCheckboxConditionItems();
      var condition = this.getCondition(responseData, checkboxConditionItems);
      for (var key in condition) {
        var queryString = "input[type='hidden']";
        if (key.lastIndexOf("$005d$002f$0040id") === -1) {
          queryString = queryString + "[name*='" + key + "']";
        } else {
          queryString = queryString + "[name='" + key + "']";
        }
        var node = query(queryString, this.textSearchJfcHiddenItem)[0];
        if (node) {
          node.value = condition[key];
        } else {
          if (key.lastIndexOf("$005d$002f$0040id") !== -1) {
            var hiddenItem = domConstruct.create("input", {
              type: "hidden",
              id: key,
              name: key,
              value: condition[key],
            }, this.textSearchJfcHiddenItem);
          }
        }
      }
    },

    getCheckboxConditionItems: function() {
      var checkboxItems = {};
      query("input[type='checkbox']").forEach(function(node) {
        checkboxItems[node.id] = true;
      });
      return checkboxItems;
    },

    getCondition: function(responseData, checkboxConditionItems) {
      var condition = {};
      var conditionp = responseData.conditionp;
      if (!conditionp) {
        return condition;
      }
      for (var key in conditionp) {
        if (this.isHash(conditionp[key]) && key.indexOf("textsearchjshparam") === -1) {
          condition[key] = conditionp[key].content || "";
        } else if (Array.isArray(conditionp[key]) && key !== "noselection_jshparam") {
          var keyPrefix = this.modelId + "_cp$002f" + key + "$005b";
          var checkboxConditions = conditionp[key];
          for (var i=0,l=checkboxConditions.length; i<l; i++) {
            var checkboxConditionKey = keyPrefix + checkboxConditions[i].id + "$005d$002f$0040id";
            if (!checkboxConditionItems[checkboxConditionKey]) {
              checkboxConditionKey = keyPrefix + "1$005d$002f$0040id";
            }
            var value = "";
            if (checkboxConditions[i].choose && !checkboxConditions[i].noId) {
              value = checkboxConditions[i].id;
            }
            condition[checkboxConditionKey] = value;
            if (!checkboxConditionItems[checkboxConditionKey] && checkboxConditions[i].choose && !checkboxConditions[i].noId) {
              break;
            }
          }
        }
      }
      return condition;
    },

    isHash: function(obj) {
      if (this.getTypeString(obj) === "object") {
        return true;
      }
      return false;
    },

    getTypeString: function(obj) {
      return Object.prototype.toString.call(obj).slice(8, -1).toLowerCase();
    },

  });
});
