/**
 * レイアウト設定
 */
define([
  "dojo/_base/declare",
  "dojo/aspect",
  "dojo/dom",
  "dojo/dom-construct",
  "dojo/dom-style",
  "dojo/store/Memory",
  "dijit/registry",
  "dijit/form/CheckBox",
  "dijit/layout/ContentPane",
  "dijit/layout/TabContainer",
  "wb/Layout",
  "wb/LayoutRepository",
  "wb/LayoutRepositoryItem",
  "wb/LayoutWidget",
  "wb/PaneContent",
  "wb/layout/ConditionLayoutPaneContent",
  "wb/layout/DetailLayoutPaneContent",
  "wb/layout/ShowListLayoutPaneContent"
], function(
  declare,
  aspect,
  dom,
  domConstruct,
  domStyle,
  Memory,
  registry,
  CheckBox,
  ContentPane,
  TabContainer,
  Layout,
  LayoutRepository,
  LayoutRepositoryItem,
  LayoutWidget,
  PaneContent,
  ConditionLayoutPaneContent,
  DetailLayoutPaneContent,
  ShowListLayoutPaneContent
) {
  return declare([PaneContent, LayoutRepository], {
    constructor: function(initValue) {
      this.title = wagbyDesigner.jfcRes.LABEL_TAB_Layout; // "レイアウト"
      this.frameName = "Layout";
      this.repositoryKeys = this.layoutRepositoryKeys;
      this.layouts = [];
      this.setContents();
      this.loadedTabContents = [];
    },

    createWidget: function(key) {
      return new LayoutWidget({
        key: key,
        container: this
      });
    },

    tabContentLoaded: function(title) {
      // 全タブのロード完了後にリソース設定の関数が実行されるようにする
      this.loadedTabContents.push(title);
      if (this.loadedTabContents.length === this.tabCount) {
        this.onContentsLoaded();
      }
    },

    onContentsLoaded: function() {
      this.inherited(arguments);
      this.layoutListDropDownButton = registry.byId("layoutListDropDownButton");
      this.parentPane.tabContentLoaded(this.title);
      this.setLayoutGroupEvent();
    },

    setScreenResource: function() {
      this.inherited(arguments);
      // レイアウトグループ・タブ一覧ギアアイコン
      jfcAddLayout.set("label", wagbyDesigner.jfcRes.LABEL_AddItem);
      jfcCopyLayout.set("label", wagbyDesigner.jfcRes.LABEL_CopyItem);
      jfcDeleteLayout.set("label", wagbyDesigner.jfcRes.LABEL_DeleteItem);
      jfcLayoutCheckAllRow.set("label", wagbyDesigner.jfcRes.LABEL_CheckAllListRow);
      jfcLayoutUncheckAllRow.set("label", wagbyDesigner.jfcRes.LABEL_UncheckAllListRow);
    },

    setLayoutGroupEvent: function() {
      var that = this;
      jfcAddLayout.on("click", function() {
        that.addLayoutAtSelectedIds();
      });
      jfcCopyLayout.on("click", function() {
        that.copyLayout();
      });
      jfcDeleteLayout.on("click", function() {
        that.deleteLayout();
      });
    },

    // 内容部はタブで表示する
    setContents: function() {
      this.inherited(arguments);
      this.tabs = [];
      var that = this;
      // コンテンツ表示用のフレーム内にタブコンテナを生成
      var tabContainer = this.createTabContainer();
      // 検索レイアウト
      this.conditionLayoutPane = new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_LAYOUT_condition, // "検索"
        href: "wb/paneContentTemplate/layout/conditionLayout.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        preload: true
      });
      this.conditionLayoutPane.on("Load", function() {
        that.conditionLayoutPaneContent = new ConditionLayoutPaneContent({
          parent: that
        });
        that.conditionLayoutPaneContent.createContainer();
      });
      this.conditionLayoutPane.on("Show", function() {
        that.showConditionLayout();
      });
      this.tabs.push(this.conditionLayoutPane);

      // 一覧レイアウト
      this.showListLayoutPane = new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_LAYOUT_showList, // "一覧"
        href: "wb/paneContentTemplate/layout/showListLayout.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        preload: true
      });
      this.showListLayoutPane.on("Load", function() {
        that.showListLayoutPaneContent = new ShowListLayoutPaneContent({
          parent: that
        });
        that.showListLayoutPaneContent.createContainer();
      });
      this.showListLayoutPane.on("Show", function() {
        that.showShowListLayout();
      });
      this.tabs.push(this.showListLayoutPane);

      this.detailLayoutPane = new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_LAYOUT_detail, // 詳細
        href: "wb/paneContentTemplate/layout/detailLayout.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        preload: true
      });
      this.detailLayoutPane.on("Load", function() {
        that.detailLayoutPaneContent = new DetailLayoutPaneContent({
          parent: that
        });
        that.detailLayoutPaneContent.create();
      });
      this.detailLayoutPane.on("Show", function() {
        that.showDetailLayout();
      });
      this.tabs.push(this.detailLayoutPane);

      this.layoutRepositoryPane = new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_LAYOUT_repository, // リポジトリ
        href: "wb/paneContentTemplate/layout/layoutRepository.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        preload: true,
      });
      // レイアウトリポジトリ設定用タブコンテナを生成
      this.layoutRepositoryPane.on("Load", function() {
        that.createLayoutRepositoryTab();
      });
      this.layoutRepositoryPane.on("Show", function() {
        that.initLayoutRepositoryTabContents();
      });
      this.layoutRepositoryPane.on("Hide", function() {
        that.hide();
      });

      this.tabs.push(this.layoutRepositoryPane);
      this.tabCount = this.tabs.length;

      for (var i=0,l=this.tabs.length; i<l; i++) {
        var tab = this.tabs[i];
        tab.on("Load", function() {
          that.tabContentLoaded(this.title);
        });
        tabContainer.addChild(tab);
      }
      tabContainer.startup();
      // レイアウト設定が変更されている場合、レイアウトタブの切り替え時に確認ダイアログを表示する
      aspect.around(tabContainer, "selectChild", function(selectChild) {
        return function(page) {
          if (!wagbyDesigner.confirmDiscardLayoutSetting()) {
            return;
          }
          selectChild.apply(this, arguments);
        }
      });
    },

    createLayoutRepositoryTab: function() {
      var that = this;
      this.repositoryTabs = [];
      this.layoutNamePane = new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_LAYOUT_layoutName, // "レイアウト名"
        href: "wb/paneContentTemplate/layout/layoutName.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        preload: true
      });
      this.layoutNamePane.on("Show", function() {
        that.createLayoutNameWidgets();
      });
      this.repositoryTabs.push(this.layoutNamePane);

      this.groupTabPane = new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_LAYOUT_groupTab, // "グループ・タブ"
        href: "wb/paneContentTemplate/layout/groupTab.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        preload: true
      });
      this.groupTabPane.on("Show", function() {
        that.createGroupTabWidgets();
      });
      this.repositoryTabs.push(this.groupTabPane);

      this.containerPane = new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_LAYOUT_container, // "繰り返しコンテナ"
        href: "wb/paneContentTemplate/layout/container.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        preload: true
      });
      this.containerPane.on("Show", function() {
        that.createContainerWidgets();
      });
      this.repositoryTabs.push(this.containerPane);

      this.conditionPane = new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_LAYOUT_condition, // "検索"
        href: "wb/paneContentTemplate/layout/condition.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        preload: true
      });
      this.conditionPane.on("Show", function() {
        that.createConditionWidgets();
      });
      this.repositoryTabs.push(this.conditionPane);

      this.showListPane = new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_LAYOUT_showList, // "一覧"
        href: "wb/paneContentTemplate/layout/showList.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        preload: true
      });
      this.showListPane.on("Show", function() {
        that.createShowListWidgets();
      });
      this.repositoryTabs.push(this.showListPane);

      var repositoryTabContainer = new TabContainer({
        style: "width: 100%; height: 100%;"
      }, "jfcLayoutRepositoryPane");

      this.tabCount = this.tabCount + this.repositoryTabs.length;

      for (var i=0,l=this.repositoryTabs.length; i<l; i++) {
        var tab = this.repositoryTabs[i];
        tab.on("Load", function() {
          that.tabContentLoaded(this.title);
        });
        repositoryTabContainer.addChild(tab);
      }
      repositoryTabContainer.startup();
    },

    showConditionLayout: function() {
      this.conditionLayoutPaneContent.show(wagbyDesigner.activeModel);
    },

    showShowListLayout: function() {
      this.showListLayoutPaneContent.show(wagbyDesigner.activeModel);
    },

    showDetailLayout: function() {
      this.detailLayoutPaneContent.show(wagbyDesigner.activeModel);
    },

    initWidgets: function() {
    },

    setRepositoryData: function() {
    },

    getLayoutNameStore: function() {
      if (!wagbyDesigner.activeModel.modelitems) {
        return null;
      }
      var layoutNameStore = new Memory();
      for (var i=0,l=wagbyDesigner.activeModel.modelitems.length; i<l; i++) {
        var modelitem = wagbyDesigner.activeModel.modelitems[i];
        var name;
        if (modelitem.isItemTypeContainer()) {
          name = modelitem.label.value;
        } else {
          name = modelitem.layoutName.value;
        }
        if (!name) {
          continue;
        }
        var layoutName = layoutNameStore.query({name: name});
        if (layoutName.total !== 0) {
          continue;
        }
        layoutNameStore.put({
          id: modelitem.id,
          name: name
        });
      }
      return layoutNameStore;
    },

    show: function() {
      jfcModelitemStandby.show();
      var that = this;
      setTimeout(function() {
        that.initTabContents();
      }, 1);
    },

    initTabContents: function() {
      var that = this;
      if (this.conditionLayoutPane.get("selected")) {
        this.showConditionLayout();
      } else if (this.showListLayoutPane.get("selected")) {
        this.showShowListLayout();
      } else if (this.detailLayoutPane.get("selected")) {
        this.showDetailLayout();
      } else if (this.layoutRepositoryPane.get("selected")) {
        this.initLayoutRepositoryTabContents();
      }
    },

    initLayoutRepositoryTabContents: function() {
      if (this.layoutNamePane.get("selected")) {
        this.createLayoutNameWidgets();
      } else if (this.groupTabPane.get("selected")) {
        this.createGroupTabWidgets();
      } else if (this.containerPane.get("selected")) {
        this.createContainerWidgets();
      } else if (this.conditionPane.get("selected")) {
        this.createConditionWidgets();
      } else if (this.showListPane.get("selected")) {
        this.createShowListWidgets();
      }
    },

    createLayoutNameTableRow: function(modelitem, rowNumber) {
      var tr = domConstruct.create("tr", {
        style: "height: 15px; marign: 0; padding: 0;"
      });

      // 行番号
      var noContainer = domConstruct.create("td", {
        style: "width: 5%; text-align: center;",
        innerHTML: rowNumber
      }, tr);

      // 項目名
      var labelContainer = domConstruct.create("td", {
        style: "width: 45%;",
        innerHTML: modelitem.getLabelWithName()
      }, tr);

      // レイアウト名
      var layoutNameId = "place_model_modelitem_layoutName" + modelitem.id;
      var nameContainer = domConstruct.create("td", {
        id: layoutNameId,
        style: "width: 45%;"
      }, tr);

      // 表示順
      var displayOrderId = "place_model_modelitem_detailDisplayOrder" + modelitem.id;
      var nameContainer = domConstruct.create("td", {
        id: displayOrderId,
        style: "width: 5%;"
      }, tr);

      return tr;
    },

    createGroupTabTableRow: function(layout) {
      var id = layout.id;
      var no = layout.getNo();

      // レイアウト行
      var tr = domConstruct.create("tr");

      // 行選択チェックボックス
      var rowSelectContainer = domConstruct.create("td", {
        id: "rowSelect" + id,
        'class': "centering"
      }, tr);
      var rowSelect = new CheckBox({
        name: "rowSelect",
        'class': "rowSelect",
        value: layout.id,
        checked: false,
        tabIndex: -1
      });
      rowSelect.placeAt(rowSelectContainer);
      rowSelect.set("readOnly", wagbyDesigner.activeModel.isLocked());

      // No.
      domConstruct.create("td", {'class': "rowNo centering", innerHTML: no}, tr);
      // レイアウト名
      domConstruct.create("td", {id: "place_model_displayitemgroup_label" + id}, tr);
      // グループの表示方法
      domConstruct.create("td", {id: "place_model_displayitemgroup_useFieldSet" + id}, tr);
      // 並び方向
      domConstruct.create("td", {id: "place_model_displayitemgroup_separate1" + id}, tr);
      // 横並び子要素の表示比率
      domConstruct.create("td", {id: "place_model_displayitemgroup_separate2" + id}, tr);
      // ラベルの幅
      domConstruct.create("td", {id: "place_model_displayitemgroup_labelwidth" + id}, tr);
      // タブ番号
      domConstruct.create("td", {id: "place_model_displayitemgroup_tabnumber" + id}, tr);
      // デフォルトタブ
      var defaultTabTd = domConstruct.create("td", {}, tr);
      // デフォルトタブチェックボックス
      domConstruct.create("span", {id: "place_model_displayitemgroup_defaulttab" + id}, defaultTabTd);
      // タブ選択条件
      domConstruct.create("span", {id: "place_model_displayitemgroup_selectTabCondition" + id}, defaultTabTd);
      // タブ表示条件
      domConstruct.create("td", {id: "place_model_displayitemgroup_tabvisible" + id}, tr);

      return tr;
    },

    // コンテナ設定テーブル作成
    createContainerOptionTableBody: function() {
      var model = wagbyDesigner.activeModel;
      for (var i=0,l=model.modelitems.length; i<l; i++) {
        var modelitem = model.modelitems[i];
        // コンテナ項目以外は表示しない
        if (!modelitem.isContainerItem()) {
          continue;
        }
        var tr = this.createContainerOptionTableRow(modelitem, (i + 1));
        domConstruct.place(tr, "containerLayoutListBody");
      }
    },

    createContainerOptionTableRow: function(modelitem, rowNumber) {
      var tr = domConstruct.create("tr", {
        style: "height: 15px; marign: 0; padding: 0;"
      });

      // 行番号
      var noContainer = domConstruct.create("td", {
        style: "width: 10%; text-align: center;",
        innerHTML: rowNumber
      }, tr);

      // 項目名
      var labelContainer = domConstruct.create("td", {
        style: "width: 30%;",
        innerHTML: modelitem.getLabelWithName()
      }, tr);

      // ラベル名
      var containerLabelId = "place_model_modelitem_containerLabel" + modelitem.id;
      var containerLabelContainer = domConstruct.create("td", {
        id: containerLabelId,
        style: "width: 20%;"
      }, tr);

      // セル結合(行)
      var containerRowspanId = "place_model_modelitem_containerRowspan" + modelitem.id;
      var containerRowspanContainer = domConstruct.create("td", {
        id: containerRowspanId,
        style: "width: 20%;"
      }, tr);

      // セル結合(列)
      var containerColspanId = "place_model_modelitem_containerColspan" + modelitem.id;
      var containerColspanContainer = domConstruct.create("td", {
        id: containerColspanId,
        style: "width: 20%;"
      }, tr);

      return tr;
    },

    // 一覧表示設定テーブル作成
    createShowlistOptionTableBody: function() {
      var model = wagbyDesigner.activeModel;
      for (var i=0,l=model.modelitems.length; i<l; i++) {
        var modelitem = model.modelitems[i];
        // 一覧表示設定無しの項目は表示しない
        if (!modelitem.hasListModel()) {
          continue;
        }
        var tr = this.createShowlistOptionTableRow(modelitem, (i + 1));
        domConstruct.place(tr, "showListLayoutListBody");
      }
    },

    createShowlistOptionTableRow: function(modelitem, rowNumber) {
      var tr = domConstruct.create("tr", {
        style: "height: 15px; marign: 0; padding: 0;"
      });

      // 行番号
      var noContainer = domConstruct.create("td", {
        style: "width: 6%; text-align: center;",
        innerHTML: rowNumber
      }, tr);

      // 項目名
      var labelContainer = domConstruct.create("td", {
        style: "width: 32%;",
        innerHTML: modelitem.getLabelWithName()
      }, tr);

      // ラベル名
      var showListLabelId = "place_model_modelitem_showListLabel" + modelitem.id;
      var showListLabelContainer = domConstruct.create("td", {
        id: showListLabelId,
        style: "width: 32%;"
      }, tr);

      // セル結合(行)
      var showListRowspanId = "place_model_modelitem_showListRowspan" + modelitem.id;
      var showListRowspanContainer = domConstruct.create("td", {
        id: showListRowspanId,
        style: "width: 10%;"
      }, tr);

      // セル結合(列)
      var showListColspanId = "place_model_modelitem_showListColspan" + modelitem.id;
      var showListColspanContainer = domConstruct.create("td", {
        id: showListColspanId,
        style: "width: 10%;"
      }, tr);

      // ソートキー
      var showListDisplayOrderId = "place_model_modelitem_showListDisplayOrder" + modelitem.id;
      var showListDisplayOrderContainer = domConstruct.create("td", {
        id: showListDisplayOrderId,
        style: "width: 10%;"
      }, tr);

      return tr;
    },

    createLayoutNameWidgets: function() {
      if (this.isLayoutNameWidgetsSet) {
        jfcModelitemStandby.hide();
        return;
      }
      var that = this;
      var model = wagbyDesigner.activeModel;
      for (var i=0,l=model.modelitems.length; i<l; i++) {
        var modelitem = model.modelitems[i];
        var tr = that.createLayoutNameTableRow(modelitem, (i + 1));
        domConstruct.place(tr, "layoutNameListBody");
        that.showLayoutWidgets(modelitem, [
          "model/modelitem/@layoutName",
          "model/modelitem/@detailDisplayOrder"
        ]);
      }
      jfcModelitemStandby.hide();
      this.isLayoutNameWidgetsSet = true;
    },

    createGroupTabWidgets: function() {
      this.setGroupTabLocked();
      if (this.isGroupTabWidgetsSet) {
        return;
      }
      var that = this;
      this.initLayouts(function() {
        // レイアウト名を取得する
        var layoutNameStore = that.getLayoutNameStore();
        for (var i=0,l=that.layouts.length; i<l; i++) {
          var layout = that.layouts[i];
          var tr = that.createGroupTabTableRow(layout);
          domConstruct.place(tr, "groupTabListBody");
          layout.setLayoutNameStore(layoutNameStore);
          layout.createWidget();
          layout.show();
          that.setLayoutLocked(layout);
        }
        jfcModelitemStandby.hide();
        that.isGroupTabWidgetsSet = true;
      });
    },

    setGroupTabLocked: function() {
      this.layoutListDropDownButton.set("disabled", wagbyDesigner.activeModel.isLocked());
    },

    setLayoutLocked: function(layout) {
      var readOnly = wagbyDesigner.activeModel.isLocked();
      for (var repositoryKey in layout.repositoryKeys) {
        layout.widgets[repositoryKey].widget.set("readOnly", readOnly);
      }
    },

    createContainerWidgets: function() {
      if (this.isContainerWidgetsSet) {
        return;
      }
      var that = this;
      var containerLayoutListBody = dom.byId("containerLayoutListBody");
      var layoutContainerTable = dom.byId("layoutContainerTable");
      var layoutContainerMessage = dom.byId("layoutContainerMessage");
      domStyle.set(layoutContainerTable, "display", "none");
      var model = wagbyDesigner.activeModel;
      var isEmpty = true;
      for (var i=0,l=model.modelitems.length; i<l; i++) {
        var modelitem = model.modelitems[i];
        // コンテナ項目以外は表示しない
        if (!modelitem.isContainerItem()) {
          continue;
        }
        var tr = that.createContainerOptionTableRow(modelitem, (i + 1));
        domConstruct.place(tr, containerLayoutListBody);
        that.showLayoutWidgets(modelitem, [
          "model/modelitem/@containerLabel",
          "model/modelitem/@containerRowspan",
          "model/modelitem/@containerColspan"
        ]);
        isEmpty = false;
      }
      if (!isEmpty) {
        domStyle.set(layoutContainerTable, "display", "");
        domStyle.set(layoutContainerMessage, "display", "none");
      } else {
        domStyle.set(layoutContainerTable, "display", "none");
        domStyle.set(layoutContainerMessage, "display", "");
      }
      jfcModelitemStandby.hide();
      this.isContainerWidgetsSet = true;
    },

    createShowListWidgets: function() {
      if (this.isShowListWidgetsSet) {
        return;
      }
      var that = this;
      var showListLayoutListBody = dom.byId("showListLayoutListBody");
      var layoutShowListTable = dom.byId("layoutShowListTable");
      var layoutShowListMessage = dom.byId("layoutShowListMessage");
      domStyle.set(layoutShowListTable, "display", "none");
      var model = wagbyDesigner.activeModel;
      var isEmpty = true;
      for (var i=0,l=model.modelitems.length; i<l; i++) {
        var modelitem = model.modelitems[i];
        // 一覧表示設定無しの項目は表示しない
        if (!modelitem.hasListModel()) {
          continue;
        }
        var tr = that.createShowlistOptionTableRow(modelitem, (i + 1));
        domConstruct.place(tr, showListLayoutListBody);
        that.showConditionShowListWidgets(modelitem, [
          "model/modelitem/@showListLabel",
          "model/modelitem/@showListRowspan",
          "model/modelitem/@showListColspan",
          "model/modelitem/@showListDisplayOrder"
        ]);
        isEmpty = false;
      }
      if (!isEmpty) {
        domStyle.set(layoutShowListTable, "display", "");
        domStyle.set(layoutShowListMessage, "display", "none");
      } else {
        domStyle.set(layoutShowListMessage, "display", "");
      }
      jfcModelitemStandby.hide();
      this.isShowListWidgetsSet = true;
    },

    createConditionWidgets: function() {
      if (this.isConditionWidgetsSet) {
        return;
      }
      var that = this;
      var layoutConditionBody = dom.byId("layoutConditionBody");
      var layoutConditionTable = dom.byId("layoutConditionTable");
      var layoutConditionMessage = dom.byId("layoutConditionMessage");
      domStyle.set(layoutConditionTable, "display", "none");
      var model = wagbyDesigner.activeModel;
      var isEmpty = true;
      for (var i=0,l=model.modelitems.length; i<l; i++) {
        var modelitem = model.modelitems[i];
        // 検索条件設定無しの項目は表示しない
        if (!modelitem.hasConditionModel()) {
          continue;
        }
        var tr = that.createConditionTableRow(modelitem, (i + 1));
        domConstruct.place(tr, layoutConditionBody);
        that.showConditionShowListWidgets(modelitem, [
          "model/modelitem/@conditionDisplayOrder",
          "presentation/displayitem/@columnForSearch"
        ]);
        isEmpty = false;
      }
      if (!isEmpty) {
        domStyle.set(layoutConditionTable, "display", "");
        domStyle.set(layoutConditionMessage, "display", "none");
      } else {
        domStyle.set(layoutConditionTable, "display", "none");
        domStyle.set(layoutConditionMessage, "display", "");
      }
      jfcModelitemStandby.hide();
      this.isConditionWidgetsSet = true;
    },

    createConditionTableRow: function(modelitem, rowNumber) {
      var tr = domConstruct.create("tr", {
        style: "height: 15px; marign: 0; padding: 0;"
      });

      // 行番号
      var noContainer = domConstruct.create("td", {
        style: "width: 10%; text-align: center;",
        innerHTML: rowNumber
      }, tr);

      // 項目名
      var labelContainer = domConstruct.create("td", {
        style: "width: 60%;",
        innerHTML: modelitem.getLabelWithName()
      }, tr);

      // ソートキー
      var conditionDisplayOrderId = "place_model_modelitem_conditionDisplayOrder" + modelitem.id;
      var conditionDisplayOrderContainer = domConstruct.create("td", {
        id: conditionDisplayOrderId,
        style: "width: 15%;"
      }, tr);

      // 段組
      var conditionColumnId = "place_presentation_displayitem_columnForSearch" + modelitem.id;
      var conditionColumnContainer = domConstruct.create("td", {
        id: conditionColumnId,
        style: "width: 15%;"
      }, tr);

      return tr;
    },

    showWidgets: function() {
    },

    showLayoutWidgets: function(modelitem, repositoryKeys) {
      for (var i=0,l=repositoryKeys.length; i<l; i++) {
        var repositoryKey = repositoryKeys[i];
        var widgets = modelitem.widgets[repositoryKey];
        if (!widgets) {
          modelitem.createWidget();
          widgets = modelitem.widgets[repositoryKey];
        }
        widgets.setWidget();
        if (modelitem.repositoryItems[repositoryKey]) {
          if (widgets.value) {
            widgets.value = modelitem.repositoryItems[repositoryKey].value;
          }
          widgets.setValue(modelitem.repositoryItems[repositoryKey].value);
        }
        widgets.show();
        widgets.widget.set("readOnly", wagbyDesigner.activeModel.isLocked());
      }
    },

    showConditionShowListWidgets: function(modelitem, repositoryKeys) {
      for (var i=0,l=repositoryKeys.length; i<l; i++) {
        var repositoryKey = repositoryKeys[i];
        var widgets = modelitem.widgets[repositoryKey];
        if (!widgets) {
          modelitem.createWidget();
          widgets = modelitem.widgets[repositoryKey];
        }
        widgets.setWidget();
        var repositoryKeyValue = repositoryKey;
        // 検索、一覧の表示順の場合は特別対応
        if (repositoryKey === "model/modelitem/@conditionDisplayOrder" ||
              repositoryKey === "model/modelitem/@showListDisplayOrder") {
          // 表示順の値変更時にサーバへデータを送信するイベントをセットする
          this.setSendDataOnChangeEvent(modelitem, repositoryKey);
  
          if (repositoryKey === "model/modelitem/@conditionDisplayOrder") {
            repositoryKeyValue = "model/modelitem/@hasConditionModel";
          } else {
            repositoryKeyValue = "model/modelitem/@hasListModel";
          }
        }
        var value = modelitem.repositoryItems[repositoryKeyValue].value;
        // 設定値に○が設定されていた場合は空白とする
        if (!value || value === wagbyDesigner.jfcRes.OPTION_CHECKED) {
          value = "";
        }
        if (widgets.widget) {
          widgets.show();
          widgets.setValue(value);
          widgets.widget.set("readOnly", wagbyDesigner.activeModel.isLocked());
        }
      }
    },

    setSendDataOnChangeEvent: function(modelitem, repositoryKey) {
      // 検索設定のソートキーのリポジトリキー("model/modelitem/@conditionDisplayOrder")をサーバへ送信する際、
      // 検索設定のリポジトリキー("model/modelitem/@hasConditionModel")にすり替える
      var widgetItem = modelitem.widgets[repositoryKey];
      var repositoryKeyValue;
      if (repositoryKey === "model/modelitem/@conditionDisplayOrder") {
        repositoryKeyValue = "model/modelitem/@hasConditionModel";
      } else {
        repositoryKeyValue = "model/modelitem/@hasListModel";
      }
      widgetItem.getRepositoryKey = function() {
        return repositoryKeyValue;
      };
      widgetItem.widget.on("Change", function(value) {
        if (value === undefined) {
          return;
        }
        // 表示順を削除した際には○に戻す
        if (isNaN(value)) {
          value = wagbyDesigner.jfcRes.OPTION_CHECKED;
        }
        widgetItem.sendData(widgetItem, value, {
          onSuccess: function() {
            // 検索、一覧設定のリポジトリデータを更新しておく
            modelitem.repositoryItems[repositoryKeyValue].value = value;
          }
        });
      });
    },

    hide: function() {
      var that = this;
      setTimeout(function() {
        that.destroy();
      }, 1);
      this.isLayoutNameWidgetsSet = false;
      this.isGroupTabWidgetsSet = false;
      this.isContainerWidgetsSet = false;
      this.isConditionWidgetsSet = false;
      this.isShowListWidgetsSet = false;
      this.inherited(arguments);
    },

    destroy: function() {
      var layoutNameListBody = dom.byId("layoutNameListBody");
      if (!layoutNameListBody) {
        return;
      }
      // レイアウト名定義テーブルを消去
      domConstruct.empty("layoutNameListBody");
      // グループ・タブ定義テーブルを消去
      domConstruct.empty("groupTabListBody");
      // コンテナ定義テーブルを消去
      domConstruct.empty("containerLayoutListBody");
      // 検索定義テーブルを消去
      domConstruct.empty("layoutConditionBody");
      // 一覧表示定義テーブルを消去
      domConstruct.empty("showListLayoutListBody");
      this.layouts = [];
    },

    refresh: function() {
    },

    getLayoutCount: function() {
      return this.layouts.length;
    },

    initLayouts: function(callback) {
      this.layouts = [];
      // グループ・タブ定義テーブルを消去
      domConstruct.empty("groupTabListBody");

      var that = this;
      wagbyDesigner.sendRequest({
        url: "/model/" + wagbyDesigner.activeModel.getId() + "/layout",
        options: {
          handleAs: "json",
        }
      }, function(data) {
        for (var i=0,l=data.length; i<l; i++) {
          that.layouts.push(that.createLayout({
            no: i + 1,
            data: data[i]
          }));
        }
        if (callback) {
          callback();
        }
      }, function(data) {
        return wagbyDesigner.jfcRes.MSG_ERROR_LAYOUT_001; // "レイアウト情報の取得に失敗しました。"
      });
    },

    createLayout: function(initValue) {
      var layout = new Layout({
        id: this.layouts.length + 1,
        parent: wagbyDesigner.activeModel,
        no: initValue.no,
        data: initValue.data
      });
      return layout;
    },

    addLayout: function(layout) {
      this.layouts.push(layout);
    },

    createNewLayout: function(no) {
      var that = this;
      wagbyDesigner.sendRequest({
        url: "/model/" + wagbyDesigner.activeModel.getId() + "/layout",
        options: {
          handleAs: "json",
          headers: {
            'x-wd-insert-into': no
          },
          method: "POST",
        }
      }, function(data) {
        var layout = that.createLayout({
          no: no,
          data: data.newlayout
        });
        that.addLayoutAtNo(no, layout);
        that.groupTabLayoutModifiedNo[no] = true;
        that.checkGroupTabLayoutModified();
      }, function(data) {
        return wagbyDesigner.jfcRes.MSG_ERROR_LAYOUT_002; // "レイアウト情報の新規作成に失敗しました。"
      });
    },

    addLayoutAtNo: function(no, layout) {
      var layoutCount = this.getLayoutCount();
      if (no > layoutCount) {
        this.addLayout(layout);
      } else {
        this.layouts.splice((no - 1), 0, layout);
      }
      for (var i=0,l=this.getLayoutCount(); i<l; i++) {
        this.layouts[i].setNo(i + 1);
      }
    },

    addLayoutAtSelectedIds: function() {
      var selectedIds = wagbyDesigner.getSelectedRowIds()
      this.groupTabLayoutModifiedNo = {};
      this.addLayoutNo = [];
      if (selectedIds.length === 0) {
        this.groupTabLayoutModifiedNo[this.getLayoutCount() + 1] = false;
        this.addLayoutNo.push(this.getLayoutCount() + 1);
      } else {
        for (var i=0,l=selectedIds.length; i<l; i++) {
          var layout = this.getLayoutById(selectedIds[i]);
          this.groupTabLayoutModifiedNo[layout.getNo()] = false;
          this.addLayoutNo.push(layout.getNo());
        }
      }
      for (var i=0,l=this.addLayoutNo.length; i<l; i++) {
        this.createNewLayout(this.addLayoutNo[i]);
      }
    },

    getLayoutById: function(id) {
      for (var i=0,l=this.layouts.length; i<l; i++) {
        var layout = this.layouts[i];
        if (layout.id === id) {
          return layout;
        }
      }
      return null;
    },

    copyLayout: function() {
      var selectedIds = wagbyDesigner.getSelectedRowIds();
      if (selectedIds.length === 0) {
        return;
      }
      this.groupTabLayoutModifiedNo = {};
      var copyLayoutNo = [];
      for (var i=selectedIds.length - 1; i>=0; i--) {
        var copySource = this.getLayoutById(selectedIds[i]);
        this.groupTabLayoutModifiedNo[copySource.getNo()] = false;
        copyLayoutNo.push(copySource.getNo());
      }
      for (var i=0,l=copyLayoutNo.length; i<l; i++) {
        this.sendCopyLayout(copyLayoutNo[i]);
      }
    },

    sendCopyLayout: function(no) {
      var that = this;
      wagbyDesigner.sendRequest({
        url: "/model/" + wagbyDesigner.activeModel.getId() + "/layout",
        options: {
          handleAs: "json",
          headers: {
            'x-wd-copy-source': no
          },
          method: "POST",
        }
      }, function(data) {
        that.groupTabLayoutModifiedNo[no] = true;
        that.checkGroupTabLayoutModified();
      }, function(data) {
        return wagbyDesigner.jfcRes.MSG_ERROR_LAYOUT_003; // "レイアウト情報のコピーに失敗しました。"
      });
    },

    checkGroupTabLayoutModified: function(callback) {
      for (var no in this.groupTabLayoutModifiedNo) {
        if (!this.groupTabLayoutModifiedNo[no]) {
          return;
        }
      }
      if (callback) {
        callback();
      }
      this.isGroupTabWidgetsSet = false;
      // レイアウトグループ・タブリスト再表示
      this.createGroupTabWidgets();
    },

    deleteLayout: function() {
      var layoutCount = this.getLayoutCount();
      if (layoutCount === 0) {
        return;
      }
      var selectedIds = wagbyDesigner.getSelectedRowIds();
      // 削除確認メッセージを表示する
      if (!this.confirmDeleteLayout(selectedIds)) {
        return;
      }
      this.groupTabLayoutModifiedNo = {};
      var deleteLayoutNo = [];
      if (selectedIds.length === 0) {
        // 削除対象行が選択されていない場合、最終行を削除対象とする
        var deleteLayout = this.layouts[layoutCount - 1];
        this.groupTabLayoutModifiedNo[deleteLayout.getNo()] = false;
        deleteLayoutNo.push(deleteLayout.getNo());
      } else {
        for (var i=selectedIds.length - 1; i>=0; i--) {
          for (var j=layoutCount - 1; j>=0; j--) {
            var deleteLayout = this.layouts[j];
            if (deleteLayout.id !== selectedIds[i]) {
              continue;
            }
            this.groupTabLayoutModifiedNo[deleteLayout.getNo()] = false;
            deleteLayoutNo.push(deleteLayout.getNo());
          }
        }
      }
      for (var i=0,l=deleteLayoutNo.length; i<l; i++) {
        this.sendDeleteLayout(deleteLayoutNo[i]);
      }
    },

    confirmDeleteLayout: function(selectedIds) {
      var deleteLayouts = [];
      if (selectedIds.length === 0) {
        deleteLayouts.push(this.layouts[this.getLayoutCount() - 1]);
      } else {
        for (var i=0,l=selectedIds.length; i<l; i++) {
          deleteLayouts.push(this.getLayoutById(selectedIds[i]));
        }
      }
      var deleteLayoutLabels = "";
      for (var i=0,l=deleteLayouts.length; i<l; i++) {
        var deleteLayout = deleteLayouts[i];
        if (!deleteLayout) {
          continue;
        }
        deleteLayoutLabels += " " + deleteLayout.getNo() + ": " + deleteLayout.getLabel() + "\n";
      }
      var confirmMsg = wagbyDesigner.jfcRes.MSG_CONFIRM_DELETE_LAYOUT + deleteLayoutLabels; // "以下のレイアウトを削除してもよろしいですか?\n\n"
      return confirm(confirmMsg);
    },

    sendDeleteLayout: function(no) {
      var that = this;
      wagbyDesigner.sendRequest({
        url: "/model/" + wagbyDesigner.activeModel.getId() + "/layout/" + no,
        options: {
          handleAs: "json",
          method: "DELETE",
        }
      }, function(data) {
        that.layouts.splice(no - 1, 1);
        that.groupTabLayoutModifiedNo[no] = true;
        that.checkGroupTabLayoutModified(function() {
          that.renumberingLayouts();
        });
      }, function(data) {
        return wagbyDesigner.jfcRes.MSG_ERROR_LAYOUT_004; // "レイアウト情報の削除に失敗しました。"
      });
    },

    renumberingLayouts: function() {
      for (var i=0,l=this.getLayoutCount(); i<l; i++) {
        this.layouts[i].setNo(i + 1);
      }
    },

  });
});
