/**
 * プロジェクト環境ペインクラス
 */
define([
  "dojo/_base/declare",
  "dojo/_base/event",
  "dojo/_base/lang",
  "dojo/dom",
  "dojo/dom-construct",
  "dojo/json",
  "dojo/keys",
  "dojo/dom-style",
  "dojo/request", 
  "dijit/layout/ContentPane",
  "dijit/layout/TabContainer",
  "dijit/registry",
  "wb/Pane",
  "wb/ProjectEnvironment"
], function(declare, baseEvent, lang, dom, domConstruct, JSON, keys, domStyle, request, ContentPane, TabContainer, registry, Pane, ProjectEnvironment) {
  return declare(Pane, {
    constructor: function(initValue) {
      // プロジェクト環境設定
      this.projectEnvironment = new ProjectEnvironment({
        parentPane: this
      });
      this.loadedTabContents = [];
      this.SUBDATABASE_REPOSITORY_START_NUM = 2; // サブデータベースのリポジトリ添え字開始番号
      this.MAX_SUBDATABASE_SETTINGS = 5; // サブデータベースの設定数
      this.SUBDATABASE_REPOSITORY_COUNT = this.SUBDATABASE_REPOSITORY_START_NUM + this.MAX_SUBDATABASE_SETTINGS;
      this.defaultDBSettings = {
        hostname: "localhost",
        instance: "wagbydb",
      };
      this.MAX_KEY_EVENT_SETTINGS = 15; // キーカスタマイズの設定数
      this.KEYCODE_PREFIX = "project/customize/key/eventhandler/@keycode";
    },

    onPaneLoaded: function() {
      this.inherited(arguments);
      var that = this;
      this.projectEnvironment.setRepositoryItemInitializerCallback(function() {
        that.setWidgets();
      });
      this.projectEnvironment.getRepositoryItems({
        url: "/project/",
        initializer: that.projectEnvironment.getRepositoryItemInitializer()
      });
    },

    setScreenResource: function() {
      this.inherited(arguments);
      // スクリプトタブ内のタブ名をセットする
      jfcProjectEnvironmentDatabaseSettingTab1.set("title", wagbyDesigner.jfcRes.LABEL_TAB_DATABASE_1);
      jfcProjectEnvironmentDatabaseSettingTab2.set("title", wagbyDesigner.jfcRes.LABEL_TAB_DATABASE_2);
    },

    setWidgets: function() {
      this.projectEnvironment.createWidget();
      this.projectEnvironment.showWidgets();
      // データベース設定の選択肢をサーバから取得する
      this.getDatabaseSettings();
      this.setWidgetEvent();
      // 「画面」->「その他」の「サブデータベース」設定の選択肢を用意する
      this.setValidSubDatabaseSettingOption();
      // 「カスタマイズ」のキーイベントを設定する
      this.setKeyEventHandler();
    },

    getIgnoreKeyEventKeys: function(keys) {
      var ignoreKeys = {};
      ignoreKeys[keys.BACKSPACE] = true;
      ignoreKeys[keys.TAB] = true;
      ignoreKeys[keys.CLEAR] = true;
      ignoreKeys[keys.ENTER] = true;
      ignoreKeys[keys.SHIFT] = true;
      ignoreKeys[keys.CTRL] = true;
      ignoreKeys[keys.ALT] = true;
      ignoreKeys[keys.PAUSE] = true;
      ignoreKeys[keys.CAPS_LOCK] = true;
      ignoreKeys[keys.SPACE] = true;
      ignoreKeys[keys.INSERT] = true;
      ignoreKeys[keys.DELETE] = true;
      ignoreKeys[keys.SELECT] = true;
      ignoreKeys[keys.NUM_LOCK] = true;
      ignoreKeys[keys.SCROLL_LOCK] = true;
      return ignoreKeys;
    },

    setKeyEventHandler: function() {
      var ignoreKeys = this.getIgnoreKeyEventKeys(keys);
      for (var i=1,l=this.MAX_KEY_EVENT_SETTINGS; i<=l; i++) {
        this.setKeyCodeSettingEventFunc(i, ignoreKeys);
        var display = "none";
        var keyCode = this.projectEnvironment.widgets[this.KEYCODE_PREFIX + i].value;
        if (keyCode && keyCode !== "") {
          display = "";
        }
//        this.displayKeyCodeSetting(i, display);
      }
    },

    setKeyCodeSettingEventFunc: function(no, ignoreKeys) {
      // キーコード取得欄のイベントをセットする
      // この入力欄でキー入力したキー名とキーコードが入力欄へセットされ、
      // サーバへ送信される
      var keyRepository = "project/customize/key/eventhandler/@key" + no;
      var keyCodeRepository = this.KEYCODE_PREFIX + no;
      this.setKeyWidgetOnKeydownEvent(keyRepository, keyCodeRepository, ignoreKeys);

      // スクリプトのテンプレート選択リストボックスのイベントをセットする
      var scriptTemplateRepository = "project/customize/key/eventhandler/@scriptTemplate" + no;
      var scriptRepository = "project/customize/key/eventhandler/@script" + no;
      this.setScriptTemplateWidgetOnChangeEvent(scriptTemplateRepository, scriptRepository);
    },

    setKeyWidgetOnKeydownEvent: function(keyRepository, keyCodeRepository, ignoreKeys) {
      var keyWidget = this.projectEnvironment.widgets[keyRepository].widget;
      var keyCodeWidget = this.projectEnvironment.widgets[keyCodeRepository].widget;
      keyWidget.on("keydown", function(event) {
        // Backspace, Delete キー押下時はキー:コード欄を空にする
        if (event.keyCode === 8 || event.keyCode === 46) {
          keyCodeWidget.set("value", "");
          return;
        }
        if (ignoreKeys[event.keyCode]) {
          return;
        }
        baseEvent.stop(event);
        var keyAndCode = event.key + ":" + event.keyCode;
        keyCodeWidget.set("value", keyAndCode);
      });
    },

    setScriptTemplateWidgetOnChangeEvent: function(scriptTemplateRepository, scriptRepository) {
      var scriptTemplateWidget = this.projectEnvironment.widgets[scriptTemplateRepository].widget;
      var scriptWidget = this.projectEnvironment.widgets[scriptRepository].widget;
      scriptTemplateWidget.on("change", function(value) {
        var script = "";
        var readOnly = true;
        if (value === "openMainMenu") {
          script = 'window.openLocation("mainMenu.do");';
        } else if (value === "openPortal") {
          script = 'window.setMainMenuSelectedTabId("menuTab0");\nwindow.openLocation("mainMenu.do");';
        } else if (value === "openSubMenu") {
          script = 'var jfcBtnMenu = registry.byId("jfcBtnMenu");\nif (!jfcBtnMenu) {\n  return;\n}\njfcBtnMenu.onClick();';
        } else if (value === "openMultiSession") {
          script = 'var opennewtabMenuItem = registry.byId("opennewtabMenuItem");\nif (!opennewtabMenuItem) {\n  return;\n}\nopennewtabMenuItem.onClick();';
        } else if (value === "logoff") {
          script = 'window.openLocation("logoff.do");';
        } else {
          script = "";
          readOnly = false;
        }
        scriptWidget.set("value", script);
        scriptWidget.set("readOnly", readOnly);
      });
      var value = scriptTemplateWidget.get("value");
      if (value === " ") {
        return;
      }
      scriptWidget.set("readOnly", true);
    },

    displayKeyCodeSetting: function(no, display) {
      if (no === 1 && display === "none") {
        return;
      }
      var id = "jfcProjectCustomizeKeySetting" + no;
      var that = this;
      domStyle.set(id, "display", display);
    },

    // プロジェクト環境の内容部はタブを表示する
    setContent: function() {
      this.tabs = [];
      // タブコンテナ設定
      var tabContainerBase = domConstruct.create("div", {style: "height: 100%;"}, this.paneId);
      domConstruct.create("div", {id: "projectEnvironmentTabContainer"}, tabContainerBase);
      var tabContainer = new TabContainer({
        style: "width: 100%; height: 100%;"
      }, "projectEnvironmentTabContainer");
      // 各タブ設定
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_ProjectPane, //プロジェクト,
        href: "wb/paneTemplate/projectEnvironment/project.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        style: "background: #fcfcfc",
        preload: true
      }));
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_DatabasePane, //データベース,
        href: "wb/paneTemplate/projectEnvironment/database.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        style: "background: #fcfcfc",
        preload: true
      }));
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_ServerPane, //サーバ,
        href: "wb/paneTemplate/projectEnvironment/server.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        style: "background: #fcfcfc",
        preload: true
      }));
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_MailPane, //メール,
        href: "wb/paneTemplate/projectEnvironment/mail.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        style: "background: #fcfcfc",
        preload: true
      }));
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_InitPane, //アプリケーション,
        href: "wb/paneTemplate/projectEnvironment/init.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        style: "background: #fcfcfc",
        preload: true
      }));
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_CustomizePane, // カスタマイズ,
        href: "wb/paneTemplate/projectEnvironment/customize.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        style: "background: #fcfcfc",
        preload: true
      }));
      this.tabs.push(new ContentPane({
        title: wagbyDesigner.jfcRes.LABEL_TAB_MessagePane, //メッセージ,
        href: "wb/paneTemplate/projectEnvironment/message.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        style: "background: #fcfcfc",
        preload: true
      }));
      var that = this;
      // 各タブの内容をタブコンテナへ追加
      for (var i=0,l=this.tabs.length; i<l; i++) {
        var tab = this.tabs[i];
        tab.on("Load", function() {
          that.tabContentLoaded(this.title);
        });
        tabContainer.addChild(tab);
      }
      tabContainer.startup();
      this.tabCount = tabContainer.getChildren().length;
    },

    tabContentLoaded: function(title) {
      // 全タブのロード完了後にリソース設定の関数が実行されるようにする
      this.loadedTabContents.push(title);
      if (this.loadedTabContents.length === this.tabCount) {
        this.onPaneLoaded();
      }
    },

    setWidgetEvent: function() {
      this.setDatabaseWidgetEvent();
      this.setDatabaseUrlWidgetEvent();
      this.setAuthenticationTypeWidgetEvent();
    },

    setDatabaseWidgetEvent: function() {
      var that = this;
      // メインデータベース設定
      var databaseWidget = this.projectEnvironment.widgets["project/environment/database/@database"].widget;
      databaseWidget.on("change", this.getDatabaseWidgetEventFunc("project/environment/database", ""));
      // サブデータベース設定
      for (var i=this.SUBDATABASE_REPOSITORY_START_NUM,l=this.SUBDATABASE_REPOSITORY_COUNT; i<l; i++) {
        databaseWidget = this.projectEnvironment.widgets["project/environment/subdatabase/database/@database" + i].widget;
        if (!databaseWidget) {
          continue;
        }
        databaseWidget.on("change", this.getDatabaseWidgetEventFunc("project/environment/subdatabase/database", i));
      }
    },

    getDatabaseWidgetEventFunc: function(repositoryKeyPrefix, num) {
      // 接続URL
      var databaseUrlWidget = this.projectEnvironment.widgets[repositoryKeyPrefix + "/@url" + num].widget;
      // ユーザ名
      var userWidget = this.projectEnvironment.widgets[repositoryKeyPrefix + "/@user" + num].widget;
      // パスワード
      var passwordWidget = this.projectEnvironment.widgets[repositoryKeyPrefix + "/@password" + num].widget;
      // 識別子の区切り文字
      var quoteidtypeWidget = this.projectEnvironment.widgets[repositoryKeyPrefix + "/@quoteidtype" + num].widget;
      var databaseRepository = repositoryKeyPrefix + "/@database" + num;

      var that = this;
      return function(value) {
        databaseUrlWidget.set("disabled", true);
        databaseUrlWidget.set("value", that.getDatabaseUrl(databaseRepository, value));
        userWidget.set("disabled", false);
        passwordWidget.set("disabled", false);
        // 外部データベースの場合、識別子の区切り文字を入力可にする
        var quoteidtypeWidgetDisabled = false;
        if (value === "hsqldb") {
          userWidget.set("value", "");
          userWidget.set("disabled", true);
          passwordWidget.set("value", "");
          passwordWidget.set("disabled", true);
          // HSQLDB の場合、「区切り文字を指定しない」を設定するとテーブル作成時にエラーとなるため、
          // 識別子の区切り文字を空欄にして入力不可にする
          quoteidtypeWidget.set("value", " ");
          quoteidtypeWidgetDisabled = true;
        } else {
          // HSQLDB 以外の場合、接続 URL 入力欄を入力可とする
          databaseUrlWidget.set("disabled", false);
        }
        quoteidtypeWidget.set("disabled", quoteidtypeWidgetDisabled);
        // サブデータベース設定変更時は「サブデータベース」設定の選択肢を更新する
        if (repositoryKeyPrefix.indexOf("subdatabase") !== -1) {
          that.setValidSubDatabaseSettingOption();
        }
      };
    },

    setDatabaseUrlWidgetEvent: function() {
      var that = this;
      // メインデータベース接続URL設定
      var databaseUrlWidget = this.projectEnvironment.widgets["project/environment/database/@url"].widget;
      databaseUrlWidget.on("change", this.getDatabaseUrlWidgetEventFunc("project/environment/database", ""));
      // サブデータベース接続URL設定
      for (var i=this.SUBDATABASE_REPOSITORY_START_NUM,l=this.SUBDATABASE_REPOSITORY_COUNT; i<l; i++) {
        databaseUrlWidget = this.projectEnvironment.widgets["project/environment/subdatabase/database/@url" + i].widget;
        if (!databaseUrlWidget) {
          continue;
        }
        databaseUrlWidget.on("change", this.getDatabaseUrlWidgetEventFunc("project/environment/subdatabase/database", i));
      }
    },

    getDatabaseUrlWidgetEventFunc: function(repositoryKeyPrefix, num) {
      var repositoryKey = repositoryKeyPrefix + "/@database" + num;
      // データベース設定
      var databaseWidget = this.projectEnvironment.widgets[repositoryKey].widget;
  
      var that = this;
      return function(value) {
        that.setDbSettingsUrl(repositoryKey, databaseWidget.get("value"), value);
      };
    },

    setAuthenticationTypeWidgetEvent: function() {
      var that = this;
      var authenticationTypeWidget = this.projectEnvironment.widgets["project/environment/security/@authenticationType"].widget;
      var authenticationTypeWidgetOnChangeFunc = this.getAuthenticationTypeWidgetOnChangeFunc();
      authenticationTypeWidgetOnChangeFunc(authenticationTypeWidget.get("value"));
      authenticationTypeWidget.on("change", authenticationTypeWidgetOnChangeFunc);
    },

    getAuthenticationTypeWidgetOnChangeFunc: function() {
      var that = this;
      return function(value) {
        if (!value || value === " ") {
          that.disableAuthenticationSettings();
        } else if (value === "LDAP") {
          that.disableADSettings();
          that.enableLDAPSettings();
        } else if (value === "ActiveDirectory") {
          that.disableLDAPSettings();
          that.enableADSettings();
        }
      }
    },

    disableLDAPSettings: function() {
      this.setWidgetsDisable(this.getLDAPSettingRepositoryKeys(), true);
      this.projectEnvironment.widgets["project/environment/security/param/ldap/url"].widget.set("value", "");
    },

    enableLDAPSettings: function() {
      this.setWidgetsDisable(this.getLDAPSettingRepositoryKeys(), false);
    },

    getLDAPSettingRepositoryKeys: function() {
      return [
        "project/environment/security/param/ldap/url",
        "project/environment/security/param/ldap/userSearchFilter",
        "project/environment/security/param/ldap/userSearchBase",
        "project/environment/security/param/ldap/managerDn",
        "project/environment/security/param/ldap/managerPassword",
        "project/environment/security/param/useJdbcAuthentication"
      ];
    },

    disableADSettings: function() {
      this.setWidgetsDisable(this.getADSettingRepositoryKeys(), true);
      this.projectEnvironment.widgets["project/environment/security/param/activeDirectory/url"].widget.set("value", "");
    },

    enableADSettings: function() {
      this.setWidgetsDisable(this.getADSettingRepositoryKeys(), false);
    },

    getADSettingRepositoryKeys: function() {
      return [
        "project/environment/security/param/activeDirectory/url",
        "project/environment/security/param/activeDirectory/domain",
        "project/environment/security/param/activeDirectory/searchFilter"
      ];
    },

    disableAuthenticationSettings: function() {
      this.disableADSettings();
      this.disableLDAPSettings();
    },

    setWidgetsDisable: function(repositoryKeys, disabled) {
      for (var i=0,l=repositoryKeys.length; i<l; i++) {
        var widget = this.projectEnvironment.widgets[repositoryKeys[i]].widget;
        if (!widget) {
          continue;
        }
        widget.set("disabled", disabled);
      }
    },

    show: function() {
      this.setAllDatabaseUrlSettings();
      this.inherited(arguments);
    },

    setAllDatabaseUrlSettings: function() {
      // メインデータベースの接続URL設定
      this.setDbSettingsUrl(
        "project/environment/database/@database",
        this.projectEnvironment.repositoryItems["project/environment/database/@database"].value,
        this.projectEnvironment.repositoryItems["project/environment/database/@url"].value
      );
      // サブデータベースの接続URL設定
      for (var i=this.SUBDATABASE_REPOSITORY_START_NUM,l=this.SUBDATABASE_REPOSITORY_COUNT; i<l; i++) {
        var repositoryKeyPrefix = "project/environment/subdatabase/database";
        this.setDbSettingsUrl(
          repositoryKeyPrefix + "/@database" + i,
          this.projectEnvironment.repositoryItems[repositoryKeyPrefix + "/@database" + i].value,
          this.projectEnvironment.repositoryItems[repositoryKeyPrefix + "/@url" + i].value
        );
      }
    },

    getDatabaseUrl: function(repositoryKey, database) {
      if (!database || database === " ") {
        return "";
      }
      var databaseSetting = this.databaseSettings[repositoryKey];
      var url = databaseSetting[database].url;
      if (!url) {
        return "";
      }
      return url;
    },

    setDbSettingsUrl: function(repositoryKey, database, url) {
      if (!database || database === "" || !url || url === "") {
        return;
      }
      this.databaseSettings[repositoryKey][database].url = url;
    },

    initDbSettings: function() {
      // メインデータベースの接続URL設定
      this.databaseSettings = {};
      this.databaseSettings["project/environment/database/@database"] = {};
      lang.mixin(this.databaseSettings["project/environment/database/@database"], lang.clone(this.defaultDBSettings));
      // サブデータベースの接続URL設定
      for (var i=this.SUBDATABASE_REPOSITORY_START_NUM,l=this.SUBDATABASE_REPOSITORY_COUNT; i<l; i++) {
        var databaseRepositoryKey = "project/environment/subdatabase/database/@database" + i;
        this.databaseSettings[databaseRepositoryKey] = {};
        lang.mixin(this.databaseSettings[databaseRepositoryKey], lang.clone(this.defaultDBSettings));
      }
    },

    getProjectId: function() {
      return this.projectEnvironment.repositoryItems["project/@id"].value;
    },

    getTomcatHttpPort: function() {
      return this.projectEnvironment.repositoryItems["project/tomcat/@tomcathttpport"].value;
    },

    setValidSubDatabaseSettingOption: function() {
      this.validSubDatabaseSettingOption = [{value: " ", label: ""}];
      var subDatabaseNum = 0;
      // サブデータベースの接続URL設定
      for (var i=this.SUBDATABASE_REPOSITORY_START_NUM,l=this.SUBDATABASE_REPOSITORY_COUNT; i<l; i++) {
        subDatabaseNum++;
        var repositoryKey = "project/environment/subdatabase/database/@database" + i;
        var databaseWidgetItem = this.projectEnvironment.widgets[repositoryKey];
        if (!databaseWidgetItem) {
          continue;
        }
        var database = databaseWidgetItem.widget && databaseWidgetItem.widget.get("value") ||
          databaseWidgetItem.value;
        if (!database || database === " ") {
          continue;
        }
        this.validSubDatabaseSettingOption.push({
          value: "database" + i,
          label: wagbyDesigner.jfcRes.OPTION_SUBDATABASE_PREFIX + subDatabaseNum + "(" + database + ")" // サブデータベース1(MySQL)
        });
      }
    },

    getValidSubDatabaseSettingOption: function() {
      return this.validSubDatabaseSettingOption;
    },

    getDatabaseSettings: function() {
      var that = this;
      request("service/project/dbConfig", {
        "handleAs": "json"
      }).then(function(data) {
        if (data && data.length > 0) {
          var databaseOptions = [];
          for (var i=0,l=data.length; i<l; i++) {
            var dbConfig = data[i];
            that.defaultDBSettings[dbConfig.value] = {
              "label": dbConfig.label,
              "url": dbConfig.url,
              "port": "",
              "value": dbConfig.value,
            };
            databaseOptions.push({
              "label": dbConfig.label,
              "value": dbConfig.value,
            });
          }
          that.setDatabaseOptions(databaseOptions);
          that.initDbSettings();
        }
      });
    },

    setDatabaseOptions: function(databaseOptions) {
      // メインデータベースの選択肢
      var mainDatabaseSelectWidgetItem = this.projectEnvironment.widgets["project/environment/database/@database"];
      mainDatabaseSelectWidgetItem.widget.set("options", lang.clone(databaseOptions));
      mainDatabaseSelectWidgetItem.setValue(mainDatabaseSelectWidgetItem.value);
      if (mainDatabaseSelectWidgetItem.value === "hsqldb") {
        // HSQLDB の場合、接続URL、ユーザ名、パスワードを入力不可とする
        this.projectEnvironment.widgets["project/environment/database/@url"].widget.set("disabled", true);
        this.projectEnvironment.widgets["project/environment/database/@user"].widget.set("disabled", true);
        this.projectEnvironment.widgets["project/environment/database/@password"].widget.set("disabled", true);
      }
      // サブデータベースの選択肢からは HSQLDB の設定を除く
      databaseOptions = this.removeHSQLDB(databaseOptions);
      // サブデータベースの選択肢
      var subDatabaseRepositoryKeyPrefix = "project/environment/subdatabase/database/@database";
      for (var i=this.SUBDATABASE_REPOSITORY_START_NUM,l=this.SUBDATABASE_REPOSITORY_START_NUM + this.MAX_SUBDATABASE_SETTINGS; i<l; i++) {
        var subDatabaseSelectWidgetItem = this.projectEnvironment.widgets[subDatabaseRepositoryKeyPrefix + i];
        var subDatabaseOptions = [{label: "", value: " "}].concat(lang.clone(databaseOptions));
        subDatabaseSelectWidgetItem.widget.set("options", subDatabaseOptions);
        subDatabaseSelectWidgetItem.setValue(subDatabaseSelectWidgetItem.value);
      }
    },

    removeHSQLDB: function(databaseOptions) {
      for (var i=0,l=databaseOptions.length; i<l; i++) {
        if (databaseOptions[i].value === "hsqldb") {
          databaseOptions.splice(i, 1);
          return databaseOptions;
        }
      }
      return databaseOptions;
    },

    isMessageQueueEnabled: function() {
      return this.projectEnvironment.repositoryItems["project/environment/messageQueue/@enableJob"].value === wagbyDesigner.jfcRes.OPTION_CHECKED ||
        this.projectEnvironment.repositoryItems["project/environment/messageQueue/@enableJob_activemq"].value === wagbyDesigner.jfcRes.OPTION_CHECKED;
    },

  });
});
