/**
 * ユーザ定義設定ダイアログ
 */
define([
  "dojo/_base/declare",
  "dojo/_base/lang",
  "dojo/dom",
  "dojo/dom-construct",
  "dojo/dom-style",
  "dojo/json",
  "dojo/store/Memory",
  "dijit/form/Button",
  "dijit/form/ComboBox",
  "dijit/form/Select",
  "dijit/layout/ContentPane",
  "dijit/registry",
  "gridx/Grid",
  "gridx/core/model/cache/Sync",
  "gridx/modules/extendedSelect/Row",
  "gridx/modules/CellWidget",
  "gridx/modules/Header",
  "gridx/modules/HeaderMenu",
  "gridx/modules/IndirectSelect",
  "gridx/modules/RowHeader",
  "gridx/modules/SingleSort",
  "wb/ModelitemDetailRepository",
  "wb/Dialog",
  "wb/ModelitemDetailWidget"
], function(
  declare,
  lang,
  dom,
  domConstruct,
  domStyle,
  JSON,
  Memory,
  Button,
  ComboBox,
  Select,
  ContentPane,
  registry,
  Grid,
  Cache, 
  SelectRow,
  CellWidget,
  Header,
  HeaderMenu,
  IndirectSelect,
  RowHeader,
  SingleSort,
  ModelitemDetailRepository,
  Dialog,
  ModelitemDetailWidget
) {
  return declare([Dialog, ModelitemDetailRepository], {
    constructor: function(initValue) {
      this.jfcRes = initValue.jfcRes;
      this.modelitemDetailRepository = this.getModelitemDetailWidgetOptions(initValue.jfcRes);
      this.addSamePrefixRepository();
    },

    createWidget: function(key) {
      return new ModelitemDetailWidget({
        key: key,
        container: this
      });
    },

    addSamePrefixRepository: function() {
      for (var repositoryKey in this.modelitemDetailSamePrefixRepositoryKeys) {
        this.modelitemDetailRepository[repositoryKey] = wagbyDesigner.widgetDefault.getOption(repositoryKey);
      }
    },

    setContent: function() {
      this.content = new ContentPane({
        href: "wb/paneContentTemplate/model/userDefinedItemDialog.html?" + wagbyDesigner.RELEASE_TIMESTAMP,
        preload: true
      }, this.contentId);
      this.inherited(arguments);
      this.dialog.okButton.set("label", wagbyDesigner.jfcRes.LABEL_BUTTON_SAVE);
      this.dialog.cancelButton.set("label", wagbyDesigner.jfcRes.LABEL_BUTTON_CLOSE);
    },

    onDialogContentLoaded: function() {
      this.inherited(arguments);
      // ユーザ定義設定一覧の表示サイズを調整する
      this.initGridSize();
      // ユーザ定義設定一覧を作成する
      this.createListGrid();
      var that = this;
      this.deleteConfirmDialog = wagbyDesigner.createConfirmDialog({
        message: this.jfcRes.MSG_CONFIRM_USERDEFINEDITEM_DIALOG_001,
        onExecute: function() {
          that.deleteModelListStore();
        },
        onCancel: function() {}
      });
      this.setEvents();
    },

    initGridSize: function() {
      // モデル一覧の表示サイズを画面に合わせる
      var dialog = this.dialog.domNode;
      var height = (domStyle.get(dialog, "height") - 160) + "px";
      var width = (domStyle.get(dialog, "width") - 45) + "px";
      domStyle.set("jfcUserDefinedItemGrid", {
        'height': height,
        'width': width,
        'padding': "0 0 5px 0"
      });
    },

    setEvents: function() {
      var that = this;
      // 追加ボタン
      jfcAddUserDefinedSettingItem.on("click", function() {
        that.addModelListStore();
      });
      // 削除ボタン
      jfcDeleteUserDefinedSettingItem.on("click", function() {
        var deleteIds = that.grid.model.getMarkedIds();
        if (!deleteIds || deleteIds.length === 0) {
          return;
        }
        that.deleteConfirmDialog.show();
      });
      this.dialog.okButton.on("click", function() {
        that.sendUserDefinedItemSettingData();
      });
    },

    addModelListStore: function() {
      var storeData = this.modelListStore.query();
      var id = storeData.total + 1;
      var no = id + "";
      this.modelListStore.add({
        id: id,
        no: no,
        'model/userDefinedItem/@id': id,
        'model/userDefinedItem/@category': "",
        'model/userDefinedItem/@repositoryKey': "",
        'model/userDefinedItem/@repositoryValue': ""
      });
    },

    deleteModelListStore: function() {
      var deleteIds = this.grid.model.getMarkedIds();
      for (var i=0,l=deleteIds.length; i<l; i++) {
        var id = parseInt(deleteIds[i], 10);
        this.modelListStore.remove(id);
      }
    },

    setScreenResource: function() {
      this.inherited(arguments);
      jfcAddUserDefinedSettingItem.set('label', this.jfcRes.LABEL_BUTTON_ADD);
      jfcDeleteUserDefinedSettingItem.set('label', this.jfcRes.LABEL_BUTTON_DELETE);
    },

    createListGrid: function() {
      var that = this;
      this.modelListStore = new Memory();
      this.grid = new Grid({
        store: that.modelListStore,
        cacheClass: Cache,
        structure: [
          { id: 'no', field: 'no', name: that.jfcRes.LABEL_MODEL_NO, width: '5%', style: 'text-align: center' },
          { id: 'model/userDefinedItem/@category', field: 'model/userDefinedItem/@category', name: that.jfcRes.LABEL_MODELITEM_BULKUPDATE_CATEGORY, width: '14%',
            widgetsInCell: true,
            initializeCellWidget: function(cellWidget, column) {
              // 以前の入力欄が残っている場合があるので、事前に削除する
              if (cellWidget.cell.column._backupWidgets) {
                var backupWidgets = cellWidget.cell.column._backupWidgets;
                for (var i=0,l=backupWidgets.length; i<l; i++) {
                  if (backupWidgets[i].widget) {
                    backupWidgets[i].widget.destroyRecursive();
                  }
                }
              }
              that.createCategorySelector(cellWidget, cellWidget.cell.row.index());
            },
            uninitializeCellWidget: function(cellWidget, column) {
              if (cellWidget.widget) {
                cellWidget.widget.destroyRecursive();
                cellWidget.widget = null;
              }
            }
          },
          { id: 'model/userDefinedItem/@repositoryKey', field: 'model/userDefinedItem/@repositoryKey', name: that.jfcRes.LABEL_MODELITEM_BULKUPDATE_REPOSITORY_KEY, width: '51%',
            widgetsInCell: true,
            initializeCellWidget: function(cellWidget, column) {
              that.createRepositoryKeyWidget(cellWidget, cellWidget.cell.row.index());
            },
            uninitializeCellWidget: function(cellWidget, column) {
              if (cellWidget.widget) {
                cellWidget.widget.destroyRecursive();
                cellWidget.widget = null;
              }
            }
          },
          { id: 'model/userDefinedItem/@repositoryValue', field: 'model/userDefinedItem/@repositoryValue', name: that.jfcRes.LABEL_MODELITEM_BULKUPDATE_HEADER3, width: '30%',
            widgetsInCell: true,
            initializeCellWidget: function(cellWidget, column) {
              var storeData = that.modelListStore.get(cellWidget.cell.row.id) || {};
              var repositoryKey = storeData["model/userDefinedItem/@repositoryKey"];
              if (!repositoryKey) {
                return;
              }
              that.createRepositoryValueWidget(repositoryKey, cellWidget);
            },
            uninitializeCellWidget: function(cellWidget, column) {
              if (cellWidget.widget) {
                cellWidget.widget.destroyRecursive();
                cellWidget.widget = null;
              }
            }
          }
		],
        selectRowTriggerOnCell: true,
        modules: [
          CellWidget,
          Header,
          HeaderMenu,
          IndirectSelect,
          RowHeader,
          SelectRow,
          SingleSort
        ]
      });
      this.grid.placeAt("jfcUserDefinedItemGrid");
      this.grid.startup();
    },

    getSendDataFunc: function(cellWidget, cell) {
      var that = this;
      return function(e) {
        var id = cell.row.id;
        var modelData = that.modelListStore.get(id);
        var modelId = modelData["model/@id"];
        // チェックボックスがチェックされている時の送信データは○とする
        var sendValue = "";
        if (e.target.checked) {
          sendValue = e.target.value;
        }
        var sendData = {};
        sendData[cell.column.id] = sendValue;

        cellWidget.chkBuild.set("disabled", true);
        wagbyDesigner.sendRequest({
          url: "service/model/" + modelId,
          options: {
            data: sendData,
            handleAs: "json",
            method: "PUT"
          }
        }, function(data) {
          cellWidget.chkBuild.set("disabled", false);
        }, function(data) {
          cellWidget.chkBuild.set("disabled", false);
        });
      };
    },

    createModelList: function(modelData) {
      // モデル一覧のデータオブジェクトを更新する
      this.updateModelListStore(modelData);
    },

    updateModelListStore: function(modelData) {
      this.clearGridStoreData();
      var modelListStoreData = [];
      for (var i=0,l=modelData.length; i<l; i++) {
        var modelListData = modelData[i];
        var id = parseInt(modelListData['model/userDefinedItem/@id'], 10);
        modelListStoreData.push({
          id: id,
          no: (i + 1) + "",
          'model/userDefinedItem/@id': id,
          'model/userDefinedItem/@category': modelListData['model/userDefinedItem/@category'],
          'model/userDefinedItem/@repositoryKey': modelListData['model/userDefinedItem/@repositoryKey'],
          'model/userDefinedItem/@repositoryValue': modelListData['model/userDefinedItem/@repositoryValue'],
        });
      }
      this.grid.model.store.setData(modelListStoreData);
      this.grid.body.refresh()
    },

    getRepositoryItem: function(key, index) {
    },

    getSelectedModelIds: function() {
      var modelIds = [];
      var selectedRowNumbers = this.grid.select.row.getSelected();
      for (var i=0,l=selectedRowNumbers.length; i<l; i++) {
        var modelData = this.modelListStore.get(selectedRowNumbers[i]);
        if (modelData) {
          modelIds.push(modelData['model/userDefinedItem/@id']);
        }
      }
      return modelIds;
    },

    // リポジトリカテゴリリストボックス生成
    createCategorySelector: function(cellWidget, rowIndex) {
      if (cellWidget.widget) {
        return;
      }
      var storeData = this.modelListStore.get(cellWidget.cell.row.id) || {};
      var widget = registry.byId("model_userDefinedItem_category" + rowIndex);
      if (widget) {
        widget.destroyRecursive();
      }
      var select = new Select({
        id: "model_userDefinedItem_category" + rowIndex,
        style: "width: 99%;",
        options: wagbyDesigner.getModelitemDetailCategoryOptions(),
        value: storeData[cellWidget.cell.column.id] || " ",
        onChange: function(value) {
          storeData[cellWidget.cell.column.id] = value;
          if (value === " ") {
            value = undefined;
          }
          var widget = registry.byId("model_userDefinedItem_repositoryKey" + rowIndex);
          if (widget) {
            widget.set("value", "");
            widget.query.category = value || /.*/;
          }
        }
      }).placeAt(cellWidget.domNode);
      cellWidget.widget = select;
    },

    getCategoryValue: function(rowId) {
      var category;
      var storeData = this.modelListStore.get(rowId);
      if (storeData) {
        category = storeData["model/userDefinedItem/@category"];
      } else {
        category = /.*/;
      }
      return {
        category: category
      }
    },

    // リポジトリキー一覧コンボボックス生成
    createRepositoryKeyWidget: function(cellWidget, rowIndex) {
      if (cellWidget.widget) {
        return;
      }
      var storeData = this.modelListStore.get(cellWidget.cell.row.id) || {};
      var widget = registry.byId("model_userDefinedItem_repositoryKey" + rowIndex);
      if (widget) {
        widget.destroyRecursive();
      }
      // リポジトリキーデータストア
      var modelitemDetailRepositoryStore = new Memory();
      this.createModelitemDetailRepositoryStore(modelitemDetailRepositoryStore);
      var that = this;
      var comboBox = new ComboBox({
        id: "model_userDefinedItem_repositoryKey" + rowIndex,
        autoComplete: true,
        maxHeight: "-1",
        query: that.getCategoryValue(cellWidget.cell.row.id),
        queryExpr: "*${0}*",
        required: true,
        searchAttr: "repositoryKeyLabel",
        searchDelay: 400,
        selectOnClick: true,
        store: modelitemDetailRepositoryStore,
        style: "width: 99%;",
        value: storeData[cellWidget.cell.column.id] || "",
        onChange: function(value) {
          // 変更したリポジトリキーを保存する
          storeData[cellWidget.cell.column.id] = value;
          // リポジトリキー変更時にリポジトリキーに対応する値設定入力欄を生成する
          var repositoryValueCellWidget = that.getRepositoryValueCellWidget(cellWidget, rowIndex);
          if (!repositoryValueCellWidget) {
            return;
          }
          // リポジトリキー変更前の値設定入力欄がある場合は削除する
          if (repositoryValueCellWidget.widget) {
            repositoryValueCellWidget.widget.destroyRecursive();
            repositoryValueCellWidget.widget = null;
            // リポジトリキーを変更した時に設定値をクリアする
            storeData["model/userDefinedItem/@repositoryValue"] = "";
          }
          var repositoryKey = that.getRepositoryKey(value);
          that.createRepositoryValueWidget(repositoryKey, repositoryValueCellWidget);
        }
      }).placeAt(cellWidget.domNode);
      cellWidget.widget = comboBox;
    },

    // 値設定入力欄を配置するグリッドのセルを返す
    getRepositoryValueCellWidget: function(repositoryKeyCellWidget, rowIndex) {
      var repositoryValueCellWidget;
      try {
        repositoryValueCellWidget =
          repositoryKeyCellWidget.cell.column.grid._columns[3]._cellWidgets[repositoryKeyCellWidget.cell.row.id];
      } catch(e) {
        console.warn(e);
      }
      return repositoryValueCellWidget;
    },

    createRepositoryValueWidget: function(repositoryKey, cellWidget) {
      if (cellWidget.widget) {
        return;
      }
      var widgetItem = this.createWidgetItem(repositoryKey);
      if (!widgetItem) {
        return;
      }
      widgetItem.getKey = function() {
        return repositoryKey + cellWidget.cell.row.index();
      };
      var storeData = this.modelListStore.get(cellWidget.cell.row.id) || {};
      widgetItem.widget.placeAt(cellWidget.domNode);
      var value = storeData[cellWidget.cell.column.id] || "";
      if (widgetItem.widget.baseClass.indexOf("dijitNumberTextBox") !== -1) {
        value = parseInt(value, 10);
      }
      if (widgetItem.widgetOptions["type"] === "checkbox") {
        widgetItem.widget.set("checked", value !== "");
      } else {
        widgetItem.widget.set("value", value);
      }
      widgetItem.widget.on("change", function(value) {
        // チェックボックスの場合、変更値が true, false となるため、
        // サーバへ送信する値を変更する
        var widgetValue = widgetItem.getSendValue(widgetItem, value);
        storeData[cellWidget.cell.column.id] = widgetValue;
      });
      cellWidget.widget = widgetItem.widget;
    },

    // リポジトリ値のウィジェットを表示する
    refreshRepositoryValueWidget: function(rowIndex, storeDataItem) {
      // 選択したリポジトリキーでデータストアのリポジトリデータを更新する
      this.updateModelitemListStore(rowIndex, storeDataItem);
    },

    getRepositoryKey: function(selectedValue) {
      // リポジトリキーのコンボボックス欄には「:」区切りで説明が付加されているため、
      // 区切り文字と説明を除いたリポジトリキーを返す
      var repositoryKey;
      if (selectedValue.indexOf(":") === -1) {
        repositoryKey = selectedValue;
      } else {
        repositoryKey = selectedValue.split(":")[0];
      }
      return repositoryKey;
    },

    // 項目詳細定義の設定欄を取得する
    createWidgetItem: function(selectedValue) {
      var repositoryKey = this.getRepositoryKey(selectedValue);
      // 設定欄の設定情報(入力タイプ、選択肢の値等)を取得する
      var widgetOptions = wagbyDesigner.widgetDefault.getOption(repositoryKey);
      widgetOptions.event = "none"; // onChange イベントでのサーバへの値送信は無し
      widgetOptions.place = "none"; // 配置先は Grid となるので、配置先は設定しない
      var widgetItem = new ModelitemDetailWidget({
        key: repositoryKey,
        container: this,
        widgetOptions: widgetOptions
      });
      return widgetItem;
    },

    updateModelitemListStore: function(rowIndex, storeDataItem) {
      if (!storeDataItem) {
        return;
      }
      var repositoryKey = storeDataItem["model/userDefinedItem/@repositoryKey"] || undefined;
      if (!repositoryKey) {
        return;
      }
      this.clearRepositoryWidget(rowIndex);
      var storeData = this.modelListStore.get(rowIndex);
    },

    clearRepositoryWidget: function(rowIndex) {
      var storeData = this.modelListStore.get(rowIndex);
      if (!storeData) {
        return;
      }
      storeData["model/userDefinedItem/@repositoryValue"] = null;
      this.modelListStore.put(storeData);
    },

    createModelitemDetailRepositoryStore: function(modelitemDetailRepositoryStore) {
      var modelitemDetailRepositoryList = [];
      for (var key in this.modelitemDetailRepository) {
        var repository = this.modelitemDetailRepository[key];
        var label = repository.label || "";
        var repositoryKeyLabel = key + ":" + label;
        modelitemDetailRepositoryList.push({
          category: repository.category || "",
          repositoryKey: key,
          label: label,
          repositoryKeyLabel: repositoryKeyLabel,
          type: repository.type,
          options: repository.options || []
        });
      }
      modelitemDetailRepositoryList =
        modelitemDetailRepositoryList.concat(this.getBasicModelitemDetailRepository());
      modelitemDetailRepositoryStore.setData(modelitemDetailRepositoryList);
    },

    getBasicModelitemDetailRepository: function() {
      var basicModelitemDetailRepositoryList = [];
      basicModelitemDetailRepositoryList.push({
        category: "basic",
        repositoryKey: "model/modelitem/@label",
        label: this.jfcRes.LABEL_MODELITEM_NAME,
        repositoryKeyLabel: "model/modelitem/@label:" + this.jfcRes.LABEL_MODELITEM_NAME,
        type: "validationTextbox",
        options: [],
      });
      basicModelitemDetailRepositoryList.push({
        category: "basic",
        repositoryKey: "model/modelitem/@type",
        label: this.jfcRes.LABEL_MODELITEM_TYPE,
        repositoryKeyLabel: "model/modelitem/@type:" + this.jfcRes.LABEL_MODELITEM_TYPE,
        type: "validationTextbox",
        options: [],
      });
      basicModelitemDetailRepositoryList.push({
        category: "basic",
        repositoryKey: "model/modelitem/@hasConditionModel",
        label: this.jfcRes.LABEL_MODELITEM_CONDITION,
        repositoryKeyLabel: "model/modelitem/@hasConditionModel:" + this.jfcRes.LABEL_MODELITEM_CONDITION,
        type: "checkbox",
        options: [{value: this.jfcRes.OPTION_CHECKED}],
      });
      basicModelitemDetailRepositoryList.push({
        category: "basic",
        repositoryKey: "model/modelitem/@hasListModel",
        label: this.jfcRes.LABEL_MODELITEM_LIST,
        repositoryKeyLabel: "model/modelitem/@hasListModel:" + this.jfcRes.LABEL_MODELITEM_LIST,
        type: "checkbox",
        options: [{value: this.jfcRes.OPTION_CHECKED}],
      });
      return basicModelitemDetailRepositoryList;
    },

    getUserDefinedItemSetting: function(id) {
      var that = this;
      wagbyDesigner.sendRequest({
        url: "/model/userDefinedItemSetting/" + id,
        options: {
          handleAs: "json"
        }
      }, function(userDefinedItemData) {
  console.log("userDefinedItemData: ", userDefinedItemData);
        // ユーザ定義情報をダイアログへ表示する
        that.setUserDefinedItemSettingData(userDefinedItemData);
      }, function(error) {
        return that.jfcRes.MSG_ERROR_USERDEFINEDITEM_DIALOG_001; // "ユーザ定義項目情報の取得中にエラーが発生しました。"
      });
    },

    setUserDefinedItemSettingData: function(userDefinedItemData) {
      try {
        // 設定名、概要の表示
        this.widgets["model/userDefinedItemSetting/@id"].widget.set("value", userDefinedItemData["id"]);
        this.widgets["model/userDefinedItemSetting/@no"].widget.set("value", userDefinedItemData["no"]);
        this.widgets["model/userDefinedItemSetting/@description"].widget.set("value", userDefinedItemData["description"]);
        this.widgets["model/userDefinedItemSetting/@outline"].widget.set("value", userDefinedItemData["outline"]);
        // ユーザ定義情報をグリッドに表示
        var userDefinedItems = [];
        for (var i in userDefinedItemData.userDefinedItems) {
          userDefinedItems.push(userDefinedItemData.userDefinedItems[i]);
        }
        this.updateModelListStore(userDefinedItems);
      } catch(e) {
        console.error(e);
      }
    },

    getUserDefinedItemSettingData: function() {
      var that = this;
      var id = parseInt(this.widgets["model/userDefinedItemSetting/@id"].widget.get("value"), 10);
      var no = parseInt(this.widgets["model/userDefinedItemSetting/@no"].widget.get("value"), 10);
      return {
        "id": id,
        "no": no,
        "description": this.widgets["model/userDefinedItemSetting/@description"].widget.get("value"),
        "outline": this.widgets["model/userDefinedItemSetting/@outline"].widget.get("value"),
        "userDefinedItems": this.getUserDefinedItems()
      };
    },

    getUserDefinedItems: function() {
      var userDefinedItems = {};
      var storeData = this.modelListStore.query();
      var id = 1;
      for (var i=0,l=storeData.total; i<l; i++) {
        var userDefinedItem;
        userDefinedItem = lang.clone(storeData[i]);
        // id, no はサーバへ送信しない
        delete userDefinedItem["id"];
        delete userDefinedItem["no"];
        userDefinedItems[id] = userDefinedItem;
        id++;
      }
      return userDefinedItems;
    },

    sendUserDefinedItemSettingData: function() {
      var that = this;
      var content;
      var userDefinedItemSettingData = this.getUserDefinedItemSettingData();
      content = JSON.stringify(userDefinedItemSettingData);
      var url = "/model/userDefinedItemSetting/" + userDefinedItemSettingData.id;
      wagbyDesigner.sendRequest({
        url: url,
        options: {
          data: content,
          handleAs: "json",
          headers: {
            'Content-Type': "application/json; charset=utf-8"
          },
          method: "PUT"
        }
      }, function(data) {
        userDefinedItemSettingData.id = data;
        wagbyDesigner.updateUserDefinedItemSetting(userDefinedItemSettingData);
      }, function(data) {
console.error("sendUserDefinedItemSettingData error: ", data);
        if (data.response.status === 403) {
          var responseData = JSON.parse(data.response.data);
          if (responseData.status && responseData.status.length > 0 &&
              responseData.status[0].errorMessage) {
            return responseData.status[0].errorMessage;
          }
        } else {
          return that.jfcRes.MSG_ERROR_USERDEFINEDITEM_DIALOG_002; // "ユーザ定義項目情報の更新中にエラーが発生しました。"
        }
      });
    },

    show: function(id) {
      // ダイアログを開く
      this.dialog.show();
      this.showWidgets();
      if (id === 0) {
        // id が 0 の場合、新規作成
        var no = wagbyDesigner.modelListPane.userDefinedItemSettingListPaneContent.getMaxNo() + 1;
        this.setUserDefinedItemSettingData({
          'id': 0,
          'no': no,
          'description': "",
          'outline': "",
          'userDefinedItems': [{
            'model/userDefinedItem/@id': "0",
            'model/userDefinedItem/@category': "",
            'model/userDefinedItem/@repositoryKey': "",
            'model/userDefinedItem/@repositoryValue': "",
          }],
        });
      } else {
        // 更新の場合、ユーザ定義情報を取得する
        this.getUserDefinedItemSetting(id);
      }
    },

    clearGridStoreData: function() {
      this.grid.model.store.setData([]);
      this.grid.model.clearCache();
    },

    hide: function() {
      // ダイアログを閉じる時にグリッドのデータをクリアする
      this.clearGridStoreData();
      this.inherited(arguments);
    },

  });
});
