/**
 * ウィジェットクラス
 */
define([
  "dojo/_base/declare",
  "dojo/dom",
  "dojo/dom-attr",
  "dojo/dom-construct",
  "dojo/dom-style",
  "dojo/json",
  "dojo/store/Memory",
  "dojo/string",
  "dojo/topic",
  "dijit/Tooltip",
  "dijit/form/Button",
  "dijit/form/CheckBox",
  "dijit/form/ComboBox",
  "dijit/form/FilteringSelect",
  "dijit/form/NumberSpinner",
  "dijit/form/NumberTextBox",
  "dijit/form/Select",
  "dijit/form/TextBox",
  "dijit/form/ValidationTextBox",
  "dijit/registry",
  "wagby/ValidationTextarea",
], function(
  declare,
  dom,
  domAttr,
  domConstruct,
  domStyle,
  JSON,
  Memory,
  string,
  topic,
  Tooltip,
  Button,
  CheckBox,
  ComboBox,
  FilteringSelect,
  NumberSpinner,
  NumberTextBox,
  Select,
  TextBox,
  ValidationTextBox,
  registry,
  ValidationTextarea
) {
  return declare(null, {
    constructor: function(initValue) {
      if (!initValue) {
        return;
      }
      this.key = initValue.key;
      this.value = initValue.value;
      this.container = initValue.container;
      this.baseUrl = wagbyDesigner.BASEURL;
      this.widgetOptions =  initValue.widgetOptions || wagbyDesigner.widgetDefault.getOption(this.key, this.getMyCategory());
      this.widgetHtmlId = this.getWidgetHtmlId();
      this.itemName = "";
      this.setWidget();
    },

    getMyCategory: function() {
      return null;
    },

    getKey: function() {
      return this.key;
    },

    getRepositoryKey: function() {
      return this.key;
    },

    getPermitedHtmlId: function (key) {
      var permitedHtmlId = key.replace(/\/|\.|\[|\]|=|:|\-/g, "_");
      permitedHtmlId = permitedHtmlId.replace(/\"|@/g, "");
      return permitedHtmlId;
    },

    getWidgetHtmlId: function() {
      if (this.widgetOptions["place"] === "none") {
        return null;
      }
      var widgetPlaceKey;
      if (this.widgetOptions["place"]) {
        widgetPlaceKey = this.widgetOptions["place"];
      } else {
        widgetPlaceKey = this.getKey();
      }
      return this.getPermitedHtmlId(widgetPlaceKey);
    },

    getWidgetPlaceId: function() {
      if (!this.widgetHtmlId) {
        return null;
      }
      return "place_" + this.widgetHtmlId;
    },

    getWidgetLabelId: function() {
      if (!this.widgetHtmlId) {
        return null;
      }
      return "label_" + this.widgetHtmlId;
    },

    getWidget: function(widgetOptions) {
      var widget = null;
      var defaultWidth;
      if (widgetOptions["type"] === "textbox" || widgetOptions["type"] === "password" || widgetOptions["type"] === "hidden") {
        var attr = { selectOnClick: true };
        if (widgetOptions["type"] === "password" || widgetOptions["type"] === "hidden") {
          attr["type"] = widgetOptions["type"];
        }
        widget = new TextBox(attr);
        defaultWidth = "width: 99%;";
      } else if (widgetOptions["type"] === "textarea" || widgetOptions["type"] === "scriptEditor") {
        widget = new wagby.ValidationTextarea({selectOnClick: true});
        var textareaStyle = "font-size: 1em; max-height: 680px;";
        if (!widgetOptions["useFunctionDialog"]) {
          textareaStyle = textareaStyle + " width: 99%;";
        }
        widget.set("style", textareaStyle);
      } else if (widgetOptions["type"] === "validationTextbox") {
        widget = new ValidationTextBox({
          selectOnClick: true,
          required: widgetOptions.required || false
        });
        defaultWidth = "width: 99%;";
        if (widgetOptions.pattern) {
          widget.set("pattern", widgetOptions.pattern);
        }
      } else if (widgetOptions["type"] === "select") {
        widget = new Select();
        if (widgetOptions.options) {
          widget.addOption(widgetOptions.options);
        }
        defaultWidth = "width: 160px;";
        if (widgetOptions['class']) {
          widget.set('class', widgetOptions['class']);
        }
      } else if (widgetOptions["type"] === "filteringSelect") {
        widget = new FilteringSelect({
          store: new Memory(),
          queryExpr: "*${0}*",
          maxHeight: widgetOptions.maxHeight || -1,
          searchAttr: widgetOptions.searchAttr || "name",
          required: widgetOptions.required,
          selectOnClick: true
        });
        defaultWidth = "width: auto;";
      } else if (widgetOptions["type"] === "checkbox") {
        widget = new CheckBox();
      } else if (widgetOptions["type"] === "combobox") {
        var store;
        if (this.container && this.container.getStore) {
          store = this.container.getStore(this.key);
        } else {
          store = new Memory();
        }
        widget = new ComboBox({
          store: store,
          searchAttr: widgetOptions.searchAttr || "name"
        });
      } else if (widgetOptions["type"] === "numberSpinner") {
        widget = new NumberSpinner();
      } else if (widgetOptions["type"] === "numberTextbox") {
        widget = new NumberTextBox({
          selectOnClick: true,
          constraints: widgetOptions.constraints || {},
          required: widgetOptions.required
        });
      } else {
        console.error("unknown type:["+widgetOptions["type"]+"]");
        return widget;
      }
      if (widgetOptions.style || defaultWidth) {
        if (!widgetOptions.style) {
          widgetOptions.style = defaultWidth;
        } else if (widgetOptions.style.indexOf("width") !== 0) {
          var style = widget.get("style");
          if (style && style !== "" && style.charAt(style.length) !== ";") {
            style = style + ";";
          }
          style = style + widgetOptions.style;
          if (style.charAt(style.length) !== ";") {
            style += ";";
          }
          widgetOptions.style = style + defaultWidth;
        }
        widget.set("style", widgetOptions.style);
      }
      if (widgetOptions["type"] === "scriptEditor") {
        widget.on("focus", function() {
          wagbyDesigner.openScriptEditor(this, widgetOptions["mode"]);
        });
      }
      return widget;
    },

    setWidget: function() {
      if (!this.widgetOptions["type"]) {
        return;
      }
      // ラベル設定
      // var widgetLabelNode = dom.byId("placeLabel_" + this.getKey());
      // if (widgetLabelNode && this.titleResourceKey !== undefined) {
      //   widgetLabelNode.innerHTML = this.titleResourceKey;
      // }
      this.widget = this.getWidget(this.widgetOptions);
      if (!this.widget) {
        return;
      }
      this.widget.set("name", this.key);
      if (this.value) {
        this.widget.set("_onChangeActive", false);
        if (this.widgetOptions["type"] === "checkbox") {
          if (this.value === wagbyDesigner.jfcRes.OPTION_CHECKED) {
            this.widget.set("checked", true);
          } else {
            this.widget.set("checked", false);
          }
        } else {
          this.widget.set("value", this.value);
        }
        this.widget.set("_onChangeActive", true);
      }
      if (this.widgetOptions.tabIndex) {
        this.widget.set("tabIndex", this.widgetOptions.tabIndex);
      }
      if (this.widgetOptions.readonly) {
        this.widget.set("readOnly", this.widgetOptions.readonly);
        if (this.widget.get("selectOnClick")) {
          this.widget.set("selectOnClick", false);
        }
      }
      if (this.widgetOptions.disabled) {
        this.widget.set("disabled", true);
      }
      if (!(this.widgetOptions.event && this.widgetOptions.event === "none")) {
        this.setOnChangeFunc();
      }
    },

    setValue: function(value, isEventActive) {
      if (!this.widget) {
        return;
      }
      var onChangeActive = false;
      if (isEventActive) {
        onChangeActive = true;
      }
      this.widget.set("_onChangeActive", onChangeActive);
      if (this.widgetOptions.type === "checkbox") {
        if (value === wagbyDesigner.jfcRes.OPTION_CHECKED || (value && value.toLowerCase() === "true")) {
          this.widget.set("checked", true);
        } else {
          this.widget.set("checked", false);
        }
      } else {
        this.widget.set("value", value || "");
      }
      this.value = value;
      this.widget.set("_onChangeActive", true);
      // エラーの設定
      var repositoryItem = this.container.getRepositoryItem(this.key);
      if (!repositoryItem || repositoryItem.isValid()) {
        return;
      }
      this.setWidgetError({
        message: repositoryItem.errorMessage
      });
    },

    show: function() {
      if (!this.widget || this.widgetOptions["place"] === "none") {
        return;
      }
      var widgetPlace = dom.byId(this.getWidgetPlaceId());
      if (!widgetPlace) {
        return;
      }
      this.widget.placeAt(widgetPlace);
      this.setWidgetLabel();
      this.setWidgetDisplay("");
  
      if (this.widgetOptions["type"] === "checkbox" &&
           this.key !== "model/modelitem/@hasConditionModel" &&
           this.key !== "model/modelitem/@hasListModel") {
        this.setCheckBoxLabelEvent(widgetPlace);
      }
      if (this.widgetOptions["useFunctionDialog"]) {
        this.setFunctionDialogButton(widgetPlace);
      }
    },

    setCheckBoxLabelEvent: function(widgetPlace) {
      // チェックボックスの場合
      if (!widgetPlace.nextSibling || widgetPlace.nextSibling.tagName !== "LABEL") {
        return;
      }
      // 隣に label タグがある場合、チェックボックスの id をセットして
      // ラベルクリックでもチェックボックスにチェックが付くようにする
      domAttr.set(widgetPlace.nextSibling, "for", this.widget.id);
    },

    hide: function() {
      if (!this.widget || this.widgetOptions["place"] === "none") {
        return;
      }
      this.setWidgetDisplay("none");
      this.widgetOptions["label"] = "";
      this.setWidgetLabel();
    },

    setWidgetDisplay: function(display) {
      var node = this.widget.domNode;
      if (display === "none" || display === "" && domStyle.get(node, "display") === "none") {
        domStyle.set(node, "display", display);
      }
    },

    setWidgetLabel: function() {
      // ラベル設定
      if (this.widgetOptions["label"] === undefined) {
        return;
      }
      var widgetLabelPlace = dom.byId(this.getWidgetLabelId());
      if (widgetLabelPlace) {
        domAttr.set(widgetLabelPlace, "innerHTML", this.widgetOptions["label"]);
      }
    },

    destroy: function() {
      if (!this.widget || this.widgetOptions["place"] === "none") {
        return;
      }
    },

    setOnChangeFunc: function() {
      var that = this;
      this.widget.on("change", function(value) {
        that.sendData(that, value);
      })
    },

    getSendValue: function(widgetItem, value) {
      var sendValue = value;
      // チェックボックスの場合、チェック時はデフォルト値を送信する
      // 外した場合は、サーバへ空文字を送信する
      if (widgetItem.widgetOptions.type === "checkbox") {
        if (value === true) {
          sendValue = widgetItem.widgetOptions.options[0].value;
        } else if (value === false) {
          sendValue = "";
        }
      } else {
        // 文字列の場合、trim をかける
        if (typeof sendValue === "string" && sendValue) {
          sendValue = sendValue.replace(/\s+$/, "");
        } else if (typeof sendValue === "number") {
          // NaN の場合、空にする
          if (isNaN(sendValue)) {
            sendValue = "";
          }
        }
      }
      return sendValue;
    },

    successOnSendData: function(data, widgetItem, sendValue) {
      widgetItem.value = sendValue;
      var repositoryItem = widgetItem.container.getRepositoryItem(widgetItem.getRepositoryKey()) ||
        widgetItem.container.getRepositoryItem(widgetItem.key);
      if (repositoryItem) {
        repositoryItem.value = sendValue;
      }
      // 入力項目にエラーがある場合元に戻す
      widgetItem.unsetWidgetErrors(data);
      // 型変更により付随する設定値を変更する
      if (!data) {
        return;
      }
      var modifiedModelitemData;
      var updateModelitemData;
      var updateModelData;
      if (data.length && data.length > 0) {
        modifiedModelitemData = data[0].modifymodelitem;
        updateModelitemData = data[0].updateModelitemInfo;
        updateModelData = data[0].updateModelInfo;
      } else {
        modifiedModelitemData = data.modifymodelitem;
        updateModelitemData = data.updateModelitemInfo;
        updateModelData = data.updateModelInfo;
      }
      if (modifiedModelitemData && widgetItem.container.getName) {
        if (widgetItem.container.getName) {
          // モデル項目の設定値変更を行なう
          widgetItem.setModifiedModelitemData(modifiedModelitemData[widgetItem.container.getName()]);
        }
      }
      if (updateModelData) {
        if (updateModelData.modifyMapOtherModel) {
          // モデルID変更後の他モデルの設定値変更を行なう
          wagbyDesigner.modifyOtherModelData(updateModelData.modifyMapOtherModel);
        }
        // 設定変更に関連する自モデルの設定値変更を行なう
        wagbyDesigner.modifyThisModelData(updateModelData.modelId, widgetItem.getRepositoryKey(), updateModelData.modifyMap);
      }
      if (updateModelitemData) {
        if (updateModelitemData.modifyMapOtherModel) {
          // 項目ID変更後の他モデルの設定値変更を行なう
          wagbyDesigner.modifyOtherModelData(updateModelitemData.modifyMapOtherModel);
        }
        // 設定変更に関連する自モデル項目の設定値変更を行なう
        if (updateModelitemData.modifyMap) {
          wagbyDesigner.modifyThisModelitemData(updateModelitemData.modelId, updateModelitemData.modifyMap);
        }
      }
    },

    errorOnSendData: function(responseData, widgetItem, sendValue) {
      if (!responseData) {
        return;
      }
      var repositoryItem = widgetItem.container.getRepositoryItem(widgetItem.key);
      if (!repositoryItem) {
        return;
      }
      if (responseData.notsaved) {
        // サーバリポジトリに保存されていないため、値を元に戻す
        setTimeout(function() {
          widgetItem.widget.set("_onChangeActive", false);
          widgetItem.widget.set("value", repositoryItem.value);
          widgetItem.widget.set("_onChangeActive", true);
          widgetItem.unsetWidgetErrors(responseData);
        }, 1900);
      } else {
        // サーバ通信エラーでもローカルのリポジトリデータは更新しておく
        repositoryItem.value = sendValue;
      }
      this.setWidgetError({
        message: responseData.status[0].errorMessage,
        notsaved: responseData.notsaved
      });
    },

    setWidgetEnabled: function() {
      this.changeWidgetDisabled(false);
      if (this.widgetOptions.type === "checkbox") {
        this.widget.focus();
      }
    },

    setWidgetDisabled: function() {
      this.changeWidgetDisabled(true);
    },

    changeWidgetDisabled: function(disabled) {
      if (!this.widget) {
        return;
      }
      this.widget.set("disabled", disabled);
    },

    sendData: function(widgetItem, value, callBack) {
      var sendValue = this.getSendValue(widgetItem, value);
      if (sendValue === undefined) {
        return;
      }
      // 値変更無しの場合、送信しない
      var repositoryItem = widgetItem.container.getRepositoryItem(widgetItem.getRepositoryKey()) ||
        widgetItem.container.getRepositoryItem(widgetItem.key);
      if (repositoryItem && repositoryItem.value === sendValue) {
        return;
      }
      topic.publish("widget", "sendDataStart");
      // 値送信前の入力欄の活性状態を保持しておく
      var disabled = this.widget.get("disabled");
  
      var that = this;
      var content = {};
      content[widgetItem.getRepositoryKey()] = sendValue;
console.log("url:["+widgetItem.getUrl()+"], content: ", content);
      // サーバからレスポンスが返るまで設定欄を非活性化する
      this.setWidgetDisabled();
      wagbyDesigner.sendRequest({
        url: widgetItem.getUrl(),
        options: {
          data: content,
          handleAs: "json",
          method: "PUT"
        }
      }, function(data) {
        // 値送信前の設定欄が入力可能だった場合、設定欄を活性化する
        if (!disabled) {
          that.setWidgetEnabled();
        }
        that.successOnSendData(data, widgetItem, sendValue);
        if (callBack && callBack.onSuccess) {
          callBack.onSuccess();
        }
        topic.publish("widget", "sendDataEnd");
      }, function(data) {
  console.log("error data: ", data);
        // 設定欄を活性化する
        that.setWidgetEnabled();
        if (data.response && data.response.data) {
          try {
            var responseData = JSON.parse(data.response.data);
            that.errorOnSendData(responseData, widgetItem, sendValue);
          } catch(e) {
            wagbyDesigner.showDialog(wagbyDesigner.jfcRes.MSG_ERROR_SAVE_REPOSITORY_DATA); // "リポジトリデータの更新でエラーが発生しました"
          }
        }
        if (callBack && callBack.onError) {
          callBack.onError();
        }
        topic.publish("widget", "sendDataEnd");
      });
    },

    getOnChangeFunction: function() {
      var that = this;
      return function(value) {
        that.sendData(that, value);
      };
    },

    setModifiedModelitemData: function(modifiedModelitemData) {
      for (var key in modifiedModelitemData) {
        var value = modifiedModelitemData[key];
        var repositoryItem = this.container.repositoryItems[key];
        if (!repositoryItem && this.container.detail &&
              this.container.detail.repositoryItems) {
          repositoryItem = this.container.detail.repositoryItems[key];
        }
        if (repositoryItem) {
          repositoryItem.value = value;
        }
        // 入力欄の表示データを更新する
        this.updateWidgetValue(key, value);
      }
    },

    updateWidgetValue: function(key, value) {
      // 入力制御->入力フィルタの選択
      if (key !== "model/modelitem/@filter") {
        return;
      }
      if (!this.container.widgets[key]) {
        return;
      }
      var widget = this.container.widgets[key].widget;
      if (!widget) {
        return;
      }
      widget.set("value", value);
    },

    getUrl: function() {
      return this.url;
    },

    getErrorRepositoryKey: function() {
      var repositoryKey;
      // 項目の型がエラーの場合、画面上でエラーが判るように型選択リストボックスの
      // リポジトリキーを返す
      if (this.key === "model/modelitem/@type") {
        repositoryKey = "model/modelitem/@typeSimple";
      } else {
        repositoryKey = this.key;
      }
      return repositoryKey;
    },

    setWidgetError: function(error) {
      if (!(error && error.message)) {
        return;
      }
      var repositoryKey = this.getErrorRepositoryKey();
      var repositoryItem = this.container.getRepositoryItem(repositoryKey);
      if (!repositoryItem) {
        return;
      }
      var widgetItem = this.container.widgets[repositoryKey];
      if (!widgetItem) {
        return;
      }
      // エラーの入力項目欄の背景色を変更し、エラーメッセージ表示用の Tooltip を設定する
      repositoryItem.setError(error.message);
      if (widgetItem.widget.errorTooltip) {
        widgetItem.widget.errorTooltip.set("label", error.message);
        return;
      }
      // 画面右上にエラーメッセージを表示する
      wagbyDesigner.bubbleErrorMessage(error.message);
      if (widgetItem.widgetOptions["type"] === "checkbox") {
        domStyle.set(widgetItem.widget.domNode.parentElement, "padding", "3px");
      }
      domStyle.set(widgetItem.widget.domNode.parentElement, "background", "yellow");
      widgetItem.widget.errorTooltip = new Tooltip({
        connectId: widgetItem.widget.id,
        label: error.message
      });
      widgetItem.widget.focus();
      // リポジトリ更新時にエラーが発生した場合はモデル一覧データ再取得のフラグをセットする
      wagbyDesigner.isNeedRefreshModelList = true;
    },

    unsetWidgetErrors: function(data) {
      // 変更があったリポジトリの入力欄にエラーがある場合、解除する
      this.unsetWidgetError(this.getErrorRepositoryKey());
      // リポジトリの変更に付随してエラーを解除する入力欄が他にある場合、解除する
      if (!data || !data.status) {
        return;
      }
      for (var i=0,l=data.status.length; i<l; i++) {
        this.unsetWidgetError(data.status[i].key, data.status[i].modelitemName);
      }
      // エラーが解消した場合はモデル一覧データ再取得のフラグをセットする
      wagbyDesigner.isNeedRefreshModelList = true;
    },

    unsetWidgetError: function(repositoryKey, modelitemName) {
      // エラー解除対象となるリポジトリを含むコンテナを取得する
      var repositoryItemContainer = this.getRepositoryItemContainer(repositoryKey);
      if (!repositoryItemContainer) {
        return;
      }
      var repositoryItem = this.getErrorRepositoryItem(repositoryKey, repositoryItemContainer, modelitemName);
      if (!repositoryItem) {
        return;
      }
      // エラー表示の背景色とエラーメッセージ表示用の Tooltip を削除する
      repositoryItem.unsetError();
      var widgetItem = repositoryItemContainer.widgets[repositoryItem.key];
      if (!widgetItem) {
        return;
      }
      if (!widgetItem.widget.errorTooltip) {
        return;
      }
      if (widgetItem.widgetOptions["type"] === "checkbox") {
        domStyle.set(widgetItem.widget.domNode.parentElement, "padding", "0");
      }
      domStyle.set(widgetItem.widget.domNode.parentElement, "background", "");
      widgetItem.widget.errorTooltip.set("connectId", "");
      widgetItem.widget.errorTooltip.destroyRecursive();
      widgetItem.widget.errorTooltip = null;
      // モデル項目詳細ダイアログの場合、エラーを確認して「詳細」ボタンの色を変更する
      if (repositoryItemContainer.name && repositoryItemContainer.name === "modelitemDetail") {
        wagbyDesigner.modelPane.getErrorModelitems();
      } else if (repositoryItemContainer.paneId && repositoryItemContainer.paneId === "modelPane") {
        // モデル設定の場合、エラーを確認してタブの色を変更する
        wagbyDesigner.modelPane.getModelError();
      }
    },

    // リポジトリキーを含むコンテナを返す
    getRepositoryItemContainer: function(repositoryKey) {
      // 自身のコンテナ
      if (this.container.getRepositoryItem(repositoryKey)) {
        return this.container;
      }
      // モデル項目詳細ダイアログ
      if (wagbyDesigner.modelitemDetailDialog.repositoryKeys[repositoryKey]) {
        return wagbyDesigner.modelitemDetailDialog;
      }
      // モデルコンテナ(画面、メール)から取得する
      return wagbyDesigner.modelPane.getRepositoryItemContainer(repositoryKey);
    },

    getErrorRepositoryItem: function(repositoryKey, repositoryItemContainer, modelitemName) {
      // 自身のコンテナ
      var repositoryItem = this.container.getRepositoryItem(repositoryKey);
      if (!repositoryItem) {
        // モデル項目詳細ダイアログ
        repositoryItem = wagbyDesigner.modelitemDetailDialog.getRepositoryItem(repositoryKey);
      }
      if (!repositoryItem) {
        // モデルコンテナ(画面、メール)から取得する
        if (repositoryItemContainer && repositoryItemContainer.getErrorRepositoryItem) {
          repositoryItem = repositoryItemContainer.getErrorRepositoryItem(repositoryKey, modelitemName);
        }
      }
      return repositoryItem;
    },

    setFunctionDialogButton: function(widgetPlace) {
      var id = "btnFunctionDialog" + this.getWidgetHtmlId();
      var btnNode = registry.byId(id);
      if (btnNode) {
        domConstruct.place(btnNode.domNode, widgetPlace, "last");
        return;
      }
      domStyle.set(widgetPlace, "whiteSpace", "nowrap");
      if (!this.widget.style || this.widget.style.indexOf("width") === -1) {
        domStyle.set(this.widget.textbox, "width", "95%");
      }
      btnNode = domConstruct.create("button", {
        id: id,
        type: "button",
      }, widgetPlace, "last");
      var btn = new Button({
        showLabel: false,
        iconClass: "fas fa-edit",
        style: "margin-right: 5px;",
      }, btnNode.id);
      var that = this;
      btn.on("click", function() {
        wagbyDesigner.showFunctionReferenceDialog(that.widget);
      });
    },

  });
});
