/**
 * コンテナ管理
 */
define([
  "dojo/_base/declare",
  "dojo/_base/event",
  "dojo/dnd/AutoSource",
  "dojo/dom",
  "dojo/dom-attr",
  "dojo/dom-class",
  "dojo/dom-construct",
  "dojo/dom-style",
  "dojo/on",
  "dojo/query",
  "wb/layout/DetailDisplayitem",
  "wb/layout/DisplayitemGroup",
  "wb/layout/FieldsetContainer",
  "wb/layout/TitlePaneContainer",
  "wb/layout/RepeatItemContainer",
  "wb/layout/TabContainer",
  "wb/layout/WizardContainer"
], function(declare, event, AutoSource, dom, domAttr, domClass, domConstruct, domStyle, on, query, DetailDisplayitem, DisplayitemGroup, FieldsetContainer, TitlePaneContainer, RepeatItemContainer, TabContainer, WizardContainer) {
  return declare(null, {
    containers: {},
    tabNameByLayoutName: {},
    tabContainerIdByDisplayitemGroupId: {},
    groupId: 0,
    oldLayoutsNo: [],
    TAB_CONTAINER_COUNT: 0,
    REPEATITEM_CONTAINER_COUNT: 0,
    LAYOUTNAME_DELIMITER: ">",
    LAYOUTNAME_HIDDEN_PREFIX: "*",
    LAYOUT_USE_FIELDSET: "surroundedByBox",
    LAYOUT_GROUP_VERTICAL: "groupInVertical", // 縦方向にグループ化
    LAYOUT_FIELDSET: "fieldset", // グループ
    LAYOUT_TITLEPANE: "titlePane",
    LAYOUT_TAB: "tab", // タブ
    REPEATITEM_DELIMITER: "/",
    DISPLAYITEM_PREFIX: "displayitem_",
    REPEATITEM_PREFIX: "repeatItem_",
    DISPLAYITEMGROUP_PREFIX: "displayitemGroup_",
    DISPLAYITEM_CONTAINER_SUFFIX: "_displayitemContainer",
    TABCONTAINER_PREFIX: "tabContainer_",
    REPEATITEM_CONTAINER_PREFIX: "repeatItemContainer_",
    WIZARD_CONTAINER_ID: "jfcWizardContainer",
    WIZARD_All_ITEMS: [],
    parentContainer: null,

    constructor: function(args) {
      declare.safeMixin(this, args);
      this.displayitemContainer =
        this.createDndSource(args.mainContainer, ["detailItem", "fieldset", "titlePane", "tab", "repeatContainer", "wizardContainer"]);
    },

    getContainerByNode: function(node) {
      if (this.isNodeTabContainer(node) || this.isNodeRepeatItemContainer(node)) {
        var container = this.containers[node.id];
        if (!container) {
          container = this.containers[this.tabContainerIdByDisplayitemGroupId[node.id]];
        }
        return container;
      }
      var displayitemGroup = this.getDisplayitemGroupByNode(node);
      if (!displayitemGroup) {
        return null;
      }
      return this.containers[displayitemGroup.id];
    },

    getParentLayoutName: function(label) {
      if (!label || label.indexOf(this.LAYOUTNAME_DELIMITER) === -1) {
        return "";
      }
      return label.substring(0, label.lastIndexOf(this.LAYOUTNAME_DELIMITER));
    },

    createContainer: function(displayitemGroup) {
      var containerType = this.getContainerType(displayitemGroup);
      if (containerType === this.LAYOUT_FIELDSET) {
        return this.createFieldsetContainer(displayitemGroup);
      } else if (containerType === this.LAYOUT_TITLEPANE) {
        return this.createTitlePaneContainer(displayitemGroup);
      } else if (containerType === this.LAYOUT_TAB) {
        return this.createTabContainer(displayitemGroup);
      } else if (containerType === "group") {
      }
    },

    getParentContainer: function(displayitemGroup) {
      var parentDisplayitemGroup =
        this.getDisplayitemGroupByLayoutName(displayitemGroup.getParentLayoutName());
      var parentContainer;
      if (parentDisplayitemGroup) {
        parentContainer = this.getContainerByDisplayitemGroup(parentDisplayitemGroup);
      } else {
        parentContainer = this;
      }
      return parentContainer;
    },

    createFieldsetContainer: function(displayitemGroup) {
      var container = new FieldsetContainer({
        id: displayitemGroup.id,
        displayitemGroup: displayitemGroup,
        parentContainer: this.getParentContainer(displayitemGroup),
        lm: this,
      });
      this.containers[displayitemGroup.id] = container;
      displayitemGroup.container = container;
      return container;
    },

    createTitlePaneContainer: function(displayitemGroup) {
      var container = new TitlePaneContainer({
        id: displayitemGroup.id,
        displayitemGroup: displayitemGroup,
        parentContainer: this.getParentContainer(displayitemGroup),
        lm: this,
      });
      this.containers[displayitemGroup.id] = container;
      displayitemGroup.container = container;
      return container;
    },

    createTabContainer: function(displayitemGroup) {
      var tabContainerId = this.getTabContainerId(displayitemGroup);
      var tabContainer = this.containers[tabContainerId];
      if (!tabContainer) {
        tabContainer = new TabContainer({
          id: tabContainerId,
          parentContainer: this.getParentContainer(displayitemGroup),
          lm: this,
        });
        this.containers[tabContainerId] = tabContainer;
      }
      displayitemGroup.container = tabContainer;
      this.tabContainerIdByDisplayitemGroupId[displayitemGroup.id] = tabContainerId;
      this.tabNameByLayoutName[displayitemGroup.getLayoutName()] = tabContainerId;
      tabContainer.addTab(displayitemGroup);
      return tabContainer;
    },

    getContainerType: function(displayitemGroup) {
      if (displayitemGroup.separate1 === this.LAYOUT_TAB) {
        return this.LAYOUT_TAB;
      } else if (displayitemGroup.useFieldset === this.LAYOUT_USE_FIELDSET || displayitemGroup.useFieldset === "") {
        return this.LAYOUT_FIELDSET;
      } else if (displayitemGroup.useFieldset === this.LAYOUT_TITLEPANE) {
        return this.LAYOUT_TITLEPANE;
      } else if (displayitemGroup.useFieldset === this.LAYOUT_GROUP_VERTICAL) {
        return this.LAYOUT_FIELDSET; // "group";
      }
      return this.LAYOUT_FIELDSET;
    },

    getContainerByLayoutName: function(layoutName) {
      var displayitemGroup = this.getDisplayitemGroupByLayoutName(layoutName);
      var container;
      if (displayitemGroup) {
        container = this.containers[displayitemGroup.id];
      }
      if (!container) {
        container = this.getTabContainerByLayoutName(layoutName);
      }
      return container;
    },

    getTabContainerByLayoutName: function(layoutName) {
      return this.containers[this.tabNameByLayoutName[layoutName]];
    },

    getContainerByDisplayitemGroup: function(displayitemGroup) {
      var container = this.containers[displayitemGroup.id];
      if (!container) {
        container = this.containers[this.tabContainerIdByDisplayitemGroupId[displayitemGroup.id]];
      }
      return container;
    },

    getRepeatItemContainerNo: function() {
      this.REPEATITEM_CONTAINER_COUNT = this.REPEATITEM_CONTAINER_COUNT + 1;
      return this.REPEATITEM_CONTAINER_COUNT;
    },

    getRepeatItemContainerId: function(displayitem) {
      return this.REPEATITEM_CONTAINER_PREFIX + this.getRepeatItemContainerName(displayitem);
    },

    getRepeatItemContainer: function(displayitem) {
      var id = this.getRepeatItemContainerId(displayitem);
      var repeatItemContainer = this.containers[id];
      if (!repeatItemContainer) {
        repeatItemContainer = this.createRepeatItemContainer(displayitem);
      }
      return repeatItemContainer;
    },

    getRepeatItemContainerName: function(displayitem) {
      var containerName = displayitem.name;
      if (containerName.indexOf(this.REPEATITEM_DELIMITER) > 0) {
        containerName = containerName.split(this.REPEATITEM_DELIMITER)[0];
      }
      return containerName;
    },

    createRepeatItemContainer: function(displayitem) {
      var no = this.getRepeatItemContainerNo();
      var id = this.getRepeatItemContainerId(displayitem);
      var containerName = this.getRepeatItemContainerName(displayitem);
      var containerDisplayitem = this.getRepeatItemContainerDisplayitem(containerName);
      var modelitem = wagbyDesigner.activeModel.getModelitemByName(containerName);
      var parentContainer = this.getContainerByLayoutName(containerDisplayitem.label);
      var container = new RepeatItemContainer({
        no: no,
        id: id,
        name: containerDisplayitem.name,
        label: containerDisplayitem.label,
        containerName: containerName,
        containerModelitem: modelitem,
        parentContainer: parentContainer,
        displayitemContainer: this.displayitemContainer,
        lm: this,
      });
      this.containers[id] = container;
      return container;
    },

    getRepeatItemContainerDisplayitem: function(containerName) {
      for (var i=0,l=this.displayitems.length; i<l; i++) {
        var displayitem = this.displayitems[i];
        if (displayitem.name === containerName) {
          return displayitem;
        }
      }
      return null;
    },

    deselectContainer: function() {
      for (var key in this.containers) {
        var container = this.containers[key];
        if (container && container.deselect) {
          container.deselect();
        }
      }
      this.displayitemContainer.selectNone();
      this.layoutPane.clearSettings();
      this.selectedContainer = null;
    },

    destroyContainers: function() {
      for (var key in this.containers) {
        var container = this.containers[key];
        if (container && container.destroy) {
          container.destroy();
        }
      }
      this.containers = {};
      this.tabNameByLayoutName = {};
    },

    removeContainer: function() {
      var displayitemContainer = this.getDisplayitemContainer();
      if (!displayitemContainer) {
        console.error("LayoutManager.js:247 removeContainer displayitemContainer is null");
        return;
      }
      var selectedContainers = this.getSelectedContainers(displayitemContainer.getAllNodes());
      for (var i=0,l=selectedContainers.length; i<l; i++) {
        var selectedContainer = selectedContainers[i];
        if (!(this.isContainerFieldset(selectedContainer) ||
              this.isContainerTitlePane(selectedContainer))) {
          continue;
        }
        var id = selectedContainer.displayitemGroup.id;
        this.removeCache(id);
        var childNodes = selectedContainer.getAllNodes();
        selectedContainer.remove();
        this.changeContainerStyle(childNodes);
        this.deselectContainer();
        this.layoutPane.showContainerInfo();
      }
    },

    init: function(model) {
      this.layoutId = 1;
      this.groupId = 0;
      this.tabId = 1;
      this.TAB_CONTAINER_COUNT = 0;
      this.REPEATITEM_CONTAINER_COUNT = 0;
      this.isContainerEventSet = {};
      this.layouts = [];
      this.oldLayoutsNo = [];
      this.WIZARD_All_ITEMS = [];
      this.setWizardModel(model);
    },

    showLayout: function(model) {
      this.model = model;
      this.init(model);
      var that = this;
      wagbyDesigner.sendRequest({
        url: "/model/" + model.getId() + "/layout",
        options: {
          handleAs: "json",
        }
      }, function(data) {
        that.setDisplayitemGroup(data);
        that.showDisplayitems(model);
        that.disableDnd(model.isLocked());
      }, function(data) {
        return wagbyDesigner.jfcRes.MSG_ERROR_LAYOUT_001; // "レイアウト情報の取得に失敗しました。"
      });
    },

    setWizardModel: function(model) {
      this.isWizardModel = false;
      for (var i=0,l=model.modelitems.length; i<l; i++) {
        var modelitem = model.modelitems[i];
        if (modelitem.getWizard() !== "") {
          this.isWizardModel = true;
          break;
        }
      }
    },

    setDisplayitemGroup: function(layouts) {
      this.displayitemGroups = {};
      if (!layouts || !layouts.length || layouts.length === 0) {
        return;
      }
      var that = this;
      for (var i=0,l=layouts.length; i<l; i++) {
        var layout = layouts[i];
        var layoutName = layout["model/displayitemgroup/@label"];
        if (!layoutName || layoutName === "") {
          continue;
        }
        this.oldLayoutsNo.push(i + 1);
        var displayitemGroup = this.createDisplayitemGroup(layout);
        var containerType = this.getContainerType(displayitemGroup);
        if (containerType === this.LAYOUT_FIELDSET || containerType === this.LAYOUT_TITLEPANE) {
          this.getGroupId();
        }
        var container = this.createContainer(displayitemGroup);
      }
    },

    getContainerCreateAnchorNode: function() {
      var anchorNode;
      if (this.selectedContainer) {
        anchorNode = this.selectedContainer.itemContainer;
      } else if (this.selectedDisplayitem) {
        anchorNode = this.selectedDisplayitem.itemContainer;
      } else {
        if (this.isWizardModel) {
          var wizardContainer = this.getWizardContainer();
          anchorNode = wizardContainer.getDisplayitemContainerNodeDisplayedScreen();
        } else {
          anchorNode = this.mainContainer;
        }
      }
      return anchorNode;
    },

    getContainerCreatePosition: function(anchorNode) {
      var position = "before";
      var wizardContainer = this.getWizardContainer();
      if (anchorNode === this.mainContainer ||
          (wizardContainer && anchorNode === wizardContainer.getDisplayitemContainerNodeDisplayedScreen())) {
        position = "last";
      }
      return position;
    },

    addDisplayitemGroup: function(layout) {
      var label = layout["model/displayitemgroup/@label"];
      layout["model/displayitemgroup/@label"] = label + this.getGroupId();
      var displayitemGroup = this.createDisplayitemGroup(layout);
      var anchorNode = this.getContainerCreateAnchorNode();
      var position = this.getContainerCreatePosition(anchorNode);
      var container = this.createContainer(displayitemGroup);
      this.createDisplayitemContainerDnd(container, anchorNode, position);
      this.setContainerEvent(container);
    },

    createDisplayitemGroup: function(layout) {
      var layoutId = this.getLayoutId();
      var displayitemGroup = new DisplayitemGroup({
        no: layoutId,
        id: this.DISPLAYITEMGROUP_PREFIX + layoutId,
        layoutName: layout["model/displayitemgroup/@label"],
        useFieldset: layout["model/displayitemgroup/@useFieldSet"] || "",
        separate1: layout["model/displayitemgroup/@separate1"],
        separate2: layout["model/displayitemgroup/@separate2"],
        labelWidth: layout["model/displayitemgroup/@labelwidth"],
        tabNumber: layout["model/displayitemgroup/@tabnumber"],
        defaultTab: layout["model/displayitemgroup/@defaulttab"],
        selectTabCondition: layout["model/displayitemgroup/@selectTabCondition"] || "",
        tabVisible: layout["model/displayitemgroup/@tabvisible"] || "",
        folded: layout["model/displayitemgroup/@folded"] || "",
        displayOrder: 1,
        container: null,
        parent: this.displayitemContainer,
      });
      this.displayitemGroups[displayitemGroup.id] = displayitemGroup;
      return displayitemGroup;
    },

    addTabContainer: function(layout) {
      var tabNumber = this.TAB_CONTAINER_COUNT + 1;
      var label = layout["model/displayitemgroup/@label"];
      layout["model/displayitemgroup/@label"] = label + tabNumber + "1";
      layout["model/displayitemgroup/@tabnumber"] = tabNumber;
      var displayitemGroup = this.createDisplayitemGroup(layout);
      var container = this.createContainer(displayitemGroup);
      var anchorNode = this.getContainerCreateAnchorNode();
      var position = this.getContainerCreatePosition(anchorNode);
      this.createDisplayitemContainerDnd(container, anchorNode, position);
      this.setContainerEvent(container);
      container.resize();
      this.TAB_CONTAINER_COUNT = tabNumber;
    },

    removeTabContainer: function() {
      var tabContainer = this.selectedContainer;
      if (!this.isContainerTab(tabContainer)) {
        return;
      }
      var tabContainerId = tabContainer.id;
      this.removeCache(tabContainerId);
      this.removeTabContainerIdFromTabNameByLayoutName(tabContainerId);
      tabContainer.destroy();
      this.deselectContainer();
      this.layoutPane.showContainerInfo();
      if (this.TAB_CONTAINER_COUNT > 0) {
        this.TAB_CONTAINER_COUNT = this.TAB_CONTAINER_COUNT - 1;
      }
    },

    removeTabContainerIdFromTabNameByLayoutName: function(tabContainerId) {
      for (var key in this.tabNameByLayoutName) {
        if (this.tabNameByLayoutName[key] === tabContainerId) {
          delete this.tabNameByLayoutName[key];
        }
      }
    },

    setTabLabel: function(value) {
      var tabContainer = this.selectedContainer;
      if (!this.isContainerTab(tabContainer)) {
        return;
      }
      this.updateTabNameByLayoutNameCache(tabContainer, value);
      tabContainer.setTabLabel(value);
    },

    updateTabNameByLayoutNameCache: function(tabContainer, value) {
      this.tabNameByLayoutName[value] = tabContainer.id;
      var oldLabel = tabContainer.getSelectedTabLabel();
      delete this.tabNameByLayoutName[oldLabel];
    },

    removeCache: function(id) {
      delete this.displayitemGroups[id];
      delete this.containers[id];
      delete this.isContainerEventSet[id];
    },

    getTabsCount: function() {
      return Object.keys(this.tabNameByLayoutName).length + 1;
    },

    addTab: function(layout) {
      var tabContainer = this.selectedContainer;
      if (!this.isContainerTab(tabContainer)) {
        return;
      }
      var label = this.getNewTabLabel(tabContainer, layout);
      layout["model/displayitemgroup/@label"] = label;
      layout["model/displayitemgroup/@tabnumber"] = tabContainer.getTabNumber();
      var displayitemGroup = this.createDisplayitemGroup(layout);
      tabContainer.addTab(displayitemGroup);
      return label;
    },

    getNewTabLabel: function(tabContainer, layout) {
      return layout["model/displayitemgroup/@label"] +
        layout["model/displayitemgroup/@tabnumber"] + (tabContainer.getTabsCount() + 1);
    },

    removeTab: function() {
      var tabContainer = this.selectedContainer;
      if (!this.isContainerTab(tabContainer)) {
        return;
      }
      tabContainer.removeTab();
    },

    showDisplayitems: function(model) {
      this.createDisplayitems(model);
      for (var i=0,l=this.displayitems.length; i<l; i++) {
        this.showDisplayitem(this.displayitems[i]);
      }
      this.resizeTabContainer();
      this.createRepeatItemRowspan();
      this.createAllWizardItems();
    },

    createAllWizardItems: function() {
      if (!this.isWizardModel) {
        return;
      }
      var wizardContainer = this.containers[this.WIZARD_CONTAINER_ID];
      wizardContainer.setAllWizardItems();
      wizardContainer.resize();
    },

    resizeTabContainer: function() {
      for (var key in this.containers) {
        var container = this.containers[key];
        if (container instanceof TabContainer) {
          container.resize();
        }
      }
    },

    createRepeatItemRowspan: function() {
      for (var key in this.containers) {
        var container = this.containers[key];
        if (container instanceof RepeatItemContainer) {
          container.createDummyDisplayitemInRow();
        }
      }
    },

    createDisplayitems: function(model) {
      var that = this;
      this.displayitems = [];
      for (var i=0,l=model.modelitems.length; i<l; i++) {
        var displayitem = this.createDisplayitem(model.modelitems[i]);
        displayitem.setShowDisplayitemInfoFunc(this.getShowDisplayitemInfoFunc());
        this.displayitems.push(displayitem);
      }
      this.sortDisplayitems(this.displayitems);
      on(this.mainContainer.parentNode.parentNode, "click", function() {
        that.clearDisplayitemInfo();
      });
    },

    getShowDisplayitemInfoFunc: function() {
      var that = this;
      return function(ditem) {
        that.selectedDisplayitem = ditem;
        var name = "";
        var id = "";
        if (ditem && ditem.modelitem) {
          name = ditem.modelitem.getLabel();
          id = ditem.modelitem.getName();
        }
        domAttr.set("jfcSelectedDetailItemName", "innerHTML", name);
        domAttr.set("jfcSelectedDetailItemId", "innerHTML", id);
      }
    },

    sortDisplayitems: function(displayitems) {
      if (!displayitems) {
        return;
      }
      displayitems.sort(function(a, b) {
        if (a.displayOrder === "" && b.displayOrder === "") {
          // 表示順設定がない場合は項目Noで並べ替えを行なう
          return a.no - b.no;
        }
        if (a.displayOrder !== "" && b.displayOrder === "") {
          return -1;
        }
        if (a.displayOrder === "" && b.displayOrder !== "") {
          return 1;
        }
        try {
          var aOrder = parseInt(a.displayOrder, 10);
          var bOrder = parseInt(b.displayOrder, 10);
          return aOrder - bOrder;
        } catch(e) {
        }
        return -1;
      });
    },

    createDisplayitem: function(modelitem) {
      var isWizardItem = false;
      var wizard = modelitem.getWizard();
      if (wizard !== "") {
        isWizardItem = true;
      }
      var displayitem = new DetailDisplayitem({
        id: this.DISPLAYITEM_PREFIX + modelitem.getName(),
        no: modelitem.getNo(),
        name: modelitem.getName(),
        label: modelitem.getLabel(),
        layoutName: modelitem.getLayoutName(),
        dndType: "detailItem",
        hidden: modelitem.getTypeForShow(),
        isRepeatItem: this.isRepeatItem(modelitem),
        isWizardItem: isWizardItem,
        displayOrder: modelitem.getDetailDisplayOrder() || "",
        modelitem: modelitem,
      });
      if (wizard === "all") {
        this.WIZARD_All_ITEMS.push(displayitem);
      }
      return displayitem;
    },

    clearDisplayitemInfo: function() {
      this.selectedDisplayitem = null;
      this.showDisplayitemInfo(null);
      this.deselectContainer();
    },

    showDisplayitemInfo: function(displayitem) {
      var name = "";
      var id = "";
      if (displayitem && displayitem.modelitem) {
        name = displayitem.modelitem.getLabel();
        id = displayitem.modelitem.getName();
      }
      domAttr.set("jfcSelectedDetailItemName", "innerHTML", name);
      domAttr.set("jfcSelectedDetailItemId", "innerHTML", id);
    },

    showDisplayitem: function(displayitem) {
      if (displayitem.hidden) {
        this.setHiddenDisplayitem(displayitem);
      } else if (displayitem.isRepeatItem) {
        this.createDisplayitemInRepeatContainer(displayitem);
      } else if (displayitem.layoutName && displayitem.layoutName !== "") {
        this.createDisplayitemInContainer(displayitem);
      } else if (displayitem.isWizardItem) {
        this.createDisplayitemInWizardContainer(displayitem);
      } else {
        displayitem.create(this.displayitemContainer.node);
      }
    },

    setHiddenDisplayitem: function(displayitem) {
      displayitem.create(this.layoutPane.hiddenDetailItemContainer.node);
    },

    createDisplayitemInRepeatContainer: function(displayitem) {
      if (displayitem.name.indexOf(this.REPEATITEM_DELIMITER) === -1) {
        return;
      }
      var repeatItemContainer = this.getRepeatItemContainer(displayitem);
      if (repeatItemContainer && !repeatItemContainer.created) {
        var containerDisplayitem = this.getRepeatItemContainerDisplayitem(this.getRepeatItemContainerName(displayitem));
        var repeatContainerAnchorNode;
        container = this.getContainerByLayoutName(containerDisplayitem.label);
        if (container) {
          if (!container.created) {
            this.createDisplayitemContainerRecursive(container, displayitem);
          }
          repeatContainerAnchorNode = container.displayitemContainer.node;
        } else {
          var displayitemContainer = this.getDisplayitemContainer(displayitem);
          repeatContainerAnchorNode = displayitemContainer.node;
        }
        this.setContainerEvent(repeatItemContainer.create(repeatContainerAnchorNode));
      }
      if (repeatItemContainer) {
        repeatItemContainer.insertDisplayitem(displayitem);
      }
    },

    createDisplayitemInWizardContainer: function(displayitem) {
      var wizardContainer = this.createWizardContainer(displayitem);
      if (wizardContainer) {
        wizardContainer.insertDisplayitem(displayitem);
      }
    },

    createWizardContainer: function(displayitem) {
      var wizardContainer = this.containers[this.WIZARD_CONTAINER_ID];
      if (!wizardContainer) {
        wizardContainer = new WizardContainer({
          id: this.WIZARD_CONTAINER_ID,
          parentContainer: this,
          lm: this,
        });
        wizardContainer.create(this.displayitemContainer.node);
        this.containers[this.WIZARD_CONTAINER_ID] = wizardContainer;
      }
      return wizardContainer;
    },

    createDisplayitemInContainer: function(displayitem) {
      var container = this.getContainerByLayoutName(displayitem.layoutName);
      if (!container) {
        return;
      }
      if (!container.created) {
        this.createDisplayitemContainerRecursive(container, displayitem);
      }
      container.insertDisplayitem(displayitem);
    },

    createDisplayitemContainerRecursive: function(container, displayitem) {
      if (container.parentContainer) {
        this.createAncestorContainer(container.parentContainer, displayitem);
      }
      this.createDisplayitemContainerDnd(container, this.getAnchorNode(container, displayitem));
    },

    getAnchorNode: function(container, displayitem) {
      var anchorNode;
      if (container.parentContainer && container.displayitemGroup && !displayitem.isWizardItem) {
        var displayitemContainer = container.parentContainer.getDisplayitemContainer(container.displayitemGroup.getParentLayoutName());
        if (displayitemContainer) {
          anchorNode = displayitemContainer.node;
        }
      }
      if (container.parentContainer === this && displayitem.isWizardItem) {
        var wizardContainer = this.getWizardContainer();
        anchorNode = (wizardContainer.getDisplayitemContainer(displayitem)).node;
      }
      return anchorNode;
    },

    createAncestorContainer: function(container, displayitem) {
      this.ancestors = [];
      this.getAncestorContainer(container, displayitem);
      var anchorNode;
      for (var i=0,l=this.ancestors.length; i<l; i++) {
        var ancestor = this.ancestors[i];
        if (ancestor.created) {
          continue;
        }
        this.createDisplayitemContainerDnd(ancestor, this.getAnchorNode(ancestor, displayitem));
      }
    },

    getAncestorContainer: function(container, displayitem) {
      if (!container.parentContainer) {
        if (displayitem.isWizardItem) {
          var wizardContainer = this.createWizardContainer(displayitem);
          if (!(this.ancestors[0] instanceof WizardContainer)) {
            this.ancestors.unshift(wizardContainer);
          }
        }
        return;
      }
      this.ancestors.unshift(container);
      this.getAncestorContainer(container.parentContainer, displayitem);
    },

    createDisplayitemContainerDnd: function(container, node, nodePosition) {
      if (container.created) {
        return;
      }
      var position = "last";
      var anchorNode = this.displayitemContainer.node;
      if (node) {
        anchorNode = node;
      }
      if (nodePosition) {
        position = nodePosition;
      }
      if (container.parentContainer && container.parentContainer.create) {
        if (!container.parentContainer.created) {
          var parentContainer = container.parentContainer.create(anchorNode);
          parentContainer.displayitemContainer =
            this.createDndSource(parentContainer.displayitemContainer, container.accept);
        }
      }
      var displayitemContainer = container.create(anchorNode, position);
      if (displayitemContainer.dndType === this.LAYOUT_FIELDSET ||
          displayitemContainer.dndType === this.LAYOUT_TITLEPANE) {
        container.displayitemContainer =
          this.createDndSource(displayitemContainer.displayitemContainer, container.accept);
      }
      if (!(displayitemContainer.dndType === this.LAYOUT_FIELDSET ||
           displayitemContainer.dndType === this.LAYOUT_TAB ||
           displayitemContainer.dndType === this.LAYOUT_TITLEPANE)) {
        return;
      }
      this.setContainerEvent(container);
    },

    createDndSource: function(container, accept) {
      var dndSource = new AutoSource(container, {
        accept: accept,
        copyState: function(keyPressed, self) {
          return false;
        },
      });
      this.setDndEvent(dndSource);
      return dndSource;
    },

    setDndEvent: function(dndSource) {
      var that = this;
      var defaultBgColor = domStyle.get(dndSource.node, "backgroundColor");
      dndSource.on("DraggingOver", function() {
        domStyle.set(dndSource.node, "backgroundColor", "#FFFBD5");
        domStyle.set(this.node, "border", "1px dashed gray");
      });
      dndSource.on("DraggingOut", function() {
        domStyle.set(dndSource.node, "backgroundColor", defaultBgColor);
        domStyle.set(this.node, "border", "none");
      });
      dndSource.on("Drop", function(source, nodes, copy) {
        domStyle.set(dndSource.node, "backgroundColor", defaultBgColor);
        domStyle.set(this.node, "border", "none");
        that.postprocessDnd(source, nodes);
      });
      dndSource.on("DndCancel", function(source, nodes, copy) {
        domStyle.set(dndSource.node, "backgroundColor", defaultBgColor);
        domStyle.set(this.node, "border", "none");
      });
    },

    postprocessDnd: function(source, nodes) {
      // 隠し項目エリアから詳細画面レイアウトエリアにドロップされたか確認する
      this.checkHiddenItemResurrection(source, nodes);
      this.changeContainerStyle(nodes);
      this.resizeContainer(source);
      this.setTitlePaneLabelByRepeatContainer(nodes);
      this.setDndGroupDisplayRatio(source, nodes);
      this.deselectContainer();
    },

    checkHiddenItemResurrection: function(source, nodes) {
      // 隠し項目エリアからドロップされていない場合は何もしない
      if (source.node.id !== "jfcHiddenDetailItemContainer") {
        return;
      }
      // 隠し項目エリアから戻された項目の隠し項目設定を解除する
      var repositoryData = [];
      for (var i=0,l=nodes.length; i<l; i++) {
        var node = nodes[i];
        var id = this.getModelitemNameFromNode(node);
        repositoryData.push({
          modelitemName: id,
          repositoryKey: "presentation/displayitem/@typeForShow=hidden",
          repositoryValue: "",
        });
      }
      this.sendBulkUpdateData(repositoryData, function() {
        jfcDetailLayoutStandby.hide();
      });
    },

    resizeContainer: function(source) {
      if (!source.anchor || !source.anchor.id) {
        return;
      }
      var id = source.anchor.id;
      if (id.lastIndexOf(this.TABCONTAINER_PREFIX, 0) !== 0) {
        return;
      }
      var container = this.containers[id];
      if (container) {
        container.resize();
      }
    },

    setTitlePaneLabelByRepeatContainer: function(nodes) {
      for (var i=0,l=nodes.length; i<l; i++) {
        var node = nodes[i];
        if (!this.isNodeRepeatItemContainer(node)) {
          continue;
        }
        var repeatItemContainer = this.getContainerByNode(node);
        if (!repeatItemContainer) {
          continue;
        }
        var container = this.getParentContainerRecursive(node);
        if (!container || !this.isContainerTitlePane(container)) {
          continue;
        }
        container.setLabel(repeatItemContainer.label);
      }
    },

    setDndGroupDisplayRatio: function(source, nodes) {
      // 横並びのグループから取り除いた場合、表示比率を再セットする
      this.setGroupNodeDisplayRatio(source);
      // 横並びのグループにドロップした場合、表示比率をセットする
      this.setDroppedNodesDisplayRatio(nodes);
    },

    setGroupNodeDisplayRatio: function(source) {
      if (!source) {
        return;
      }
      this.setNodeDisplayRatio(source.node);
    },

    setDroppedNodesDisplayRatio: function(nodes) {
      if (!nodes || nodes.length === 0) {
        return;
      }
      this.setNodeDisplayRatio(nodes[0].parentNode);
    },

    setNodeDisplayRatio: function(node) {
      var container = this.getContainerByNode(node);
      if (container && (container instanceof FieldsetContainer)) {
        container.setDroppedNodesDisplayRatio();
      }
    },

    isNodeDisplayitem: function(node) {
      if (!node || !node.id) {
        return false;
      }
      if (node.id.lastIndexOf(this.DISPLAYITEM_PREFIX, 0) === 0) {
        return true;
      }
      return false;
    },

    isNodeDisplayitemGroup: function(node) {
      if (!node || !node.id) {
        return false;
      }
      if (node.id.lastIndexOf(this.DISPLAYITEMGROUP_PREFIX, 0) === 0 ||
          node.id.lastIndexOf(this.TABCONTAINER_PREFIX, 0) === 0) {
        return true;
      }
      return false;
    },

    isNodeTabContainer: function(node) {
      if (!node || !node.id) {
        return false;
      }
      if (node.id.lastIndexOf(this.TABCONTAINER_PREFIX, 0) === 0) {
        return true;
      }
      if (this.tabContainerIdByDisplayitemGroupId[node.id]) {
        return true;
      }
      return false;
    },

    isNodeRepeatItemContainer: function(node) {
      if (!node || !node.id) {
        return false;
      }
      if (node.id.lastIndexOf(this.REPEATITEM_CONTAINER_PREFIX, 0) === 0) {
        return true;
      }
      return false;
    },

    replaceParentContainer: function(node) {
      var container = this.getContainerByNode(node);
      if (!container) {
        return;
      }
      container.parentContainer = this.getParentContainerRecursive(node);
    },

    getParentContainerRecursive: function(node) {
      var parentNode = node.parentNode;
      if (!parentNode || parentNode.id === "jfcDetailItemContainer") {
        return this;
      }
      if (!this.isNodeDisplayitemGroup(parentNode)) {
        return this.getParentContainerRecursive(parentNode);
      }
      return this.getContainerByNode(parentNode);
    },

    getParentDisplayitemGroupRecursive: function(node) {
      var parentNode = node.parentNode;
      if (parentNode.id === "jfcDetailItemContainer") {
        return null;
      }
      if (this.isNodeDisplayitemGroup(parentNode)) {
        return this.getParentDisplayitemGroupRecursive(parentNode);
      }
      return this.getDisplayitemGroupByNode(parentNode);
    },

    changeContainerStyle: function(nodes) {
      for (var i=0,l=nodes.length; i<l; i++) {
        var node = nodes[i];
        var parentDisplayitemGroup = this.getParentDisplayitemGroupRecursive(node);
        if (!parentDisplayitemGroup) {
          this.resetContainerStyle(node);
        } else {
          this.setContainerStyle(parentDisplayitemGroup, node);
        }
      }
    },

    resetContainerStyle: function(node) {
      var container = this.getContainerByNode(node);
      if (!container || !container.itemContainer) {
        domAttr.remove(node, "style");
        return;
      }
      domAttr.remove(container.itemContainer, "style");
      domStyle.set(container.itemContainer, {
        paddingBottom: "5px",
      });
    },

    setContainerStyle: function(parentDisplayitemGroup, node) {
      var container = this.containers[parentDisplayitemGroup.id];
      if (!container) {
        return;
      }
      if (!this.isContainerFieldset(container)) {
        return;
      }
      container.setGroupDirection(parentDisplayitemGroup.separate1);
    },

    getDisplayitemGroupByNode: function(node) {
      var id = node.id;
      var displayitemGroupId = id;
      if (id.indexOf(this.DISPLAYITEM_CONTAINER_SUFFIX) !== -1) {
        displayitemGroupId = id.split(this.DISPLAYITEM_CONTAINER_SUFFIX)[0];
      } else if (id.indexOf(this.TABCONTAINER_PREFIX) !== -1) {
        for (var key in this.tabContainerIdByDisplayitemGroupId) {
          if (this.tabContainerIdByDisplayitemGroupId[key] === id) {
            displayitemGroupId = key;
          }
        }
      }
      return this.displayitemGroups[displayitemGroupId];
    },

    isRepeatItem: function(modelitem) {
      if (modelitem.isItemTypeContainer() || modelitem.isContainerItem()) {
        return true;
      }
      return false;
    },

    setContainerEvent: function(container) {
      this.setContainerClickEvent(container);
      if (!container.parentContainer || !container.parentContainer.itemContainer) {
        return;
      }
      this.setContainerClickEvent(container.parentContainer);
    },

    setContainerClickEvent: function(container) {
      if (!container || !container.itemContainer || this.isContainerEventSet[container.id] || this.model.isLocked()) {
        return;
      }
      var that = this;
      on(container.itemContainer, "click", function(e) {
        event.stop(e);
        var selectedContainer = null;
        if (that.isSelectedContainer(container)) {
          selectedContainer = container;
          that.deselectDescendentContainer(container);
        }
        that.layoutPane.showContainerInfo(container);
        that.selectedContainer = container;
      });
      this.isContainerEventSet[container.id] = true;
    },

    isSelectedContainer: function(container) {
      var selection = this.getContainerSelection(container);
      if (selection[container.id] === 1) {
        return true;
      }
      return false;
    },

    getContainerSelection: function(container) {
      if (container.parentContainer && container.parentContainer.displayitemContainer) {
        return container.parentContainer.displayitemContainer.selection;
      }
      return this.displayitemContainer.selection;
    },

    deselectDescendentContainer: function(container) {
      if (!(container instanceof FieldsetContainer)) {
        return;
      }
      var childNodes = container.getAllNodes();
      for (var i=0,l=childNodes.length; i<l; i++) {
        var descendentContainer = this.getContainerByNode(childNodes[i]);
        if (!descendentContainer || !(descendentContainer instanceof FieldsetContainer)) {
          continue;
        }
        descendentContainer.deselect();
        this.deselectDescendentContainer(descendentContainer);
      }
    },

    getLayoutId: function() {
      return Object.keys(this.displayitemGroups).length + 1;
    },

    getGroupId: function() {
      this.groupId = this.groupId + 1;
      return this.groupId;
    },

    getTabNumber: function(displayitemGroup) {
      var tabNumber = parseInt(displayitemGroup.tabNumber, 10);
      if (isNaN(tabNumber)) {
        this.TAB_CONTAINER_COUNT = this.TAB_CONTAINER_COUNT + 1;
        return this.TAB_CONTAINER_COUNT;
      } else {
        if (tabNumber > this.TAB_CONTAINER_COUNT) {
          this.TAB_CONTAINER_COUNT = tabNumber;
        }
      }
      return tabNumber;
    },

    getTabContainerId: function(displayitemGroup) {
      return this.TABCONTAINER_PREFIX + this.getTabNumber(displayitemGroup);
    },

    getDisplayitemContainer: function(displayitem) {
      if (this.isWizardModel) {
        var wizardContainer = this.getWizardContainer();
        return wizardContainer.getDisplayitemContainer(displayitem);
      } else {
        return this.displayitemContainer;
      }
    },

    saveLayoutData: function(callback) {
      this.createLayoutInfo();
      this.saveLayoutNodes(callback);
    },

    createLayoutInfo: function() {
      this.displayitemGroupsToSave = [];
      this.displayitemGroupNodes = {};
      var noLayoutNodes = [];
      this.displayOrder = 1;
      var nodes;
      if (this.isWizardModel) {
        var wizardContainer = this.getWizardContainer();
        nodes = wizardContainer.getAllNodes();
      } else {
        nodes = this.displayitemContainer.getAllNodes();
      }
      for (var i=0,l=nodes.length; i<l; i++) {
        var node = nodes[i];
        // ウィザードの all 項目のダミーノードは保存対象としない
        if (node.id.indexOf("_wizardall_dummy_") !== -1) {
          continue;
        }
        if (this.isNodeDisplayitemGroup(node) ||
            this.isNodeRepeatItemContainer(node)) {
          this.setLayoutNameRecursive(node);
        } else {
          this.setNodeDisplayOrder(node);
          noLayoutNodes.push(node);
        }
      }
      this.displayitemGroupNodes[""] = noLayoutNodes;
    },

    setNodeDisplayOrder: function(node) {
      domAttr.set(node, "data-displayOrder", this.displayOrder);
      this.displayOrder = this.displayOrder + 1;
    },

    setLayoutNameRecursive: function(node, parentLayoutName) {
      var container = this.getContainerByNode(node);
      if (!container) {
        return;
      }
      if (container instanceof TabContainer) {
        this.setTabContainerSaveData(container, parentLayoutName);
      } else if (container instanceof RepeatItemContainer) {
        this.setRepeatItemSaveData(container, parentLayoutName);
      } else {
        this.setFieldsetSaveData(container, parentLayoutName);
      }
    },

    setTabContainerSaveData: function(container, parentLayoutName) {
      for (var i=0,l=container.displayitemGroups.length; i<l; i++) {
        var displayitemGroup = container.displayitemGroups[i];
        this.displayitemGroupsToSave.push(displayitemGroup);
        var nodes;
        if (displayitemGroup) {
          nodes = this.displayitemGroupNodes[displayitemGroup.id];
        }
        if (!nodes) {
          nodes = [];
        }
        if (parentLayoutName && parentLayoutName !== "") {
          displayitemGroup.setLayoutName(parentLayoutName + this.LAYOUTNAME_DELIMITER + displayitemGroup.getMyLayoutName());
        }
        var childNodes = container.getAllNodesInDisplayitemGroup(displayitemGroup);
        for (var j=0,m=childNodes.length; j<m; j++) {
          var childNode = childNodes[j];
          if (this.isNodeDisplayitemGroup(childNode) ||
              this.isNodeRepeatItemContainer(childNode)) {
            this.setLayoutNameRecursive(childNode, displayitemGroup.getLayoutName());
          } else {
            this.setNodeDisplayOrder(childNode);
            nodes.push(childNode);
          }
        }
        this.displayitemGroupNodes[displayitemGroup.id] = nodes;
      }
    },

    setRepeatItemSaveData: function(container, parentLayoutName) {
      var repeatItems = container.getAllNodes();
      for (var i=0,l=repeatItems.length; i<l; i++) {
        var repeatItem = repeatItems[i];
        this.setNodeDisplayOrder(repeatItem);
      }
      this.displayitemGroupNodes[container.id] = repeatItems;
    },

    setFieldsetSaveData: function(container, parentLayoutName) {
      var displayitemGroup = this.displayitemGroups[container.id];
      if (!displayitemGroup) {
        return;
      }
      this.displayitemGroupsToSave.push(displayitemGroup);

      // タイトルペイン内部に繰り返しコンテナがある場合は、
      // 繰り返しコンテナ項目のレイアウト名は変更しない
      if (container instanceof TitlePaneContainer) {
        if (container.containsRepeatItemContainer()) {
          return;
        }
      }
      var nodes;
      if (displayitemGroup) {
        nodes = this.displayitemGroupNodes[displayitemGroup.id];
      }
      if (!nodes) {
        nodes = [];
      }
      if (parentLayoutName && parentLayoutName !== "") {
        displayitemGroup.setLayoutName(parentLayoutName + this.LAYOUTNAME_DELIMITER + displayitemGroup.getMyLayoutName());
      }
      var parentLayoutName = displayitemGroup.getLayoutName();
      var childNodes = this.getAllNodes(container);
      for (var j=0,m=childNodes.length; j<m; j++) {
        var childNode = childNodes[j];
        if (this.isNodeDisplayitemGroup(childNode) ||
            this.isNodeRepeatItemContainer(childNode)) {
          this.setLayoutNameRecursive(childNode, parentLayoutName);
        } else {
          this.setNodeDisplayOrder(childNode);
          nodes.push(childNode);
        }
      }
      this.displayitemGroupNodes[displayitemGroup.id] = nodes;
    },

    saveLayoutNodes: function(callback) {
      var repositoryData = [];
      for (var id in this.displayitemGroupNodes) {
        var layoutName = "";
        var displayitemGroup = this.displayitemGroups[id];
        if (displayitemGroup) {
          layoutName = displayitemGroup.getLayoutName();
        }
        var nodes = this.displayitemGroupNodes[id];
        for (var i=0,l=nodes.length; i<l; i++) {
          var node = nodes[i];
          var modelitemId;
          if (node.id.lastIndexOf(this.DISPLAYITEM_PREFIX, 0) === 0) {
            modelitemId = node.id.replace(this.DISPLAYITEM_PREFIX, "");
          } else if (node.id.lastIndexOf(this.REPEATITEM_PREFIX, 0) === 0) {
            modelitemId = node.id.replace(this.REPEATITEM_PREFIX, "");
            layoutName = domAttr.get(node, "data-layoutName") || "";
            this.saveRepeatItemContainerRows(node, modelitemId, repositoryData);
            this.saveRepeatItemRowColSpan(node, modelitemId, repositoryData);
          } else {
            continue;
          }
          if (!modelitemId) {
            continue;
          }
          // レイアウト名
          repositoryData.push({
            modelitemName: modelitemId,
            repositoryKey: "model/modelitem/@layoutName",
            repositoryValue: layoutName,
          });
          var displayOrder = domAttr.get(node, "data-displayOrder");
          if (displayOrder) {
            repositoryData.push({
              modelitemName: modelitemId,
              repositoryKey: "model/modelitem/@detailDisplayOrder",
              repositoryValue: displayOrder,
            });
          }
        }
      }
      // 隠し項目
      repositoryData = repositoryData.concat(this.getHiddenDisplayitemRepository());
      if (repositoryData.length === 0) {
        callback();
        return;
      }
      var that = this;
      this.sendBulkUpdateData(repositoryData, function() {
        that.deleteGroupTabData(callback);
      });
    },

    saveRepeatItemContainerRows: function(node, modelitemId, repositoryData) {
      var rows = domAttr.get(node, "data-repeatItemContainerRows");
      if (!rows) {
        return;
      }
      repositoryData.push({
        modelitemName: modelitemId,
        repositoryKey: "model/modelitem/container/@rows",
        repositoryValue: rows,
      });
    },

    saveRepeatItemRowColSpan: function(node, modelitemId, repositoryData) {
      // セル結合(行)
      var rowspan = domAttr.get(node, "data-repeatItemRowspan");
      if (rowspan && rowspan !== "") {
        repositoryData.push({
          modelitemName: modelitemId,
          repositoryKey: "model/modelitem/@containerRowspan",
          repositoryValue: rowspan + "",
        });
      }
      // セル結合(列)
      var colspan = domAttr.get(node, "data-repeatItemColspan");
      if (colspan && colspan !== "") {
        repositoryData.push({
          modelitemName: modelitemId,
          repositoryKey: "model/modelitem/@containerColspan",
          repositoryValue: colspan + "",
        });
      }
    },

    isGroupTabDataDeleted: function() {
      for (var key in this.deletedLayout) {
        if (!this.deletedLayout[key]) {
          return false;
        }
      }
      return true;
    },

    deleteGroupTabData: function(callback) {
      if (this.oldLayoutsNo.length === 0) {
        // グループ・タブ情報を送信
        this.saveGroupTabData(callback);
      } else {
        this.deletedLayout = {};
        for (var i=0,l=this.oldLayoutsNo.length; i<l; i++) {
          this.deletedLayout[this.oldLayoutsNo[i]] = false;
        }
        for (var i=0,l=this.oldLayoutsNo.length; i<l; i++) {
          this.sendDeleteLayout(this.oldLayoutsNo[i], callback);
        }
      }
    },

    sendDeleteLayout: function(no, callback) {
      var that = this;
      var url = "/model/" + this.model.getId() + "/layout/1";
      wagbyDesigner.sendRequest({
        url: url,
        options: {
          handleAs: "json",
          method: "DELETE",
        },
      }, function(data) {
        that.deletedLayout[no] = true;
        if (that.isGroupTabDataDeleted()) {
          // グループ・タブ情報を送信
          that.saveGroupTabData(callback);
        }
      }, function(data) {
        return wagbyDesigner.jfcRes.MSG_ERROR_LAYOUT_004; // "レイアウト情報の削除に失敗しました。"
      });
    },

    getAllNodes: function(container, layoutName) {
      if (!container) {
        return [];
      }
      if (container instanceof TabContainer) {
        return container.getAllNodesInTab(layoutName);
      }
      if (!container.getDisplayitemContainer) {
        return [];
      }
      var displayitemContainer = container.getDisplayitemContainer();
      if (!displayitemContainer || !displayitemContainer.getAllNodes) {
        return [];
      }
      return displayitemContainer.getAllNodes();
    },

    getGroupTabDisplayOrder: function(container, containerLayoutName) {
      if (!container || !container.getAllNodes) {
        return;
      }
      var nodes = container.getAllNodes();
      if (nodes.length === 0) {
        return;
      }
      for (var i=0,l=nodes.length; i<l; i++) {
        var node = nodes[i];
        if (!this.isNodeDisplayitemGroup(node)) {
          continue;
        }
        if (this.isNodeTabContainer(node)) {
          this.getGroupTabDisplayOrder(this.containers[node.id]);
        } else {
          var displayitemGroup = this.displayitemGroups[node.id];
          if (!displayitemGroup) {
            continue;
          }
          this.groupTabDisplayOrder[node.id] = Object.keys(this.groupTabDisplayOrder).length + 1;
          var childContainers = this.getChildContainersByLayoutName(this.getContainerByDisplayitemGroup(displayitemGroup));
          for (var j=0,m=childContainers.length; j<m; j++) {
            var childContainer = childContainers[j];
            this.groupTabDisplayOrder[childContainer.id] = Object.keys(this.groupTabDisplayOrder).length + 1;
            this.getGroupTabDisplayOrder(childContainer);
          }
        }
      }
    },

    getChildContainersByLayoutName: function(parentContainer) {
      var childContainers = [];
      var nodes = parentContainer.getAllNodes();
      for (var i=0,l=nodes.length; i<l; i++) {
        var node = nodes[i];
        if (!this.isNodeDisplayitemGroup(node)) {
          continue;
        }
        var displayitemGroup = this.getDisplayitemGroupByNode(node);
        if (!displayitemGroup) {
          continue;
        }
        var childContainer = this.containers[displayitemGroup.id];
        if (childContainer) {
          childContainers.push(childContainer);
        }
      }
      return childContainers;
    },

    saveGroupTabData: function(callback) {
      if (this.displayitemGroupsToSave.length === 0) {
        if (callback) {
          callback();
        }
        return;
      }
      var that = this;
      for (var i=0,l=this.displayitemGroupsToSave.length; i<l; i++) {
        var no = i + 1;
        var displayitemGroup = this.displayitemGroupsToSave[i];
        this.createNewGroupTab(no, function() {
          that.sendGroupTabData(no, displayitemGroup, callback);
        });
      }
    },

    sortDisplayitemGroup: function(displayitemGroup) {
      displayitemGroup.sort(function(a, b) {
        return a.no - b.no;
      });
    },

    createNewGroupTab: function(no, callback) {
      var url = "/model/" + this.model.getId() + "/layout";
      var that = this;
      wagbyDesigner.sendRequest({
        url: url,
        options: {
          handleAs: "json",
          headers: {
            'x-wd-insert-into': no
          },
          method: "POST",
          sync: true
        }
      }, function(data) {
        if (callback) {
          callback();
        }
      }, function(data) {
        return wagbyDesigner.jfcRes.MSG_ERROR_LAYOUT_002; // "レイアウト情報の新規作成に失敗しました。"
      });
    },

    sendGroupTabData: function(no, displayitemGroup, callback) {
      this.bulkUpdateGroupTabData(no, displayitemGroup, callback);
    },

    getGroupTabRepositoryData: function(displayitemGroup) {
      return {
        'model/displayitemgroup/@label': displayitemGroup.getLayoutName(),
        'model/displayitemgroup/@useFieldSet': displayitemGroup.useFieldset,
        'model/displayitemgroup/@separate1': displayitemGroup.separate1,
        'model/displayitemgroup/@separate2': displayitemGroup.separate2,
        'model/displayitemgroup/@labelwidth': displayitemGroup.labelWidth,
        'model/displayitemgroup/@tabnumber': displayitemGroup.tabNumber,
        'model/displayitemgroup/@defaulttab': displayitemGroup.defaultTab,
        'model/displayitemgroup/@selectTabCondition': displayitemGroup.selectTabCondition,
        'model/displayitemgroup/@tabvisible': displayitemGroup.tabVisible,
        'model/displayitemgroup/@folded': displayitemGroup.folded,
      };
    },

    bulkUpdateGroupTabData: function(no, displayitemGroup, callback) {
      var that = this;
      var content = this.getGroupTabRepositoryData(displayitemGroup);
      var url = wagbyDesigner.BASEURL + "/model/" + this.model.getId() + "/layout/" + no;
      wagbyDesigner.sendRequest({
        url: url,
        options: {
          data: content,
          handleAs: "json",
          method: "PUT"
        }
      }, function(data) {
console.log("send bulkUpdateGroupTabData succsess: ", data);
        if (no === that.displayitemGroupsToSave.length) {
          if (callback) {
            callback();
          }
        }
      }, function(data) {
console.error("send bulkUpdate error: ", data);
        jfcBtnDetailLayoutSave.set("disabled", false);
        jfcDetailLayoutStandby.hide();
        wagbyDesigner.bubbleErrorMessage(wagbyDesigner.jfcRes.MSG_ERROR_LAYOUT_005);
      });
    },

    getModelitemNameFromNode: function(node) {
      if (!node) {
        return "";
      }
      return node.id.replace(this.DISPLAYITEM_PREFIX, "");
    },

    sendBulkUpdateData: function(data, callback) {
      var that = this;
      var content = JSON.stringify(data);
      var url = wagbyDesigner.BASEURL + "/model/" + this.model.getId() + "/modelitems";
console.log("url: ", url);
console.log("content: ", content);
      jfcDetailLayoutStandby.show();
      wagbyDesigner.sendRequest({
        url: url,
        options: {
          data: content,
          handleAs: "json",
          headers: {
            'Content-Type': "application/json; charset=utf-8"
          },
          method: "PUT"
        }
      }, function(data) {
console.log("send bulkUpdate succsess: ", data);
        that.updateModelitems(data.modifymodelitem);
        if (callback) {
          callback();
        }
      }, function(data) {
console.error("send bulkUpdate error: ", data);
        jfcBtnDetailLayoutSave.set("disabled", false);
        jfcDetailLayoutStandby.hide();
        wagbyDesigner.bubbleErrorMessage(wagbyDesigner.jfcRes.MSG_ERROR_LAYOUT_005);
      });
    },

    updateModelitems: function(modifymodelitem) {
      // サーバから受信した変更されたモデル項目値を WagbyDesigner の
      // リポジトリオブジェクトと変更確認用の oldValue へ反映させる
      for (var name in modifymodelitem) {
        var modelitem = wagbyDesigner.activeModel.getModelitemByName(name);
        var modifiedValue = modifymodelitem[name];
        if (modelitem) {
          for (var repositoryKey in modifiedValue) {
            var repositoryItem = modelitem.detail.repositoryItems[repositoryKey];
            if (!repositoryItem) {
              repositoryItem = modelitem.repositoryItems[repositoryKey];
            }
            if (!repositoryItem) {
              continue;
            }
            var value = modifiedValue[repositoryKey];
            if (value === " ") {
              value = "";
            }
            repositoryItem.value = value;
          }
        }
      }
    },

    getHiddenDisplayitemRepository: function() {
      var hiddenDisplayitems = [];
      var nodes = this.layoutPane.hiddenDetailItemContainer.getAllNodes();
      for (var i=0,l=nodes.length; i<l; i++) {
        var node = nodes[i];
        var id = this.getModelitemNameFromNode(node);
        hiddenDisplayitems.push({
          modelitemName: id,
          repositoryKey: "presentation/displayitem/@typeForShow=hidden",
          repositoryValue: wagbyDesigner.jfcRes.OPTION_CHECKED,
        });
        // 隠し項目のレイアウト名、表示順をクリアする
        hiddenDisplayitems.push({
          modelitemName: id,
          repositoryKey: "model/modelitem/@layoutName",
          repositoryValue: "",
        });
        hiddenDisplayitems.push({
          modelitemName: id,
          repositoryKey: "model/modelitem/@detailDisplayOrder",
          repositoryValue: "",
        });
      }
      return hiddenDisplayitems;
    },

    clearLayout: function() {
      this.deselectContainer();
      this.clearDisplayitems();
    },

    clearDisplayitems: function() {
      this.destroyContainers();
      this.clearDisplayitemsInContainer(this.displayitemContainer);
      this.clearHiddenDisplayitems();
    },

    clearDisplayitemsInContainer: function(container) {
      if (!container) {
        return;
      }
      container.clearItems();
      var nodes = container.getAllNodes();
      for (var i=0,l=nodes.length; i<l; i++) {
        domConstruct.destroy(nodes[i]);
      }
    },

    clearHiddenDisplayitems: function() {
      this.clearDisplayitemsInContainer(this.layoutPane.hiddenDetailItemContainer);
    },

    isSelectedContainerFieldset: function() {
      if (!this.selectedContainer) {
        return false;
      }
      return this.isContainerFieldset(this.selectedContainer);
    },

    isContainerFieldset: function(container) {
      if (!container || !(container instanceof FieldsetContainer)) {
        return false;
      }
      return true;
    },

    isSelectedContainerTab: function() {
      if (!this.selectedContainer) {
        return false;
      }
      return this.isContainerTab(this.selectedContainer);
    },

    isContainerTab: function(container) {
      if (!container || !(container instanceof TabContainer)) {
        return false;
      }
      return true;
    },

    isRepeatItemContainer: function(container) {
      if (!container || !(container instanceof RepeatItemContainer)) {
        return false;
      }
      return true;
    },

    isContainerTitlePane: function(container) {
      if (!container || !(container instanceof TitlePaneContainer)) {
        return false;
      }
      return true;
    },

    setGroupUseFieldset: function(value) {
      var useFieldSet = "";
      if (value) {
        useFieldSet = this.LAYOUT_USE_FIELDSET;
      }
      var selectedContainers = this.getSelectedContainers(this.displayitemContainer.getAllNodes());
      for (var i=0,l=selectedContainers.length; i<l; i++) {
        var selectedContainer = selectedContainers[i];
        if (!this.isContainerFieldset(selectedContainer)) {
          continue;
        }
        selectedContainer.setUseFieldset(useFieldSet);
      }
    },

    setGroupFieldsetLabel: function(value) {
      if (!this.isSelectedContainerFieldset()) {
        return;
      }
      this.selectedContainer.setLabel(value);
    },

    getSelectedContainers: function(nodes) {
      var selectedContainers = [];
      var that = this;
      query(".dojoDndItemSelected, .dojoDndItemAnchor", this.detailLayoutContainer).forEach(function(node) {
        var container = that.getContainerByNode(node);
        if (container) {
          selectedContainers.push(container);
        }
      });
      return selectedContainers;
    },

    setGroupDirection: function(value) {
      var selectedContainers = this.getSelectedContainers(this.displayitemContainer.getAllNodes());
      for (var i=0,l=selectedContainers.length; i<l; i++) {
        var selectedContainer = selectedContainers[i];
        if (!this.isContainerFieldset(selectedContainer)) {
          continue;
        }
        selectedContainer.setGroupDirection(value);
      }
    },

    setGroupDisplayRatio: function(value) {
      var selectedContainers = this.getSelectedContainers(this.displayitemContainer.getAllNodes());
      for (var i=0,l=selectedContainers.length; i<l; i++) {
        var selectedContainer = selectedContainers[i];
        if (!this.isContainerFieldset(selectedContainer)) {
          continue;
        }
        selectedContainer.setDisplayRatio(value);
      }
    },

    getDisplayitemGroup: function(container) {
      return this.displayitemGroups[container.id];
    },

    getDisplayitemGroupByLayoutName: function(layoutName) {
      for (var id in this.displayitemGroups) {
        var displayitemGroup = this.displayitemGroups[id];
        if (displayitemGroup.getMyLayoutName() === layoutName ||
            displayitemGroup.getLayoutName() === layoutName) {
          return displayitemGroup;
        }
      }
      return null;
    },

    setDefaultDisplayTab: function(value) {
      var tabContainer = this.selectedContainer;
      if (!this.isContainerTab(tabContainer)) {
        return;
      }
      tabContainer.setDefaultDisplayTab(value);
    },

    setDefaultDisplayTabCondition: function(value) {
      var tabContainer = this.selectedContainer;
      if (!this.isContainerTab(tabContainer)) {
        return;
      }
      tabContainer.setDefaultDisplayTabCondition(value);
    },

    setTabDisplayCondition: function(value) {
      var tabContainer = this.selectedContainer;
      if (!this.isContainerTab(tabContainer)) {
        return;
      }
      tabContainer.setTabDisplayCondition(value);
    },

    changeRepeatItemContainerRows: function(value) {
      var repeatItemContainer = this.selectedContainer;
      if (!this.isRepeatItemContainer(repeatItemContainer)) {
        return;
      }
      repeatItemContainer.changeRepeatItemContainerRows(value);
    },

    changeRepeatItemRowspan: function(value) {
      var repeatItemContainer = this.selectedContainer;
      if (!this.isRepeatItemContainer(repeatItemContainer)) {
        return;
      }
      repeatItemContainer.changeRepeatItemRowspan(value);
    },

    changeRepeatItemColspan: function(value) {
      var repeatItemContainer = this.selectedContainer;
      if (!this.isRepeatItemContainer(repeatItemContainer)) {
        return;
      }
      repeatItemContainer.changeRepeatItemColspan(value);
    },

    showRepeatItemInfo: function(repeatItem) {
      this.layoutPane.showRepeatItemInfo(repeatItem);
    },

    getWizardContainer: function() {
      return this.containers[this.WIZARD_CONTAINER_ID];
    },

    getWizardLabels: function() {
      var wizardLabel = wagbyDesigner.activeModel.getWizardLabel();
      if (wizardLabel === "") {
        return [];
      }
      if (wizardLabel.indexOf(",") === -1) {
        return [wizardLabel];
      }
      wizardLabels = wizardLabel.split(",");
      return wizardLabels;
    },

    addTitlePaneContainer: function(layout) {
      var label = layout["model/displayitemgroup/@label"];
      if (label === wagbyDesigner.jfcRes.LABEL_LAYOUT_DETAIL_NEW_GROUP) {
        layout["model/displayitemgroup/@label"] = label + this.getGroupId();
      }
      var displayitemGroup = this.createDisplayitemGroup(layout);
      var anchorNode = this.getContainerCreateAnchorNode();
      var position = this.getContainerCreatePosition(anchorNode);
      var container = this.createContainer(displayitemGroup);
      this.createDisplayitemContainerDnd(container, anchorNode, position);
      this.setContainerEvent(container);
    },

    setTitlePaneLabel: function(value) {
      var container = this.selectedContainer;
      if (!this.isContainerTitlePane(container)) {
        return;
      }
      container.setLabel(value);
    },

    setTitlePaneFolded: function(value) {
      var container = this.selectedContainer;
      if (!this.isContainerTitlePane(container)) {
        return;
      }
      container.setFolded(value);
    },

    changeRepeatItemContainerParent: function(node) {
      var repeatItemContainer = this.getContainerByNode(node);
      if (!repeatItemContainer) {
        return;
      }
      repeatItemContainer.parentContainer = this;
    },

    disableDnd: function(disabled) {
      this.displayitemContainer.isSource = !disabled;
      for (var key in this.containers) {
        var container = this.containers[key];
        container.disableDnd(disabled);
      }
    },

  });
});
